Package: highcharter
Type: Package
Version: 0.9.5
Title: A Wrapper for the 'Highcharts' Library
Description: A wrapper for the 'Highcharts' library including
    shortcut functions to plot R objects. 'Highcharts'
    <https://www.highcharts.com/> is a charting library offering
    numerous chart types with a simple configuration syntax.
Authors@R: c(
  person("Joshua", "Kunst", role = c("aut", "cre"), email = "jbkunst@gmail.com"),
  person("Nuno", "Agostinho", role = c("ctb"), comment  = "hchart.survfit, densities and hc_add_series_scatter"),
  person("Danton", "Noriega", role = c("ctb"), comment  = "hcaes_"),
  person("Martin John ", "Hadley", role = c("ctb"), comment  = "hc_add_event_point improvement"),
  person("Eduardo", "Flores", role = c("ctb"), comment  = "First version hc_add_series_df_tidy"),
  person("Dean", "Kilfoyle", role = c("ctb"), comment  = "First version hc_add_series_boxplot"),
  person("Adline", "Dsilva", role = c("ctb"), comment  = "First version Matrix heatmap"),
  person("Kamil", "Slowikowski", role = c("ctb"), comment = c(ORCID = "0000-0002-2843-6370")),
  person("Christian", "Minich", role = c("ctb"), comment  = "hcaes mutate_mapping improvement"),
  person("Jonathan", "Armond", role = c("ctb"), comment = "mutate_mapping bugfix"),
  person("David", "Breuer", role = c("ctb"), comment = "download_map_data quiet parameter"),
  person("Mauricio", "Vargas", role = c("ctb"), comment = "tests and gh-actions"),
  person("Michael", "Yan", role = c("ctb"), comment = "Motivational example for treemap/sunburst data helper"),
  person("Bart", "Oortwijn", role = c("ctb"), comment = "rjson option, hc_add_yAxis, and GH issues collaborator"),
  person("Paul", "Campbell", role = c("ctb"), comment = "additional proxy methods")
  )
URL: https://jkunst.com/highcharter/,
        https://github.com/jbkunst/highcharter
BugReports: https://github.com/jbkunst/highcharter/issues
License: MIT + file LICENSE
RoxygenNote: 7.3.3
Encoding: UTF-8
Depends: R (>= 2.10)
Imports: htmlwidgets, magrittr, purrr, rlist, assertthat, zoo, dplyr
        (>= 1.0.0), tibble (>= 1.1), stringr (>= 1.3.0), broom, xts,
        quantmod, tidyr, htmltools, jsonlite, igraph, lubridate, yaml,
        rlang (>= 0.1.1), rjson
Suggests: knitr, rmarkdown, survival, ggplot2, httr, viridisLite,
        shiny, MASS, gapminder, forecast, geojsonio, testthat, covr,
        spelling
LazyData: true
Language: en-US
NeedsCompilation: no
Packaged: 2026-04-21 18:09:41 UTC; kupietz
Author: Joshua Kunst [aut, cre],
  Nuno Agostinho [ctb] (hchart.survfit, densities and
    hc_add_series_scatter),
  Danton Noriega [ctb] (hcaes_),
  Martin John Hadley [ctb] (hc_add_event_point improvement),
  Eduardo Flores [ctb] (First version hc_add_series_df_tidy),
  Dean Kilfoyle [ctb] (First version hc_add_series_boxplot),
  Adline Dsilva [ctb] (First version Matrix heatmap),
  Kamil Slowikowski [ctb] (ORCID:
    <https://orcid.org/0000-0002-2843-6370>),
  Christian Minich [ctb] (hcaes mutate_mapping improvement),
  Jonathan Armond [ctb] (mutate_mapping bugfix),
  David Breuer [ctb] (download_map_data quiet parameter),
  Mauricio Vargas [ctb] (tests and gh-actions),
  Michael Yan [ctb] (Motivational example for treemap/sunburst data
    helper),
  Bart Oortwijn [ctb] (rjson option, hc_add_yAxis, and GH issues
    collaborator),
  Paul Campbell [ctb] (additional proxy methods)
Maintainer: Joshua Kunst <jbkunst@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-22 09:38:59 UTC
Built: R 4.5.2; ; 2026-04-22 13:05:42 UTC; unix

Package: highcharter
Type: Package
Version: 0.9.5
Title: A Wrapper for the 'Highcharts' Library
Description: A wrapper for the 'Highcharts' library including
    shortcut functions to plot R objects. 'Highcharts'
    <https://www.highcharts.com/> is a charting library offering
    numerous chart types with a simple configuration syntax.
Authors@R: c(
  person("Joshua", "Kunst", role = c("aut", "cre"), email = "jbkunst@gmail.com"),
  person("Nuno", "Agostinho", role = c("ctb"), comment  = "hchart.survfit, densities and hc_add_series_scatter"),
  person("Danton", "Noriega", role = c("ctb"), comment  = "hcaes_"),
  person("Martin John ", "Hadley", role = c("ctb"), comment  = "hc_add_event_point improvement"),
  person("Eduardo", "Flores", role = c("ctb"), comment  = "First version hc_add_series_df_tidy"),
  person("Dean", "Kilfoyle", role = c("ctb"), comment  = "First version hc_add_series_boxplot"),
  person("Adline", "Dsilva", role = c("ctb"), comment  = "First version Matrix heatmap"),
  person("Kamil", "Slowikowski", role = c("ctb"), comment = c(ORCID = "0000-0002-2843-6370")),
  person("Christian", "Minich", role = c("ctb"), comment  = "hcaes mutate_mapping improvement"),
  person("Jonathan", "Armond", role = c("ctb"), comment = "mutate_mapping bugfix"),
  person("David", "Breuer", role = c("ctb"), comment = "download_map_data quiet parameter"),
  person("Mauricio", "Vargas", role = c("ctb"), comment = "tests and gh-actions"),
  person("Michael", "Yan", role = c("ctb"), comment = "Motivational example for treemap/sunburst data helper"),
  person("Bart", "Oortwijn", role = c("ctb"), comment = "rjson option, hc_add_yAxis, and GH issues collaborator"),
  person("Paul", "Campbell", role = c("ctb"), comment = "additional proxy methods")
  )
URL: https://jkunst.com/highcharter/,
        https://github.com/jbkunst/highcharter
BugReports: https://github.com/jbkunst/highcharter/issues
License: MIT + file LICENSE
RoxygenNote: 7.3.3
Encoding: UTF-8
Depends: R (>= 2.10)
Imports: htmlwidgets, magrittr, purrr, rlist, assertthat, zoo, dplyr
        (>= 1.0.0), tibble (>= 1.1), stringr (>= 1.3.0), broom, xts,
        quantmod, tidyr, htmltools, jsonlite, igraph, lubridate, yaml,
        rlang (>= 0.1.1), rjson
Suggests: knitr, rmarkdown, survival, ggplot2, httr, viridisLite,
        shiny, MASS, gapminder, forecast, geojsonio, testthat, covr,
        spelling
LazyData: true
Language: en-US
NeedsCompilation: no
Packaged: 2026-04-21 18:09:41 UTC; kupietz
Author: Joshua Kunst [aut, cre],
  Nuno Agostinho [ctb] (hchart.survfit, densities and
    hc_add_series_scatter),
  Danton Noriega [ctb] (hcaes_),
  Martin John Hadley [ctb] (hc_add_event_point improvement),
  Eduardo Flores [ctb] (First version hc_add_series_df_tidy),
  Dean Kilfoyle [ctb] (First version hc_add_series_boxplot),
  Adline Dsilva [ctb] (First version Matrix heatmap),
  Kamil Slowikowski [ctb] (ORCID:
    <https://orcid.org/0000-0002-2843-6370>),
  Christian Minich [ctb] (hcaes mutate_mapping improvement),
  Jonathan Armond [ctb] (mutate_mapping bugfix),
  David Breuer [ctb] (download_map_data quiet parameter),
  Mauricio Vargas [ctb] (tests and gh-actions),
  Michael Yan [ctb] (Motivational example for treemap/sunburst data
    helper),
  Bart Oortwijn [ctb] (rjson option, hc_add_yAxis, and GH issues
    collaborator),
  Paul Campbell [ctb] (additional proxy methods)
Maintainer: Joshua Kunst <jbkunst@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-22 09:38:59 UTC
Built: R 4.6.0; ; 2026-04-30 07:03:01 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(hc_add_series,character)
S3method(hc_add_series,data.frame)
S3method(hc_add_series,default)
S3method(hc_add_series,density)
S3method(hc_add_series,factor)
S3method(hc_add_series,forecast)
S3method(hc_add_series,geo_json)
S3method(hc_add_series,geo_list)
S3method(hc_add_series,lm)
S3method(hc_add_series,loess)
S3method(hc_add_series,numeric)
S3method(hc_add_series,ohlc)
S3method(hc_add_series,ts)
S3method(hc_add_series,xts)
S3method(hchart,acf)
S3method(hchart,character)
S3method(hchart,data.frame)
S3method(hchart,default)
S3method(hchart,density)
S3method(hchart,dist)
S3method(hchart,ets)
S3method(hchart,factor)
S3method(hchart,forecast)
S3method(hchart,histogram)
S3method(hchart,igraph)
S3method(hchart,list)
S3method(hchart,matrix)
S3method(hchart,mforecast)
S3method(hchart,mts)
S3method(hchart,numeric)
S3method(hchart,prcomp)
S3method(hchart,princomp)
S3method(hchart,stl)
S3method(hchart,survfit)
S3method(hchart,tibble)
S3method(hchart,ts)
S3method(hchart,xts)
export("%>%")
export(JS)
export(color_classes)
export(color_stops)
export(colorize)
export(create_axis)
export(create_yaxis)
export(data_to_boxplot)
export(data_to_hierarchical)
export(data_to_sankey)
export(datetime_to_timestamp)
export(df_to_annotations_labels)
export(download_map_data)
export(dt_tstp)
export(export_hc)
export(fa_icon)
export(fa_icon_mark)
export(get_data_from_map)
export(hc_add_annotation)
export(hc_add_annotations)
export(hc_add_dependency)
export(hc_add_dependency_fa)
export(hc_add_event_point)
export(hc_add_event_series)
export(hc_add_series)
export(hc_add_series_list)
export(hc_add_series_map)
export(hc_add_theme)
export(hc_add_yAxis)
export(hc_annotations)
export(hc_boost)
export(hc_caption)
export(hc_chart)
export(hc_colorAxis)
export(hc_colors)
export(hc_credits)
export(hc_drilldown)
export(hc_elementId)
export(hc_exporting)
export(hc_labels)
export(hc_legend)
export(hc_loading)
export(hc_mapNavigation)
export(hc_motion)
export(hc_navigator)
export(hc_pane)
export(hc_plotOptions)
export(hc_rangeSelector)
export(hc_responsive)
export(hc_rm_series)
export(hc_scrollbar)
export(hc_series)
export(hc_size)
export(hc_subtitle)
export(hc_theme)
export(hc_theme_538)
export(hc_theme_alone)
export(hc_theme_bloom)
export(hc_theme_chalk)
export(hc_theme_darkunica)
export(hc_theme_db)
export(hc_theme_economist)
export(hc_theme_elementary)
export(hc_theme_ffx)
export(hc_theme_flat)
export(hc_theme_flatdark)
export(hc_theme_ft)
export(hc_theme_ggplot2)
export(hc_theme_google)
export(hc_theme_gridlight)
export(hc_theme_handdrawn)
export(hc_theme_hcrt)
export(hc_theme_merge)
export(hc_theme_monokai)
export(hc_theme_null)
export(hc_theme_sandsignika)
export(hc_theme_smpl)
export(hc_theme_sparkline)
export(hc_theme_sparkline_vb)
export(hc_theme_superheroes)
export(hc_theme_tufte)
export(hc_theme_tufte2)
export(hc_title)
export(hc_tooltip)
export(hc_xAxis)
export(hc_xAxis_multiples)
export(hc_yAxis)
export(hc_yAxis_multiples)
export(hc_zAxis)
export(hc_zAxis_multiples)
export(hcaes)
export(hcaes_)
export(hcaes_string)
export(hcboxplot)
export(hchart)
export(hciconarray)
export(hcmap)
export(hcparcords)
export(hcpxy_add_point)
export(hcpxy_add_series)
export(hcpxy_loading)
export(hcpxy_redraw)
export(hcpxy_remove_point)
export(hcpxy_remove_series)
export(hcpxy_set_data)
export(hcpxy_update)
export(hcpxy_update_point)
export(hcpxy_update_series)
export(hcspark)
export(hctreemap)
export(hctreemap2)
export(hex_to_rgba)
export(highchart)
export(highchart2)
export(highchartOutput)
export(highchartOutput2)
export(highchartOutputZ)
export(highchartProxy)
export(highcharts_demo)
export(highchartzero)
export(hw_grid)
export(is.hexcolor)
export(is.highchart)
export(list_parse)
export(list_parse2)
export(mutate_mapping)
export(renderHighchart)
export(renderHighchart2)
export(renderHighchartZ)
export(str_to_id)
export(tags)
export(tooltip_chart)
export(tooltip_table)
importFrom(assertthat,are_equal)
importFrom(assertthat,assert_that)
importFrom(assertthat,on_failure)
importFrom(broom,augment)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,arrange_)
importFrom(dplyr,as_tibble)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,count)
importFrom(dplyr,count_)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,do)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,filter_)
importFrom(dplyr,glimpse)
importFrom(dplyr,group_by)
importFrom(dplyr,group_by_all)
importFrom(dplyr,group_by_at)
importFrom(dplyr,group_nest)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_)
importFrom(dplyr,mutate_at)
importFrom(dplyr,mutate_if)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,rename_all)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,select_)
importFrom(dplyr,summarise)
importFrom(dplyr,tbl_df)
importFrom(dplyr,transmute)
importFrom(dplyr,ungroup)
importFrom(dplyr,vars)
importFrom(grDevices,boxplot.stats)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,n2mfrow)
importFrom(graphics,hist)
importFrom(htmltools,htmlDependency)
importFrom(htmltools,tagList)
importFrom(htmltools,tags)
importFrom(htmlwidgets,JS)
importFrom(htmlwidgets,createWidget)
importFrom(htmlwidgets,shinyRenderWidget)
importFrom(htmlwidgets,shinyWidgetOutput)
importFrom(htmlwidgets,sizingPolicy)
importFrom(jsonlite,toJSON)
importFrom(lubridate,is.Date)
importFrom(magrittr,"%>%")
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map_chr)
importFrom(purrr,map_df)
importFrom(purrr,map_if)
importFrom(purrr,map_lgl)
importFrom(purrr,pmap_chr)
importFrom(purrr,reduce)
importFrom(purrr,transpose)
importFrom(quantmod,is.OHLC)
importFrom(rlang,"!!!")
importFrom(rlang,"!!")
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,enexprs)
importFrom(rlang,is_missing)
importFrom(rlang,parse_quo)
importFrom(rlang,syms)
importFrom(rlist,list.merge)
importFrom(rlist,list.parse)
importFrom(stats,complete.cases)
importFrom(stats,ecdf)
importFrom(stats,qnorm)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,time)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(stringr,str_glue)
importFrom(stringr,str_remove)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_to_lower)
importFrom(stringr,str_to_title)
importFrom(stringr,str_trim)
importFrom(tibble,has_name)
importFrom(tibble,rownames_to_column)
importFrom(tibble,tibble)
importFrom(tibble,tibble_)
importFrom(tidyr,gather)
importFrom(tidyr,unite)
importFrom(tidyr,unite_)
importFrom(tidyr,unnest)
importFrom(utils,data)
importFrom(utils,download.file)
importFrom(utils,hasName)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(xts,is.xts)
importFrom(yaml,yaml.load_file)
importFrom(zoo,as.Date)

# HIGHCHARTER

[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/highcharter?color=brightgreen)](https://cran.r-project.org/package=highcharter)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/highcharter?color=brightgreen)](https://www.r-pkg.org/pkg/highcharter)
[![Project Status: Active – The project has reached a stable, usablestate and is being activelydeveloped.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![Lifecycle:stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![Codecov test coverage](https://codecov.io/gh/jbkunst/highcharter/branch/master/graph/badge.svg)](https://app.codecov.io/gh/jbkunst/highcharter?branch=master)
[![Last commit](https://img.shields.io/github/last-commit/jbkunst/highcharter.svg)](https://github.com/jbkunst/highcharter/issues)
[![GitHub closed issues](https://img.shields.io/github/issues-raw/jbkunst/highcharter.svg)](https://github.com/jbkunst/highcharter/issues)
[![GitHub issues](https://img.shields.io/github/issues-closed-raw/jbkunst/highcharter.svg)](https://github.com/jbkunst/highcharter/issues)
[![check-windows](https://github.com/jbkunst/highcharter/workflows/check-windows/badge.svg)](https://github.com/jbkunst/highcharter/actions?workflow=check-windows)
[![check-mac](https://github.com/jbkunst/highcharter/workflows/check-mac/badge.svg)](https://github.com/jbkunst/highcharter/actions?workflow=check-mac)
[![check-linux](https://github.com/jbkunst/highcharter/workflows/check-linux/badge.svg)](https://github.com/jbkunst/highcharter/actions?workflow=check-linux)
[![Github Stars](https://img.shields.io/github/stars/jbkunst/highcharter.svg?style=social&label=Github)](https://github.com/jbkunst/highcharter)

## Installation

CRAN version:

```r
install.packages("highcharter")
```

Development version:

```r
remotes::install_github("jbkunst/highcharter")
```

## How to start

There are a lot of vignettes to show how highcharter works and what you 
can do:

- `vignette("highcharter")` explore the basics of the package. 
- `vignette("showcase")` to see how much highcharts is flexible in terms of 
customization and design.
- `vignette("highcharts-api")` show the main functions to configure charts with
the implemented Highcharts API.
- `vignette("highchartsjs-api-basics")` explain the relationship between the
highchartsJS API and highcharter.


## Licence 

Highcharter has a dependency on Highcharts, a commercial JavaScript charting library. Highcharts offers both a commercial license as well as a free non-commercial license. Please review the licensing options and terms before using this software, as the `highcharter` license neither provides nor implies a license for Highcharts.

Highcharts (https://highcharts.com) is a Highsoft product which is not free for commercial and Governmental use.

## Discount for highcharter users

Highsoft provide a discount to the highcharter users. It is a 50% discount on our Single Developer license. More details in https://newsletter.highcharts.com/foss/.

#' Create a highchart object from a particular data type
#'
#' \code{hchart} uses `highchart` to draw a particular plot for an
#' object of a particular class in a single command. This defines the S3
#' generic that other classes and packages can extend.
#'
#' Run \code{methods(hchart)} to see what objects are supported.
#'
#' @param object  A R object.
#' @param ... Additional arguments for the data series
#'    (\url{https://api.highcharts.com/highcharts/series}).
#'
#' @export
hchart <- function(object, ...) {
  UseMethod("hchart")
}

#' @export
hchart.default <- function(object, ...) {
  stop("Objects of class/type ", paste(class(object), collapse = "/"),
    " are not supported by hchart (yet).",
    call. = FALSE
  )
}

#' @export
hchart.list <- function(object, ...) {
  if (getOption("highcharter.verbose")) {
    message("hchart.list")
  }

  highchart() %>%
    hc_add_series(data = object, ...)
}

#' @export
hchart.data.frame <- function(object, type = NULL, mapping = hcaes(), ...) {
  if (getOption("highcharter.verbose")) {
    message("hchart.data.frame")
  }

  object <- as.data.frame(object)

  data <- mutate_mapping(object, mapping)

  series <- data_to_series(data, mapping, type = type, ...)

  opts <- data_to_options(data, type)

  hc <- highchart()

  if (opts$add_colorAxis) {
    hc <- hc_colorAxis(hc, auxpar = NULL)
  }

  hc <- hc %>%
    hc_add_series_list(series) %>%
    hc_xAxis(
      type = opts$xAxis_type,
      title = list(text = as.character(mapping$x)),
      categories = opts$xAxis_categories
    ) %>%
    hc_yAxis(
      type = opts$yAxis_type,
      title = list(text = as.character(mapping$y)),
      categories = opts$yAxis_categories
    ) %>%
    hc_plotOptions(
      series = list(
        showInLegend = opts$series_plotOptions_showInLegend
      ),
      scatter = list(marker = list(symbol = "circle"))
    )

  hc
}

#' @export
hchart.tibble <- hchart.data.frame

#' @importFrom graphics hist
#' @export
hchart.numeric <- function(object, breaks = "FD", ...) {
  h <- hist(object, plot = FALSE, breaks = breaks)

  hchart.histogram(h, ...)
}

#' @export
hchart.histogram <- function(object, ...) {
  d <- diff(object$breaks)[1]

  df <- tibble(
    x = object$mids,
    y = object$counts,
    name = sprintf("(%s, %s]", object$mids - d / 2, object$mids + d / 2)
  )

  highchart() %>%
    hc_chart(zoomType = "x") %>%
    hc_tooltip(formatter = JS("function() { return  this.point.name + '<br/>' + this.y; }")) %>%
    hc_add_series(
      data = list_parse(df),
      type = "column",
      pointRange = d,
      groupPadding = 0,
      pointPadding = 0,
      borderWidth = 0,
      ...
    )
}

#' @export
hchart.character <- function(object, type = "column", ...) {
  highchart() %>%
    hc_xAxis(type = "category") %>%
    hc_add_series(data = object, type = type, ...)
}

#' @export
hchart.factor <- hchart.character

#' @export
hchart.ts <- function(object, ...) {
  highchart() %>%
    hc_xAxis(type = "datetime") %>%
    hc_add_series(data = object, ...)
}

#' @export
hchart.xts <- function(object, ...) {
  highchart(type = "stock") %>%
    hc_add_series(data = object, ...)
}

#' @export
hchart.forecast <- function(object, addOriginal = TRUE, addLevels = TRUE, ...) {
  highchart() %>%
    hc_xAxis(type = "datetime") %>%
    hc_add_series(
      data = object, addOriginal = addOriginal,
      addLevels = addLevels, ...
    )
}

#' @export
hchart.mforecast <- function(object, separate = TRUE, fillOpacity = 0.3, ...) {
  ntss <- ncol(object$x)
  lvls <- object$level
  tmf <- datetime_to_timestamp(zoo::as.Date(time(object$mean[[1]])))
  nms <- attr(object$x, "dimnames")[[2]]

  hc <- hchart_mts2(object$x) %>%
    hc_plotOptions(
      series = list(
        marker = list(enabled = FALSE)
      )
    )

  # means
  hc <- hc %>% hc_add_series(data = NULL, id = "forecasts", name = "forecasts")

  for (i in seq(ntss)) {
    nm <- nms[i]
    hc <- hc %>%
      hc_add_series(object$mean[[nm]],
        name = paste("forecast", nm),
        yAxis = i - 1, linkedTo = "forecasts", ...
      )
  }


  # levels
  for (lvl in seq_along(lvls)) {
    idlvl <- paste0("level", lvls[lvl])
    nmlvl <- paste("level", lvls[lvl])

    hc <- hc %>% hc_add_series(data = NULL, id = idlvl, name = nmlvl, ...)

    for (i in seq(ntss)) {
      nm <- nms[i]

      dfbands <- tibble(
        t = tmf,
        u = as.vector(object$upper[[i]][, lvl]),
        l = as.vector(object$lower[[i]][, lvl])
      )

      hc <- hc %>%
        hc_add_series(
          data = list_parse2(dfbands),
          name = paste(nmlvl, nm),
          linkedTo = idlvl,
          yAxis = i - 1,
          type = "arearange", fillOpacity = fillOpacity,
          zIndex = 1, lineWidth = 0, ...
        )
    }
  }

  if (!separate) {
    hc$x$hc_opts$yAxis <- NULL
    hc$x$hc_opts$series <- map(hc$x$hc_opts$series, function(x) {
      x$yAxis <- NULL
      x
    })
  }

  hc
}

#' @importFrom stats qnorm
#' @export
hchart.acf <- function(object, ...) {
  ytitle <- switch(object$type,
    partial = "Partial ACF",
    covariance = "ACF (cov)",
    correlation = "ACF"
  )

  sv <- qnorm(1 - 0.05 / 2) / sqrt(object$n.used)

  ds <- tibble(
    x = seq(object$lag[, , ]),
    y = object$acf[, , ]
  )

  hc <- highchart() %>%
    hc_add_series(
      data = list_parse(ds),
      type = "column",
      groupPadding = 1,
      name = ytitle
    )

  if (object$type != "covariance") {
    hc <- hc %>%
      hc_yAxis(
        plotLines = list(
          list(width = 1, value = sv, color = "gray"),
          list(width = 1, value = -sv, color = "gray")
        )
      )
  }

  hc
}

#' @export
hchart.mts <- function(object, ..., separate = TRUE,
                       heights = rep(1, ncol(object))) {
  if (separate) {
    hc <- hchart_mts2(object, heights = heights, ...)
  } else {
    hc <- hchart_mts1(object, ...)
  }

  hc
}

hchart_mts1 <- function(object, ...) {
  hc <- highchart() %>%
    hc_xAxis(type = "datetime")

  for (i in seq(dim(object)[2])) {
    nm <- attr(object, "dimnames")[[2]][i]
    if ("ts" %in% class(object[, i])) {
      hc <- hc %>% hc_add_series(object[, i], name = nm, id = nm, ...)
    } else {
      hc <- hc %>% hc_add_series(object[, i], name = nm, id = nm, ...)
    }
  }

  hc
}

hchart_mts2 <- function(object, ..., heights = rep(1, ncol(object)), sep = 0.01) {
  ntss <- ncol(object)

  hc <- highchart() %>%
    hc_xAxis(type = "datetime") %>%
    hc_tooltip(shared = TRUE)

  hc <- hc %>%
    hc_yAxis_multiples(
      create_axis(ntss,
        heights = heights, turnopposite = TRUE,
        title = list(text = NULL), offset = 0, lineWidth = 2,
        showFirstLabel = FALSE, showLastLabel = FALSE, ...
      )
    )

  namestss <- as.character(attr(object, "dimnames")[[2]])

  for (col in seq(ntss)) {
    nm <- namestss[col]
    hc <- hc %>% hc_add_series(object[, col], yAxis = col - 1, name = nm, id = nm, ...)
  }

  hc

}

#' @export
hchart.stl <- function(object, ..., heights = c(2, 1, 1, 1), sep = 0.01) {
  tss <- object$time.series
  ncomp <- ncol(tss)
  data <- drop(tss %*% rep(1, ncomp))
  tss <- cbind(data = data, tss)

  attr(tss, "dimnames")[[2]] <- gsub("tss\\.", "", attr(tss, "dimnames")[[2]])

  hchart_mts2(tss)
}

#' @export
hchart.ets <- function(object, ...) {
  names <- c(y = "observed", l = "level", b = "slope", s1 = "season")

  data <- cbind(object$x, object$states[, colnames(object$states) %in% names(names)])

  cn <- c("y", c(colnames(object$states)))

  colnames(data) <- cn <- names[stats::na.exclude(match(cn, names(names)))]

  hc <- hchart_mts2(data)

  hc <- hc_title(hc, text = paste("Decomposition by", object$method, "method"))

  hc
}

#' @importFrom tidyr gather
#' @importFrom dplyr count_ left_join select
#' @importFrom rlang .data
#' @export
hchart.matrix <- function(object, label = FALSE, showInLegend = FALSE, ...) {
  if (getOption("highcharter.verbose")) {
    message("hchart.maxtrix")
  }

  stopifnot(is.numeric(object))

  df <- as.data.frame(object)

  ismatrix <- is.null(colnames(object)) & is.null(rownames(object))
  pos <- ifelse(ismatrix, 0, 1)

  xnm <- if (is.null(colnames(object))) seq_len(ncol(object)) else colnames(object)
  xnm <- as.character(xnm)
  xid <- seq(length(xnm)) - pos

  ynm <- if (is.null(rownames(object))) seq_len(nrow(object)) else rownames(object)
  ynm <- as.character(ynm)
  yid <- seq(length(ynm)) - pos

  ds <- as.data.frame(df) %>%
    tibble::as_tibble()
  ds <- bind_cols(tibble(ynm), ds)
  ds <- ds %>%
    gather("key", "value", -ynm) %>%
    rename(xnm = .data$key) %>%
    mutate(
      xnm = as.character(.data$xnm),
      ynm = as.character(.data$ynm)
    )

  ds$xnm <- if (is.null(colnames(object))) str_replace(ds$xnm, "V", "") else ds$xnm

  ds <- ds %>%
    left_join(tibble(xnm, xid), by = "xnm") %>%
    left_join(tibble(ynm, yid), by = "ynm") %>%
    mutate(name = paste(.data$xnm, .data$ynm, sep = " ~ ")) %>%
    select(x = .data$xid, y = .data$yid, .data$value, .data$name)

  fntltp <- JS("function(){
                 return this.point.name + ': ' +
                   Highcharts.numberFormat(this.point.value, 2)
               }")

  hc <- highchart() %>%
    hc_add_series(data = list_parse(ds), type = "heatmap") %>%
    hc_plotOptions(
      series = list(
        showInLegend = showInLegend,
        boderWidth = 0,
        dataLabels = list(enabled = label)
      )
    ) %>%
    hc_tooltip(formatter = fntltp) %>%
    hc_legend(enabled = TRUE) %>%
    hc_colorAxis(auxarg = TRUE)

  if (ismatrix) {
    hc <- hc %>%
      hc_xAxis(visible = FALSE) %>%
      hc_yAxis(visible = FALSE, reversed = TRUE)
  } else {
    hc <- hc %>%
      hc_xAxis(categories = xnm, title = list(text = ""), opposite = TRUE) %>%
      hc_yAxis(categories = ynm, title = list(text = ""), reversed = TRUE)
  }

  # correlogram
  if (-1 <= min(object) & min(object) < 0 & max(object) <= 1) {
    hc <- hc_colorAxis(hc,
      stops = color_stops(3, c("#FF5733", "#F8F5F5", "#2E86C1")),
      min = -1, max = 1
    )
  } else {
    hc <- hc_colorAxis(hc, nullpar = NULL)
  }

  hc
}

#' @importFrom tidyr gather
#' @importFrom dplyr count_ left_join select_
#' @export
hchart.dist <- function(object, ...) {
  hchart.matrix(as.matrix(object))
}

#' Plot igraph objects using Highcharts
#' 
#' @param object An igraph object.
#' @param layout A layout from igraph package.
#' @param ... Additional arguments for the data series
#'    (\url{https://api.highcharts.com/highcharts/series}). 
#' @importFrom stats setNames
#' @importFrom rlang .data
#' @examples
#' 
#' if(require("igraph")) {
#' 
#' N <- 40
#' net <- sample_gnp(N, p = 2 / N)
#' wc <- cluster_walktrap(net)
#' V(net)$label <- seq(N)
#' V(net)$name <- paste("I'm #", seq(N))
#' V(net)$page_rank <- round(page_rank(net)$vector, 2)
#' V(net)$betweenness <- round(betweenness(net), 2)
#' V(net)$degree <- degree(net)
#' V(net)$size <- V(net)$degree
#' V(net)$comm <- as.vector(membership(wc))
#' V(net)$color <- colorize(membership(wc))
#' hchart(net, layout = layout_with_fr)
#' 
#' }
#' 
#' 
#' @export
hchart.igraph <- function(object, ..., layout = NULL) {
  # @importFrom igraph vertex_attr edge_attr get.edgelist layout_nicely
  requireNamespace("igraph") 
  # data
  if(missing(layout)) layout <- igraph::layout_nicely
  dfv <- layout(object) %>%
    round(5) %>%
    data.frame() %>%
    tibble::as_tibble() %>%
    setNames(c("x", "y"))

  dfvex <- object %>%
    igraph::vertex_attr() %>%
    data.frame(stringsAsFactors = FALSE) %>%
    tibble::as_tibble()

  if (nrow(dfvex) > 0) {
    dfv <- tibble::as_tibble(cbind(dfv, dfvex))
  }

  if (is.null(dfv[["name"]])) {
    dfv <- dfv %>% mutate(name = seq(nrow(dfv)))
  }

  names(dfv) <- str_replace_all(names(dfv), "\\.", "_")

  dfe <- object %>%
    igraph::get.edgelist() %>%
    data.frame(stringsAsFactors = FALSE) %>%
    tibble::as_tibble() %>%
    setNames(c("from", "to")) %>%
    left_join(dfv %>%
      select(.data$name, .data$x, .data$y) %>%
      setNames(c("from", "xf", "yf")), by = "from") %>%
    left_join(dfv %>%
      select(.data$name, .data$x, .data$y) %>%
      setNames(c("to", "xt", "yt")), by = "to") %>%
    mutate(linkedTo = "e")

  dfex <- object %>%
    igraph::edge_attr() %>%
    data.frame(stringsAsFactors = FALSE) %>%
    tibble::as_tibble()

  if (nrow(dfex) > 0) {
    dfe <- tibble::as_tibble(cbind(dfe, dfex))
  }

  # Checking opts
  type <- "scatter"

  if ("size" %in% names(dfv)) {
    dfv <- dfv %>% rename(z = .data$size)
    type <- "bubble"
  }

  if ("group" %in% names(dfv)) {
    dfv <- dfv %>% rename(groupvar = .data$group)
  }

  if ("width" %in% names(dfe)) {
    dfe <- dfe %>% rename(lineWidth = .data$width)
  }

  if (!"color" %in% names(dfe)) {
    dfe <- dfe %>% mutate("color" = hex_to_rgba("#d3d3d3", 0.5))
  }

  dse <- list_parse(dfe) %>%
    map(function(x) {
      x$data <- list(
        list(x = x$xf, y = x$yf),
        list(x = x$xt, y = x$yt)
      )
      x[c("xf", "yf", "xt", "yt")] <- NULL
      x
    })

  vattrs <- setdiff(names(dfv), c("x", "y", "z", "color", "label", "name"))

  tltip_fmt <- tooltip_table(
    str_to_title(str_replace_all(vattrs, "_", " ")),
    sprintf("{point.%s}", vattrs)
  )

  hc <- highchart() %>%
    hc_chart(zoomType = "xy", type = "line") %>%
    hc_tooltip(
      useHTML = TRUE
    ) %>%
    hc_plotOptions(
      line = list(enableMouseTracking = FALSE),
      bubble = list(
        marker = list(fillOpacity = 0.65),
        minSize = 5, maxSize = 20
      )
    ) %>%
    hc_add_theme(
      hc_theme_null(legend = list(enabled = TRUE))
    )

  hc <- hc %>%
    hc_add_series(
      data = list_parse(dfv),
      type = type, name = "nodes", zIndex = 3,
      tooltip = list(
        headerFormat = as.character(tags$small("{point.key}")),
        pointFormat = tltip_fmt
      ), ...
    )

  if ("label" %in% names(dfv)) {
    hc <- hc %>%
      hc_add_series(
        data = list_parse(dfv %>% select(.data$x, .data$y, .data$label)),
        type = "scatter", name = "labels", zIndex = 4,
        marker = list(radius = 0), enableMouseTracking = FALSE,
        dataLabels = list(enabled = TRUE, format = "{point.label}")
      )
  }

  hc <- hc %>%
    hc_add_series(data = NULL, name = "edges", id = "e") %>%
    hc_add_series_list(dse)

  hc
}

#' Plot survival curves using Highcharts
#'
#' @param object A survfit object as returned from the \code{survfit} function
#' @param ... Extra parameters to pass to \code{hc_add_series} function
#' @param fun Name of function or function used to transform the survival curve:
#' \code{log} will put y axis on log scale, \code{event} plots cumulative events
#' (f(y) = 1-y), \code{cumhaz} plots the cumulative hazard function (f(y) =
#' -log(y)), and \code{cloglog} creates a complimentary log-log survival plot
#' (f(y) = log(-log(y)) along with log scale for the x-axis.
#' @param markTimes Label curves marked at each censoring time? TRUE by default
#' @param symbol Symbol to use as marker (plus sign by default)
#' @param markerColor Color of the marker ("black" by default); use NULL to use
#' the respective color of each series
#' @param ranges Plot interval ranges? FALSE by default
#' @param rangesOpacity Opacity of the interval ranges (0.3 by default)
#'
#' @return Highcharts object to plot survival curves
#'
#' @examples
#'
#' # Plot Kaplan-Meier curves
#' require("survival")
#' leukemia.surv <- survfit(Surv(time, status) ~ x, data = aml)
#' hchart(leukemia.surv)
#'
#' # Plot the cumulative hazard function
#' lsurv2 <- survfit(Surv(time, status) ~ x, aml, type = "fleming")
#' hchart(lsurv2, fun = "cumhaz")
#'
#' # Plot the fit of a Cox proportional hazards regression model
#' fit <- coxph(Surv(futime, fustat) ~ age, data = ovarian)
#' ovarian.surv <- survfit(fit, newdata = data.frame(age = 60))
#' hchart(ovarian.surv, ranges = TRUE)
#' @export
hchart.survfit <- function(object, ..., fun = NULL, markTimes = TRUE,
                           symbol = "plus", markerColor = "black",
                           ranges = FALSE, rangesOpacity = 0.3) {
  group <- NULL

  # Check if there are groups
  if (is.null(object$strata)) {
    strata <- c("Series 1" = length(object$time))
  } else {
    strata <- object$strata
  }

  # Modify data according to functions (adapted from survival:::plot.survfit)
  if (is.character(fun)) {
    tfun <- switch(fun,
      log = function(x) x,
      event = function(x) 1 - x,
      cumhaz = function(x) -log(x),
      cloglog = function(x) log(-log(x)),
      pct = function(x) x * 100,
      logpct = function(x) 100 * x,
      identity = function(x) x,
      function(x) x
    )
  } else if (is.function(fun)) {
    tfun <- fun
  } else {
    tfun <- function(x) x
  }

  firsty <- tfun(1)

  object$surv <- tfun(object$surv)

  if (ranges && !is.null(object$upper)) {
    object$upper <- tfun(object$upper)
    object$lower <- tfun(object$lower)
  }

  # Prepare data
  data <- data.frame(
    x = object$time, y = object$surv,
    up = object$upper, low = object$lower,
    group = rep(names(strata), strata),
    stringsAsFactors = FALSE
  )

  # Data markers
  marker <- list(list(fillColor = markerColor, symbol = symbol, enabled = TRUE))

  if (markTimes) {
    mark <- object$n.censor == 1
  } else {
    mark <- FALSE
  }

  # Adjust Y axis range
  yValues <- object$surv
  ymin <- ifelse(min(yValues) >= 0, 0, min(yValues))
  ymax <- ifelse(max(yValues) <= 1, 1, max(yValues))

  hc <- highchart() %>%
    hc_tooltip(shared = TRUE) %>%
    hc_yAxis(min = ymin, max = ymax) %>%
    hc_plotOptions(line = list(marker = list(enabled = FALSE)))

  count <- 0

  # Process groups by columns (CoxPH-like) or in a single column
  if (!is.null(ncol(object$surv))) {
    groups <- seq(ncol(object$surv))
  } else {
    groups <- names(strata)
  }

  for (name in groups) {
    if (!is.null(ncol(object$surv))) {
      df <- df[c("x", paste(c("y", "low", "up"), col, sep = "."))]
      names(df) <- c("x", "y", "low", "up")
      submark <- mark
    } else {
      df <- subset(data, group == name)
      submark <- mark[data$group == name]
    }

    # Add first value if there is no value for time at 0 in the data
    if (!0 %in% df$x) {
      first <- list(list(x = 0, y = firsty))
    } else {
      first <- NULL
    }

    # Mark events
    ls <- list_parse(df)
    if (markTimes) {
      ls[submark] <- lapply(ls[submark], c, marker = marker)
    }


    hc <- hc %>%
      hc_add_series(
        data = c(first, ls), step = "left", name = name, zIndex = 1,
        color = JS(sprintf("Highcharts.getOptions().colors[%s]", count)),
        ...
      )

    if (ranges && !is.null(object$upper)) {
      # Add interval range
      range <- lapply(ls, function(i) {
        setNames(i[c("x", "low", "up")], NULL)
      })

      hc <- hc %>%
        hc_add_series(
          data = range, step = "left", name = "Ranges",
          type = "arearange", zIndex = 0, linkedTo = ":previous",
          fillOpacity = rangesOpacity, lineWidth = 0,
          color = JS(sprintf("Highcharts.getOptions().colors[%s]", count)),
          ...
        )
    }
    count <- count + 1
  }

  hc <- hc %>%
    hc_plotOptions(
      series = list(
        states = list(hover = list(enabled = FALSE))
      )
    )

  return(hc)
}

#' @importFrom tibble rownames_to_column
#' @importFrom rlang .data
#' @export
hchart.density <- function(object, type = "area", ...) {
  hc_add_series(highchart(), data = object, type = type, ...)
}

#' @importFrom dplyr as_tibble
hchart_pca <- function(sdev, n.obs, scores, loadings, ...,
                       choices = 1L:2L, scale = 1) {
  stopifnot(length(choices) == 2L)
  stopifnot(0 <= scale | scale <= 1)

  lam <- sdev[choices]
  lam <- lam * sqrt(n.obs)

  if (scale != 0) {
    lam <- lam^scale
  } else {
    lam <- 1
  }

  dfobs <- (scores[, choices] / lam) %>%
    as.data.frame() %>%
    setNames(c("x", "y")) %>%
    rownames_to_column("name")

  dfcomp <- loadings[, choices] * lam

  mx <- max(abs(dfobs[, 2:3]))
  mc <- max(abs(dfcomp))

  dfcomp <- dfcomp
  dfcomp <- dfcomp / mc * mx
  dfcomp <- dfcomp %>% 
    as.data.frame() %>%
    setNames(c("x", "y")) %>%
    rownames_to_column("name") %>%
    as_tibble() %>%
    group_by(.data$name) %>%
    do(data = list(c(0, 0), c(.$x, .$y))) %>%
    list_parse()

  highchart() %>%
    hc_plotOptions(
      line = list(
        marker = list(enabled = FALSE)
      )
    ) %>%
    hc_add_series(
      data = list_parse(dfobs), name = "observations", type = "scatter",
      dataLabels = list(enabled = TRUE, format = "{point.name}"), ...
    ) %>%
    hc_add_series_list(dfcomp)
}

#' @export
hchart.princomp <- function(object, ..., choices = 1L:2L, scale = 1) {
  hchart_pca(object$sdev, object$n.obs, object$scores, object$loadings,
    choices = choices, scale = scale, ...
  )
}

#' @importFrom dplyr as_tibble
#' @export
hchart.prcomp <- function(object, ..., choices = 1L:2L, scale = 1) {
  hchart_pca(object$sdev, nrow(object$x), object$x, object$rotation,
    choices = choices, scale = scale, ...
  )
}

checkProxy <- function(proxy) {
  if (!"higchartProxy" %in% class(proxy)) {
    stop("This function must be used with a highchart proxy object")
  }
}

#' Send commands to a Highcharts instance in a Shiny app
#'
#' @param shinyId Single-element character vector indicating the output ID of
#'   the chart to modify
#' @param session The Shiny session object to which the map belongs; usually
#' the default value will suffice.
#' @export
highchartProxy <- function(shinyId, session = shiny::getDefaultReactiveDomain()) {
  proxy <- list(id = shinyId, session = session)
  class(proxy) <- c("higchartProxy")

  return(proxy)
}

#' Add data to higchartProxy element
#'
#' @param proxy A `higchartProxy` object.
#' @param data An R object supported by `hc_add_series` like data frame, ts, etc.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.
#' @export
hcpxy_add_series <- function(proxy, data = NULL, ...) {
  checkProxy(proxy)

  stopifnot(!is.null(data))

  hcaux <- highchart() %>%
    hc_add_series(data = data, ...)

  series <- hcaux[["x"]][["hc_opts"]][["series"]]

  for (i in 1:length(series)) {
    if (getOption("highcharter.verbose")) {
      message("hcpxy_add_series")
    }

    proxy$session$sendCustomMessage(
      type = "addSeries",
      message = list(
        id = proxy$id,
        series = series[[i]]
      )
    )
  }

  proxy
}

#' Remove series to higchartProxy element
#'
#' @param proxy A `higchartProxy` object.
#' @param id A character vector indicating the `id` (or `id`s) of the series to remove.
#' @param all A logical value to indicate to remove or not all series. The values
#'   is used only when the value is `TRUE`.
#' @export
hcpxy_remove_series <- function(proxy, id = NULL, all = FALSE) {
  checkProxy(proxy)

  # stopifnot(
  #   !is.null(id),
  #   is.character(id),
  #   is.logical(all)
  #   )

  if (all) {
    proxy$session$sendCustomMessage(
      type = "removeAllSeries",
      message = list(
        id = proxy$id
      )
    )

    return(proxy)
  }


  for (i in 1:length(id)) {
    if (getOption("highcharter.verbose")) {
      message("hcpxy_remove_series")
    }

    proxy$session$sendCustomMessage(
      type = "removeSeries",
      message = list(
        id = proxy$id,
        idSeries = id[[i]]
      )
    )
  }

  proxy
}

#' Show or hide loading text for a higchartProxy object
#'
#' @param proxy A `higchartProxy` object.
#' @param action Single-element character vector indicating to `"show"` or
#'   `"hide"` the loading text defined in `lang` options.
#' @export
hcpxy_loading <- function(proxy, action = "show") {
  checkProxy(proxy)

  stopifnot(
    is.character(action),
    action %in% c("show", "hide")
  )

  value <- action == "show"

  proxy$session$sendCustomMessage(
    type = "showLoading",
    message = list(
      id = proxy$id,
      showLoading = value
    )
  )

  proxy
}

#' Update options for a higchartProxy object
#'
#' @param proxy A `higchartProxy` object.
#' @param ... Named options.
#' @export
hcpxy_update <- function(proxy, ...) {
  checkProxy(proxy)

  proxy$session$sendCustomMessage(
    type = "updateChart",
    message = list(
      id = proxy$id,
      options = list(...)
    )
  )

  proxy
}

#' Update options series in a higchartProxy object
#'
#' @param proxy A `higchartProxy` object.
#' @param id A character vector indicating the `id` (or `id`s) of the series to update.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.
#'   The arguments will be the same for each series. So if you want update data it
#'   is used this function sequentially for each series.
#'
#' @export
hcpxy_update_series <- function(proxy, id = NULL, ...) {
  checkProxy(proxy)

  for (i in 1:length(id)) {
    if (getOption("highcharter.verbose")) {
      message("hcpxy_update_series")
    }

    proxy$session$sendCustomMessage(
      type = "updateSeries",
      message = list(
        id = proxy$id,
        idSeries = id[[i]],
        options = list(...)
      )
    )
  }

  proxy
}

#' Update options series in a higchartProxy object
#'
#' @param proxy A `higchartProxy` object.
#' @param id A character indicating the `id` of the series' point to update.
#' @param id_point A vector value indicating the point's index to update, (0 based).
#' @param ... Arguments defined in \url{https://api.highcharts.com/class-reference/Highcharts.Point}.
#'   The arguments will be the same for each series. So if you want update data it
#'   is used this function sequentially for each point
#'
#' @export
hcpxy_update_point <- function(proxy, id = NULL, id_point = NULL, ...) {
  checkProxy(proxy)

  for (i in 1:length(id_point)) {
    if (getOption("highcharter.verbose")) {
      message("hcpxy_update_point")
    }

    proxy$session$sendCustomMessage(
      type = "updatePoint",
      message = list(
        id = proxy$id,
        idSeries = id,
        idPoint = id_point[[i]],
        options = list(...)
      )
    )
  }

  proxy
}


#' Add point to a series of a higchartProxy object
#'
#' @param proxy A `higchartProxy` object.
#' @param id A character vector indicating the `id` of the series to update.
#' @param point The point options. If options is a single number, a point with
#'   that y value is appended to the series. If it is an list, it will be
#'   interpreted as x and y values respectively. If it is an object,
#'   advanced options as outlined under series.data are applied
#' @param redraw Whether to redraw the chart after the point is added. When
#'   adding more than one point, it is highly recommended that the redraw option
#'   be set to false, and instead Highcharts.Chart#redraw is explicitly called
#'   after the adding of points is finished. Otherwise, the chart will redraw
#'   after adding each point.
#' @param shift If `TRUE`, a point is shifted off the start of the series as
#'   one is appended to the end.
#' @param animation Whether to apply animation, and optionally animation configuration.
#' @export
hcpxy_add_point <- function(proxy, id = NULL, point, redraw = TRUE, shift = FALSE, animation = TRUE) {
  checkProxy(proxy)

  proxy$session$sendCustomMessage(
    type = "addPoint",
    message = list(
      id = proxy$id,
      idSeries = id,
      point = point,
      redraw = redraw,
      shift = shift,
      animation = animation
    )
  )

  proxy
}

#' Remove point to a series of a higchartProxy object
#'
#' @param proxy A `higchartProxy` object.
#' @param id A character vector indicating the `id` of the series to update.
#' @param i The index of the point in the data array. Remember js is 0 based index.
#' @param redraw Whether to redraw the chart after the point is added. When
#'   adding more than one point, it is highly recommended that the redraw option
#'   be set to false, and instead Highcharts.Chart#redraw is explicitly called
#'   after the adding of points is finished. Otherwise, the chart will redraw
#'   after adding each point.
#' @export
hcpxy_remove_point <- function(proxy, id = NULL, i = NULL, redraw = TRUE) {
  checkProxy(proxy)

  proxy$session$sendCustomMessage(
    type = "removePoint",
    message = list(
      id = proxy$id,
      idSeries = id,
      i = i,
      redraw = redraw
    )
  )

  proxy
}

#' Update data for a higchartProxy object
#'
#' @param proxy A `higchartProxy` object.
#' @param type series type (column, bar, line, etc)
#' @param data dataframe of new data to send to chart
#' @param mapping how data should be mapped using `hcaes()`
#' @param redraw boolean Whether to redraw the chart after the series is altered.
#'   If doing more operations on the chart, it is a good idea to set redraw to false and call hcpxy_redraw after.
#' @param animation boolean When the updated data is the same length as the existing data, points will be updated by default,
#'   and animation visualizes how the points are changed. Set false to disable animation, or a configuration object to set duration or easing.
#' @param updatePoints boolean When this is TRUE, points will be updated instead of replaced whenever possible.
#'   This occurs a) when the updated data is the same length as the existing data, b) when points are matched by their id's, or c) when points can be matched by X values.
#'   This allows updating with animation and performs better. In this case, the original array is not passed by reference. Set FALSE to prevent.
#'
#' @export
hcpxy_set_data <- function(proxy, type, data, mapping = hcaes(), redraw = FALSE, animation = NULL, updatePoints = TRUE) {
  checkProxy(proxy)

  data <- mutate_mapping(data, mapping)

  series <- data_to_series(data, mapping, type = type)

  for (i in 1:length(series)) {
    proxy$session$sendCustomMessage(
      type = "setData",
      message = list(
        id = proxy$id,
        serie = i - 1,
        data = series[[i]]$data,
        redraw = redraw,
        animation = animation,
        updatePoints = updatePoints
      )
    )
  }

  proxy
}

#' Redraw a higchartProxy object
#'
#' @param proxy A `higchartProxy` object.
#'
#' @export
hcpxy_redraw <- function(proxy) {
  checkProxy(proxy)

  proxy$session$sendCustomMessage(
    type = "redraw",
    message = list(id = proxy$id)
  )

  proxy
}

#' @importFrom stats runif
.join_hc_opts <- function() {
  list(
    global = getOption("highcharter.global"),
    lang = getOption("highcharter.lang"),
    chart = getOption("highcharter.chart")
  )
}

.onAttach <- function(libname = find.package("highcharter"),
                      pkgname = "highcharter") {
  if (stats::runif(1) <= 1 / 3) {
    packageStartupMessage("Highcharts (www.highcharts.com) is a Highsoft software product which is")

    packageStartupMessage("not free for commercial and Governmental use")
  }
}

.onLoad <- function(libname = find.package("highcharter"),
                    pkgname = "highcharter") {
  options(
    highcharter.global = list(
      Date = NULL,
      VMLRadialGradientURL =
        "http =//code.highcharts.com/list(version)/gfx/vml-radial-gradient.png",
      canvasToolsURL =
        "http =//code.highcharts.com/list(version)/modules/canvas-tools.js",
      getTimezoneOffset = NULL,
      timezoneOffset = 0,
      useUTC = TRUE
    )
  )

  options(
    highcharter.lang = list(
      contextButtonTitle = "Chart context menu",
      decimalPoint = ".",
      downloadCSV = "Download CSV",
      downloadJPEG = "Download JPEG image",
      downloadPDF = "Download PDF document",
      downloadPNG = "Download PNG image",
      downloadSVG = "Download SVG vector image",
      downloadXLS = "Download XLS",
      drillUpText = "\u25C1 Back to {series.name}",
      exitFullscreen = "Exit from full screen",
      exportData = list(
        annotationHeader = "Annotations",
        categoryDatetimeHeader = "DateTime",
        categoryHeader = "Category"
      ),
      hideData = "Hide data table",
      invalidDate = NULL,
      loading = "Loading...",
      months = c(
        "January", "February", "March", "April",
        "May", "June", "July", "August",
        "September", "October", "November", "December"
      ),
      # navigation
      noData = "No data to display",
      numericSymbolMagnitude = 1000,
      numericSymbols = c("k", "M", "G", "T", "P", "E"),
      printChart = "Print chart",
      resetZoom = "Reset zoom",
      resetZoomTitle = "Reset zoom level 1:1",
      shortMonths = c(
        "Jan", "Feb", "Mar", "Apr", "May", "Jun",
        "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
      ),
      shortWeekdays = c("Sat", "Sun", "Mon", "Tue", "Wed", "Thu", "Fri"),
      thousandsSep = " ",
      viewData = "View data table",
      viewFullscreen = "View in full screen",
      weekdays = c(
        "Sunday", "Monday", "Tuesday", "Wednesday",
        "Thursday", "Friday", "Saturday"
      )
    )
  )

  options(
    highcharter.chart = list(
      chart = list(
        reflow = TRUE
      ),
      title = list(
        text = NULL
      ),
      yAxis = list(
        title = list(
          text = NULL
        )
      ),
      credits = list(
        enabled = FALSE
      ),
      exporting = list(
        enabled = FALSE
      ),
      boost = list(
        enabled = FALSE
      ),
      plotOptions = list(
        series = list(
          # start disabled series-label.js module https://api.highcharts.com/highcharts/plotOptions.series.label
          label = list(enabled = FALSE),
          # this is DUE inteally all data is given as named arrays
          # https://www.highcharts.com/errors/12/
          turboThreshold = 0
        ),
        treemap = list(layoutAlgorithm = "squarified")
      )
    )
  )

  options(
    highcharter.theme = hc_theme(
      chart = list(backgroundColor = "transparent"),
      colors = c(
        "#7cb5ec", "#434348", "#90ed7d", "#f7a35c", "#8085e9",
        "#f15c80", "#e4d354", "#2b908f", "#f45b5b", "#91e8e1"
      )
    ),
    highcharter.verbose = FALSE,
    highcharter.google_fonts = TRUE,
    highcharter.debug = FALSE,
    highcharter.rjson = FALSE,
    highcharter.download_map_data = TRUE,
    highcharter.color_palette = c(
      "#7cb5ec", "#434348", "#90ed7d", "#f7a35c", "#8085e9",
      "#f15c80", "#e4d354", "#2b908f", "#f45b5b", "#91e8e1"
    )
  )
}

#' Sparkline theme for highcharts
#'
#' Sparkline theme is based on \url{https://www.highcharts.com/demo/sparkline}
#' and this post  \url{https://jkunst.com/blog/posts/2020-06-26-valuebox-and-sparklines/}.
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_sparkline())
#' @export
hc_theme_sparkline <- function(...) {
  theme <- list(
    chart = list(
      backgroundColor = NULL,
      borderWidth = 0,
      margin = list(2, 0, 2, 0),
      style = list(
        overflow = "visible"
      ),
      skipClone = TRUE
    ),
    title = list(text = ""),
    xAxis = list(
      labels = list(enabled = FALSE),
      title = list(text = NULL),
      startOnTick = FALSE,
      endOnTick = FALSE,
      tickPositions = list()
    ),
    yAxis = list(
      endOnTick = FALSE,
      startOnTick = FALSE,
      labels = list(
        enabled = FALSE
      ),
      title = list(
        text = NULL
      ),
      tickPositions = list()
    ),
    tooltip = list(
      backgroundColor = NULL,
      borderWidth = 0,
      shadow = FALSE,
      useHTML = TRUE,
      hideDelay = 0,
      shared = TRUE,
      padding = 0,
      positioner = JS(
        "function (w, h, point) { return {
            x: point.plotX - w / 2,
            y: point.plotY - h
          };
        }"
      )
    ),
    plotOptions = list(
      series = list(
        animation = FALSE,
        lineWidth = 1,
        shadow = FALSE,
        states = list(
          hover = list(
            lineWidth = 1
          )
        ),
        marker = list(
          radius = 1,
          states = list(
            hover = list(
              radius = 2
            )
          )
        ),
        fillOpacity = 0.25
      )
    )
  )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}


#' @rdname hc_theme_538
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_sparkline_vb())
#' @export
hc_theme_sparkline_vb <- function(...) {
  theme <- list(
    chart = list(
      backgroundColor = NULL,
      margins = c(0, 0, 0, 0),
      spacingTop = 0,
      spacingRight = 0,
      spacingBottom = 0,
      spacingLeft = 0,
      plotBorderWidth = 0,
      borderWidth = 0,
      style = list(overflow = "visible")
    ),
    xAxis = list(
      visible = FALSE,
      endOnTick = FALSE,
      startOnTick = FALSE
    ),
    yAxis = list(
      visible = FALSE,
      endOnTick = FALSE,
      startOnTick = FALSE
    ),
    tooltip = list(
      outside = FALSE,
      shadow = FALSE,
      borderColor = "transparent",
      botderWidth = 0,
      backgroundColor = "transparent",
      style = list(textOutline = "5px white")
    ),
    plotOptions = list(
      series = list(
        marker = list(enabled = FALSE),
        shadow = FALSE,
        fillOpacity = 0.25,
        color = "#FFFFFFBF",
        fillColor = list(
          linearGradient = list(x1 = 0, y1 = 1, x2 = 0, y2 = 0),
          stops = list(
            list(0.00, "#FFFFFF00"),
            list(0.50, "#FFFFFF7F"),
            list(1.00, "#FFFFFFFF")
          )
        )
      )
    ),
    credits = list(
      enabled = FALSE,
      text = ""
    )
  )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' @importFrom assertthat are_equal on_failure assert_that
.is_hc_theme <- function(hc_theme) {
  are_equal(class(hc_theme), "hc_theme")
}

assertthat::on_failure(.is_hc_theme) <- function(call, env) {
  "The theme used is not from hc_theme class"
}

#' Superheroes theme for highcharts
#'
#' The superheroes theme is inspired by \url{https://public.tableau.com/app/profile/ryansmith#!/vizhome/HeroesofNewYork/SuperheroesinNewYork}
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_superheroes())
#' @export
hc_theme_superheroes <- function(...) {
  theme <- hc_theme_flat(
    chart = list(
      backgroundColor = "#0B486B",
      style = list(
        color = "white",
        fontFamily = "Oswald",
        fontWeight = "normal"
      )
    ),
    xAxis = list(
      gridLineColor = "#46627f",
      tickColor = "#46627f",
      lineColor = "#46627f",
      title = list(
        style = list(
          color = "#FFFFFF"
        )
      )
    ),
    yAxis = list(
      gridLineColor = "#46627f",
      tickColor = "#46627f",
      title = list(
        style = list(
          color = "#FFFFFF"
        )
      )
    ),
    title = list(
      style = list(
        fontSize = "2em",
        fontFamily = "Bangers",
        color = "#FFFFFF"
      )
    ),
    subtitle = list(
      style = list(
        color = "#FFFFFF"
      )
    ),
    legend = list(
      itemStyle = list(
        color = "white",
        fontWeight = "normal"
      ),
      itemHoverStyle = list(
        color = "#C0C0C0"
      ),
      itemHiddenStyle = list(
        color = "#444444"
      )
    )
  )

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' Grid Light theme for highcharts
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_gridlight())
#' @export
hc_theme_gridlight <- function(...) {
  theme <-
    list(
      colors = c(
        "#7CB5EC", "#F7A35C", "#90EE7E", "#7798BF",
        "#AAEEEE", "#FF0066", "#EEAAEE", "#55BF3B"
      ),
      chart = list(
        backgroundColor = NULL,
        style = list(
          fontFamily = "Dosis, sans-serif"
        )
      ),
      title = list(
        style = list(
          fontSize = "16px",
          fontWeight = "bold",
          textTransform = "uppercase"
        )
      ),
      tooltip = list(
        borderWidth = 0,
        backgroundColor = "rgba(219,219,216,0.8)",
        shadow = FALSE
      ),
      legend = list(
        itemStyle = list(
          fontWeight = "bold",
          fontSize = "13px"
        )
      ),
      xAxis = list(
        gridLineWidth = 1,
        labels = list(
          style = list(
            fontSize = "12px"
          )
        )
      ),
      yAxis = list(
        minorTickInterval = "auto",
        title = list(
          style = list(
            textTransform = "uppercase"
          )
        ),
        labels = list(
          style = list(
            fontSize = "12px"
          )
        )
      ),
      plotOptions = list(
        candlestick = list(
          lineColor = "#404048"
        )
      ),
      background2 = "#F0F0EA"
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' Bloomberg Graphics theme for highcharts
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_bloom())
#' @export
hc_theme_bloom <- function(...) {
  theme <-
    list(
      colors = c("#E10033", "#000000", "#767676", "#E4E4E4"),
      chart = list(
        backgroundColor = "#FFFFFF",
        style = list(
          fontFamily = "Roboto",
          color = "#000000"
        )
      ),
      title = list(
        align = "left",
        style = list(
          fontFamily = "Roboto",
          color = "#000000",
          fontWeight = "bold"
        )
      ),
      subtitle = list(
        align = "left",
        style = list(
          fontFamily = "Roboto",
          color = "#000000",
          fontWeight = "bold"
        )
      ),
      xAxis = list(
        lineColor = "#000000",
        lineWidth = 2,
        tickColor = "#000000",
        tickWidth = 2,
        labels = list(
          style = list(
            color = "black"
          )
        ),
        title = list(
          style = list(
            color = "black"
          )
        )
      ),
      yAxis = list(
        opposite = TRUE,
        # gridLineDashStyle = "Dot",
        gridLineWidth = 2,
        gridLineColor = "#F3F3F3",
        lineColor = "#CEC6B9",
        minorGridLineColor = "#CEC6B9",
        labels = list(
          align = "left",
          style = list(
            color = "black"
          ),
          x = 0,
          y = -2
        ),
        tickLength = 0,
        tickColor = "#CEC6B9",
        tickWidth = 1,
        title = list(
          style = list(
            color = "black"
          )
        )
      ),
      tooltip = list(
        backgroundColor = "#FFFFFF",
        borderColor = "#76c0c1",
        style = list(
          color = "#000000"
        )
      ),
      legend = list(
        layout = "horizontal",
        align = "left",
        verticalAlign = "top",
        itemStyle = list(
          color = "#3C3C3C"
        ),
        itemHiddenStyle = list(
          color = "#606063"
        )
      ),
      credits = list(
        style = list(
          color = "#666"
        )
      ),
      labels = list(
        style = list(
          color = "#D7D7D8"
        )
      ),
      drilldown = list(
        activeAxisLabelStyle = list(
          color = "#F0F0F3"
        ),
        activeDataLabelStyle = list(
          color = "#F0F0F3"
        )
      ),
      navigation = list(
        buttonOptions = list(
          symbolStroke = "#DDDDDD",
          theme = list(
            fill = "#505053"
          )
        )
      ),
      legendBackgroundColor = "rgba(0, 0, 0, 0.5)",
      background2 = "#505053",
      dataLabelsColor = "#B0B0B3",
      textColor = "#C0C0C0",
      contrastTextColor = "#F0F0F3",
      maskColor = "rgba(255,255,255,0.3)"
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' An `htmlwidget` interface to the
#' Highcharts javascript chart library
#'
#' Highcharts \url{https://www.highcharts.com/} is a mature javascript
#' charting library. Highcharts provide a various type of charts, from
#' scatters to heatmaps or treemaps.
#'
#' @author Joshua Kunst (@@jbkunst)
#' @importFrom utils data
#' @importFrom stringr str_c str_detect str_extract str_replace str_replace_all
#'      str_trim str_to_lower str_to_title
#' @importFrom htmlwidgets JS
"_PACKAGE"

#' highcharter exported operators and S3 methods
#'
#' The following functions are imported and then re-exported
#' from the highcharter package to avoid listing the magrittr
#' as Depends of highcharter.
#'
#' @name highcharter-exports
NULL

#' @importFrom htmlwidgets JS
#' @name JS
#' @export
#' @rdname highcharter-exports
NULL

#' @importFrom magrittr %>%
#' @name %>%
#' @export
#' @rdname highcharter-exports
NULL

#' @importFrom htmltools tags
#' @name tags
#' @export
#' @rdname highcharter-exports
NULL

if (getRversion() >= "2.15.1") utils::globalVariables(c(".", "colorValue", "level", "name", "parent"))

#' Dark Unica theme for highcharts
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_darkunica())
#' @export
hc_theme_darkunica <- function(...) {
  theme <-
    list(
      colors = c(
        "#2b908f", "#90ee7e", "#f45b5b", "#7798BF",
        "#aaeeee", "#ff0066", "#eeaaee", "#55BF3B"
      ),
      chart = list(
        backgroundColor = list(
          linearGradient = list(x1 = 0, y1 = 0, x2 = 1, y2 = 1),
          stops = list(
            list(0, "#2a2a2b"),
            list(1, "#3e3e40")
          )
        ),
        style = list(
          fontFamily = "Unica One, sans-serif"
        ),
        plotBorderColor = "#606063"
      ),
      title = list(
        style = list(
          color = "#E0E0E3",
          textTransform = "uppercase",
          fontSize = "20px"
        )
      ),
      subtitle = list(
        style = list(
          color = "#E0E0E3",
          textTransform = "uppercase"
        )
      ),
      xAxis = list(
        gridLineColor = "#707073",
        labels = list(
          style = list(
            color = "#E0E0E3"
          )
        ),
        lineColor = "#707073",
        minorGridLineColor = "#505053",
        tickColor = "#707073",
        title = list(
          style = list(
            color = "#A0A0A3"
          )
        )
      ),
      yAxis = list(
        gridLineColor = "#707073",
        labels = list(
          style = list(
            color = "#E0E0E3"
          )
        ),
        lineColor = "#707073",
        minorGridLineColor = "#505053",
        tickColor = "#707073",
        tickWidth = 1,
        title = list(
          style = list(
            color = "#A0A0A3"
          )
        )
      ),
      tooltip = list(
        backgroundColor = "rgba(0, 0, 0, 0.85)",
        style = list(
          color = "#F0F0F0"
        )
      ),
      plotOptions = list(
        series = list(
          dataLabels = list(
            color = "#B0B0B3"
          ),
          marker = list(
            lineColor = "#333"
          )
        ),
        boxplot = list(
          fillColor = "#505053"
        ),
        candlestick = list(
          lineColor = "white"
        ),
        errorbar = list(
          color = "white"
        )
      ),
      legend = list(
        itemStyle = list(
          color = "#E0E0E3"
        ),
        itemHoverStyle = list(
          color = "#FFF"
        ),
        itemHiddenStyle = list(
          color = "#606063"
        )
      ),
      credits = list(
        style = list(
          color = "#666"
        )
      ),
      labels = list(
        style = list(
          color = "#707073"
        )
      ),
      drilldown = list(
        activeAxisLabelStyle = list(
          color = "#F0F0F3"
        ),
        activeDataLabelStyle = list(
          color = "#F0F0F3"
        )
      ),
      navigation = list(
        buttonOptions = list(
          symbolStroke = "#DDDDDD",
          theme = list(
            fill = "#505053"
          )
        )
      ),
      rangeSelector = list(
        buttonTheme = list(
          fill = "#505053",
          stroke = "#000000",
          style = list(
            color = "#CCC"
          ),
          states = list(
            hover = list(
              fill = "#707073",
              stroke = "#000000",
              style = list(
                color = "white"
              )
            ),
            select = list(
              fill = "#000003",
              stroke = "#000000",
              style = list(
                color = "white"
              )
            )
          )
        ),
        inputBoxBorderColor = "#505053",
        inputStyle = list(
          backgroundColor = "#333",
          color = "silver"
        ),
        labelStyle = list(
          color = "silver"
        )
      ),
      navigator = list(
        handles = list(
          backgroundColor = "#666",
          borderColor = "#AAA"
        ),
        outlineColor = "#CCC",
        maskFill = "rgba(255,255,255,0.1)",
        series = list(
          color = "#7798BF",
          lineColor = "#A6C7ED"
        ),
        xAxis = list(
          gridLineColor = "#505053"
        )
      ),
      scrollbar = list(
        barBackgroundColor = "#808083",
        barBorderColor = "#808083",
        buttonArrowColor = "#CCC",
        buttonBackgroundColor = "#606063",
        buttonBorderColor = "#606063",
        rifleColor = "#FFF",
        trackBackgroundColor = "#404043",
        trackBorderColor = "#404043"
      ),
      legendBackgroundColor = "rgba(0, 0, 0, 0.5)",
      background2 = "#505053",
      dataLabelsColor = "#B0B0B3",
      textColor = "#C0C0C0",
      contrastTextColor = "#F0F0F3",
      maskColor = "rgba(255,255,255,0.3)"
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

get_box_values <- function(x) {
  boxplot.stats(x)$stats %>%
    t() %>%
    as.data.frame() %>%
    setNames(c("low", "q1", "median", "q3", "high"))
}

get_outliers_values <- function(x) {
  boxplot.stats(x)$out
}

#' Helper to transform data frame for boxplot highcharts format
#'
#' @param data The data frame containing variables.
#' @param variable The variable to calculate the box plot data.
#' @param group_var A variable to split calculation
#' @param group_var2 A second variable to create separate series.
#' @param add_outliers A logical value indicating if outliers series should
#'   be calculated. Default to \code{FALSE}.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.
#'
#' @examples
#'
#' data(pokemon)
#'
#' dat <- data_to_boxplot(pokemon, height)
#'
#' highchart() %>%
#'   hc_xAxis(type = "category") %>%
#'   hc_add_series_list(dat)
#'
#' dat <- data_to_boxplot(pokemon, height, type_1, name = "height in meters")
#'
#' highchart() %>%
#'   hc_xAxis(type = "category") %>%
#'   hc_add_series_list(dat)
#' \dontrun{
#'
#' }
#'
#' @importFrom dplyr transmute group_nest
#' @export
data_to_boxplot <- function(data, variable, group_var = NULL, group_var2 = NULL, add_outliers = FALSE, ...) {
  stopifnot(
    is.data.frame(data),
    !missing(variable)
  )

  dx <- data %>%
    transmute(x := {{ variable }})

  if (!missing(group_var)) {
    dg <- data %>%
      select({{ group_var }})
  } else {
    dg <- data.frame(rep(0, nrow(dx)))
  }

  if (!missing(group_var2)) {
    dg2 <- data %>%
      select({{ group_var2 }})
  } else {
    dg2 <- data.frame(rep(NA, nrow(dx)))
  }

  dg <- dg %>% setNames("name")
  dg2 <- dg2 %>% setNames("series")

  dat <- bind_cols(dx, dg, dg2)

  dat1 <- dat %>%
    group_by(series, name) %>%
    summarise(data = list(get_box_values(x)), .groups = "drop") %>%
    unnest(cols = data) %>%
    group_nest(series) %>%
    mutate(data = map(data, list_parse)) %>%
    rename(name = series) %>%
    mutate(id = name) %>%
    mutate(type = "boxplot", ...)
  # list_parse()

  if (add_outliers) {
    dat2 <- dat %>%
      mutate(name = as.numeric(factor(name)) - 1) %>%
      group_by(series, name) %>%
      summarise(y = list(get_outliers_values(x)), .groups = "drop") %>%
      unnest(cols = y) %>%
      rename(x = name) %>%
      group_nest(series) %>%
      mutate(data = map(data, list_parse)) %>%
      rename(linkedTo = series) %>%
      mutate(type = "scatter", showInLegend = FALSE, ...)

    dout <- bind_rows(dat1, dat2)
  } else {
    dout <- dat1
  }

  dout
}



#' Helper to transform data frame for treemap/sunburst highcharts format
#'
#' @param data data frame containing variables to organize each level of
#'   the treemap.
#' @param group_vars Variables to generate treemap levels.
#' @param size_var Variable to aggregate.
#' @param colors Color to chart every item in the first level.
#'
#' @examples
#' \dontrun{
#'
#' library(dplyr)
#' data(gapminder, package = "gapminder")
#'
#' gapminder_2007 <- gapminder::gapminder %>%
#'   filter(year == max(year)) %>%
#'   mutate(pop_mm = round(pop / 1e6))
#'
#' dout <- data_to_hierarchical(gapminder_2007, c(continent, country), pop_mm)
#'
#' hchart(dout, type = "sunburst")
#'
#' hchart(dout, type = "treemap")
#' }
#'
#' @importFrom dplyr group_by_at arrange desc distinct rename_all everything
#' @importFrom tidyr unite
#' @importFrom purrr reduce
#' @export
data_to_hierarchical <- function(data, group_vars, size_var, colors = getOption("highcharter.color_palette")) {
  dat <- data %>%
    select({{ group_vars }}, {{ size_var }})

  ngvars <- ncol(dat) - 1

  names(dat) <- c(str_c("group_var_", seq(1, ngvars)), "value")

  gvars <- names(dat)[seq(1, ngvars)]

  # group to calculate the sum if there are duplicated combinations
  dat <- dat %>%
    group_by_at(all_of(gvars)) %>%
    summarise(value = sum(value, na.rm = TRUE), .groups = "drop") %>%
    ungroup() %>%
    mutate_if(is.factor, as.character) %>%
    arrange(desc(value))

  dout <- map(seq_along(gvars), function(depth = 1) {
    datg <- dat %>%
      select(1:depth) %>%
      distinct()

    datg_name <- datg %>%
      select(depth) %>%
      rename_all(~"name")

    datg_id <- datg %>%
      unite("id", everything(), sep = "_") %>%
      mutate(id = str_to_id_vec(id))

    datg_parent <- datg %>%
      select(1:(depth - 1)) %>%
      unite("parent", everything(), sep = "_") %>%
      mutate(parent = str_to_id(parent))

    dd <- list(datg_name, datg_id)

    # depth != 1 add parents
    if (depth != 1) dd <- dd %>% append(list(datg_parent))

    # depth == 1 add colors
    if (depth == 1 & !is.null(colors)) {
      dd <- dd %>% append(list(tibble(color = rep(colors, length.out = nrow(datg)))))
    }

    # depth == lastdepth add value
    if (depth == ngvars) dd <- dd %>% append(list(dat %>% select(value)))

    dd <- dd %>%
      bind_cols() %>%
      mutate(level = depth)

    dd

    list_parse(dd)
  })

  dout <- reduce(dout, c)

  dout
}

#' Helper to transform data frame for sankey highcharts format
#'
#' @param data A data frame
#'
#' @examples
#' \dontrun{
#' library(dplyr)
#' data(diamonds, package = "ggplot2")
#'
#' diamonds2 <- select(diamonds, cut, color, clarity)
#'
#' data_to_sankey(diamonds2)
#'
#' hchart(data_to_sankey(diamonds2), "sankey", name = "diamonds")
#' }
#'
#' @importFrom dplyr all_of count group_by_all vars
#' @importFrom stats complete.cases
#' @export
data_to_sankey <- function(data = NULL) {

  # numeric to categories
  if (any(unlist(purrr::map(data, class)) %in% c("integer", "numeric"))) {
    warning("numeric variables were treated with cut() function")
    data <- dplyr::mutate_if(data, function(x) is.numeric(x) && length(unique(x)) > 10, ~ cut(.x, breaks = c(-Inf, quantile(.x, c(2, 4, 6, 8) / 10), Inf)))
  }

  # combintaion data
  dcmb <- tibble::tibble(
    var1 = names(data),
    var2 = dplyr::lead(var1)
  ) %>%
    dplyr::filter(complete.cases(.))

  # sankey data
  dsnky <- purrr::pmap_df(dcmb, function(var1 = "cut", var2 = "color") {
    data %>%
      select(all_of(var1), all_of(var2)) %>%
      group_by_all() %>%
      count() %>%
      ungroup() %>%
      setNames(c("from", "to", "weight")) %>%
      mutate_if(is.factor, as.character) %>%
      mutate_at(vars(1, 2), as.character) %>%
      mutate(id := paste0(from, to))
  })

  dsnky
}

#' Alone theme for highcharts
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_alone())
#' @export
hc_theme_alone <- function(...) {
  theme <-
    hc_theme(
      colors = c(
        "#d35400", "#2980b9", "#2ecc71",
        "#f1c40f", "#2c3e50", "#7f8c8d"
      ),
      chart = list(
        backgroundColor = "#161C20",
        style = list(
          fontFamily = "Roboto",
          color = "#666666"
        )
      ),
      title = list(
        align = "left",
        style = list(
          fontFamily = "Roboto Condensed",
          fontWeight = "bold"
        )
      ),
      subtitle = list(
        align = "left",
        style = list(
          fontFamily = "Roboto Condensed"
        )
      ),
      legend = list(
        align = "right",
        verticalAlign = "bottom",
        itemStyle = list(
          color = "#424242"
        )
      ),
      xAxis = list(
        gridLineColor = "#424242",
        gridLineWidth = 1,
        minorGridLineColor = "#424242",
        minoGridLineWidth = 0.5,
        tickColor = "#424242",
        minorTickColor = "#424242",
        lineColor = "#424242"
      ),
      yAxis = list(
        gridLineColor = "#424242",
        ridLineWidth = 1,
        minorGridLineColor = "#424242",
        inoGridLineWidth = 0.5,
        tickColor = "#424242",
        minorTickColor = "#424242",
        lineColor = "#424242"
      ),
      plotOptions = list(
        line = list(
          marker = list(enabled = FALSE)
        ),
        spline = list(
          marker = list(enabled = FALSE)
        ),
        area = list(
          marker = list(enabled = FALSE)
        ),
        areaspline = list(
          marker = list(enabled = FALSE)
        ),
        arearange = list(
          marker = list(enabled = FALSE)
        ),
        bubble = list(
          maxSize = "10%"
        )
      )
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' Convert an object to list with identical structure
#'
#' This functions are similar to \code{rlist::list.parse} but this removes
#' names. \code{NA}s are removed for compatibility with rjson::toJSON.
#'
#' @param df A data frame to parse to list
#' @examples
#'
#' x <- data.frame(a = 1:3, type = c("A", "C", "B"), stringsAsFactors = FALSE)
#' list_parse(x)
#' list_parse2(x)
#' @importFrom purrr transpose
#' @export
list_parse <- function(df) {
  assertthat::assert_that(is.data.frame(df))

  if (getOption("highcharter.rjson")) {
    rows <- nrow(df)
    df <- tidyr::drop_na(df)
    if (getOption("highcharter.verbose") && (rows > nrow(df))) {
      warning("Removed ", (rows - nrow(df)), " rows with NA's.")
    }
  }

  purrr::map_if(df, is.factor, as.character) %>%
    as_tibble() %>%
    list.parse() %>%
    setNames(NULL)
}

#' @importFrom rlist list.parse
#' @rdname list_parse
#' @export
list_parse2 <- function(df) {
  assertthat::assert_that(is.data.frame(df))

  list_parse(df) %>%
    map(setNames, NULL)
}

#' String to 'id' format
#'
#' Turn a string to \code{id} format used in treemaps.
#'
#' @param x A vector string.
#'
#' @examples
#'
#' str_to_id(" A string _ with sd / sdg    Underscores \   ")
#' @export
str_to_id <- function(x) {
  assertthat::assert_that(is.character(x) | is.factor(x))

  x %>%
    as.character() %>%
    stringr::str_trim() %>%
    stringr::str_to_lower() %>%
    stringr::str_replace_all("\\s+", "_") %>%
    stringr::str_replace_all("\\\\|/", "_") %>%
    stringr::str_replace_all("\\[|\\]", "_") %>%
    stringr::str_replace_all("_+", "_") %>%
    stringr::str_replace_all("_$|^_", "")
}

#' @rdname str_to_id
str_to_id_vec <- function(x) {

  # x <- c("A_ aa", "A_  Aa", "a_   aa"

  tibble(
    var = x,
    id = str_to_id(x),
    un = cumsum(duplicated(id))
  ) %>%
    mutate(
      un = ifelse(un == 0, "", str_c("_", un)),
      id2 = str_c(id, un)
    ) %>%
    pull("id2")
}


#' Date to timestamps
#'
#' Turn a date time vector to \code{timestamp} format
#'
#' @param dt Date or datetime vector
#'
#' @examples
#'
#' datetime_to_timestamp(
#'   as.Date(c("2015-05-08", "2015-09-12"),
#'     format = "%Y-%m-%d"
#'   )
#' )
#' @export
datetime_to_timestamp <- function(dt) {

  # http://stackoverflow.com/questions/10160822/
  assertthat::assert_that(assertthat::is.date(dt) | assertthat::is.time(dt))

  tmstmp <- as.numeric(as.POSIXct(dt))

  tmstmp <- 1000 * tmstmp

  tmstmp
}

#' @rdname datetime_to_timestamp
#' @export
dt_tstp <- datetime_to_timestamp

#' Transform colors from hexadecimal format to rgba hc notation
#'
#' @param x colors in hexadecimal format
#' @param alpha alpha
#'
#' @examples
#'
#' hex_to_rgba(x <- c("#440154", "#21908C", "#FDE725"))
#' @importFrom grDevices col2rgb
#' @importFrom tidyr unite_
#' @export
hex_to_rgba <- function(x, alpha = 1) {
  x %>%
    col2rgb() %>%
    t() %>%
    as.data.frame() %>%
    unite_(col = "rgba", from = c("red", "green", "blue"), sep = ",") %>%
    .[[1]] %>%
    sprintf("rgba(%s,%s)", ., alpha)
}

#' Chart a demo for testing themes
#'
#' Chart a demo for testing themes
#'
#' @examples
#'
#' highcharts_demo()
#' @export
highcharts_demo <- function() {
  dtemp <- structure(
    list(
      month = c(
        "Jan", "Feb", "Mar", "Apr", "May", "Jun",
        "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
      ),
      tokyo = c(
        7, 6.9, 9.5, 14.5, 18.2, 21.5,
        25.2, 26.5, 23.3, 18.3, 13.9, 9.6
      ),
      new_york = c(
        -0.2, 0.8, 5.7, 11.3, 17, 22,
        24.8, 24.1, 20.1, 14.1, 8.6, 2.5
      ),
      berlin = c(
        -0.9, 0.6, 3.5, 8.4, 13.5, 17,
        18.6, 17.9, 14.3, 9, 3.9, 1
      ),
      london = c(
        3.9, 4.2, 5.7, 8.5, 11.9, 15.2,
        17, 16.6, 14.2, 10.3, 6.6, 4.8
      )
    ),
    .Names = c("month", "tokyo", "new_york", "berlin", "london"),
    row.names = c(NA, 12L), class = c("tbl_df", "tbl", "data.frame")
  )

  highchart() %>%
    hc_title(text = "Monthly Average Temperature") %>%
    hc_subtitle(text = "Source: WorldClimate.com") %>%
    hc_caption(text = "This is a caption text to show the style of this type of text") %>%
    hc_credits(text = "Made with highcharter", href = "http://jkunst.com/highcharter/", enabled = TRUE) %>%
    hc_yAxis(title = list(text = "Temperature")) %>%
    hc_xAxis(title = list(text = "Months")) %>%
    hc_xAxis(categories = dtemp$month) %>%
    hc_add_series(name = "Tokyo", data = dtemp$tokyo) %>%
    hc_add_series(name = "London", data = dtemp$london) %>%
    hc_add_series(name = "Berlin", data = dtemp$berlin)
}

#' Create vector of color from vector
#'
#' @param x A numeric, character or factor object.
#' @param colors A character string of colors (ordered) to colorize `x`
#' @examples
#'
#' colorize(runif(10))
#'
#' colorize(LETTERS[rbinom(20, 5, 0.5)], c("#FF0000", "#00FFFF"))
#' @importFrom grDevices colorRampPalette
#' @importFrom stats ecdf
#' @export
colorize <- function(x, colors = c("#440154", "#21908C", "#FDE725")) {
  nuniques <- length(unique(x))

  palcols <- grDevices::colorRampPalette(colors)(nuniques)

  if (!is.numeric(x) | nuniques < 10) {
    y <- as.numeric(as.factor(x))
    xcols <- palcols[y]
  } else {
    ecum <- ecdf(x)
    xcols <- palcols[ceiling(nuniques * ecum(x))]
  }

  xcols
}

#' Function to create \code{stops} argument in \code{hc_colorAxis}
#'
#' @param n A numeric indicating how much quantiles generate.
#' @param colors A character string of colors (ordered)
#'
#' @examples
#'
#' color_stops(5)
#' @export
color_stops <- function(n = 10, colors = c("#440154", "#21908C", "#FDE725")) {
  palcols <- grDevices::colorRampPalette(colors)(n)

  list_parse2(
    data.frame(
      q = seq(0, n - 1) / (n - 1),
      c = palcols
    )
  )
}

#' Function to create \code{dataClasses} argument in \code{hc_colorAxis}
#'
#' @param breaks A numeric vector
#' @param colors A character string of colors (ordered)
#'
#' @examples
#'
#' color_classes(c(0, 10, 20, 50))
#' @export
color_classes <- function(breaks = NULL,
                          colors = c("#440154", "#21908C", "#FDE725")) {
  lbrks <- length(breaks)

  list_parse(
    data.frame(
      from = breaks[-lbrks],
      to = breaks[-1],
      color = grDevices::colorRampPalette(colors)(lbrks - 1),
      stringsAsFactors = FALSE
    )
  )
}

#' Auxiliar function to get series and options from tidy frame for hchart.data.frame
#'
#' This function is used in \code{hchart.data.frame}.
#'
#' @param data A `data.frame` object.
#' @param type The type of chart. Possible values are line, scatter, point, column.
#' @param ... Aesthetic mappings as \code{x y group color low high}.
#'
#' @examples
#'
#' highcharter:::get_hc_series_from_df(iris, type = "point", x = Sepal.Width)
#' 
#' @importFrom tibble has_name
#' @importFrom rlang .data
get_hc_series_from_df <- function(data, type = NULL, ...) {
  assertthat::assert_that(is.data.frame(data))
  stopifnot(!is.null(type))

  pars <- eval(substitute(alist(...)))
  parsc <- map(pars, as.character)

  data <- mutate(data, ...)
  data <- ungroup(data)

  # check type
  type <- ifelse(type == "point", "scatter", type)
  type <- ifelse("size" %in% names(data) & type == "scatter", "bubble", type)

  # heatmap
  if (type == "heatmap") {
    if (!is.numeric(data[["x"]])) {
      data[["xf"]] <- as.factor(data[["x"]])
      data[["x"]] <- as.numeric(as.factor(data[["x"]])) - 1
    }
    if (!is.numeric(data[["y"]])) {
      data[["yf"]] <- as.factor(data[["y"]])
      data[["y"]] <- as.numeric(as.factor(data[["y"]])) - 1
    }
  }

  # x
  if (has_name(parsc, "x")) {
    if (is.Date(data[["x"]])) {
      data[["x"]] <- datetime_to_timestamp(data[["x"]])
    } else if (is.character(data[["x"]]) | is.factor(data[["x"]])) {
      data[["name"]] <- data[["x"]]
      data[["x"]] <- NULL
    }
  }

  # color
  if (has_name(parsc, "color")) {
    if (type == "treemap") {
      data <- rename(data, colorValue = .data$color)
    } else {
      data <- mutate(data,
        colorv = .data$color,
        color = highcharter::colorize(.data$color)
      )
    }
  }

  # size
  if (has_name(parsc, "size") & type %in% c("bubble", "scatter")) {
    data <- mutate(data, z = .data$size)
  }

  # group
  if (!has_name(parsc, "group")) {
    data[["group"]] <- "Series"
  }

  data[["charttpye"]] <- type

  dfs <- data %>%
    group_by(.data$group, .data$charttpye) %>%
    do(data = list_parse(select(., -.data$group, -.data$charttpye))) %>%
    ungroup() %>%
    rename(name = .data$group, type = .data$charttpye)

  if (!has_name(parsc, "group")) {
    dfs[["name"]] <- NULL
  }

  series <- list_parse(dfs)

  series
}

#' Check if a string vector is in hexadecimal color format
#'
#' @param x A string vectors
#'
#' @examples
#'
#' x <- c("#f0f0f0", "#FFf", "#99990000", "#00FFFFFF")
#'
#' is.hexcolor(x)
#' @export
is.hexcolor <- function(x) {
  pattern <- "^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3}|[A-Fa-f0-9]{8})$"

  str_detect(x, pattern)
}

#' Function to generate iids
#' @param n Number of ids
#' @param length Length of ids
random_id <- function(n = 1, length = 10) {
  source <- c(seq(0, 9), letters)

  replicate(n, paste0(sample(x = source, size = length, replace = TRUE), collapse = ""))
}

#' Function to avoid the jsonlite::auto_unbox default
#' @param x And element, numeric or character
#' @examples
#'
#' hchart("A")
#'
#' highchart() %>%
#'   hc_add_series(data = 1)
#' @noRd
fix_1_length_data <- function(x) {
  if (getOption("highcharter.verbose")) {
    message("fix_1_length")
  }

  if (!is.list(x) && length(x) == 1) {
    x <- list(x)
  }
  x
}

#' Function to create annotations arguments from a data frame
#' @param df A data frame with `x`, `y` and `text` columns names.
#' @param xAxis Index (js 0-based) of the x axis to put the annotations.
#' @param yAxis Index (js 0-based) of the y axis to put the annotations.
#' @examples
#'
#' df <- data.frame(text = c("hi", "bye"), x = c(0, 1), y = c(1, 0))
#'
#' df_to_annotations_labels(df)
#' @importFrom utils hasName
#' @importFrom dplyr rowwise
#' @export
df_to_annotations_labels <- function(df, xAxis = 0, yAxis = 0) {
  stopifnot(hasName(df, "x"))
  stopifnot(hasName(df, "y"))
  stopifnot(hasName(df, "text"))

  df %>%
    rowwise() %>%
    mutate(point = list(list(x = x, y = y, xAxis = 0, yAxis = 0))) %>%
    select(-x, -y) %>%
    list_parse()
}

#' Helpers functions to get FontAwesome icons code
#' @param hc A `highchart` `htmlwidget` object.
#' @export
hc_add_dependency_fa <- function(hc) {
  .Deprecated(msg = "Deprecated function. Use the fontawesome package. Example in the docs website.")
  dep <- htmlDependency(
    name = "font-awesome",
    version = "0.0.0",
    src = c(file = system.file("htmlwidgets/lib/font-awesome-4.7.0", package = "highcharter")),
    stylesheet = "css/font-awesome.min.css"
  )

  hc$dependencies <- c(hc$dependencies, list(dep))

  hc
}

#' @rdname hc_add_dependency_fa
#' @param iconname The icon's name
#' @export
fa_icon <- function(iconname = "circle") {
  .Deprecated(msg = "Deprecated function. Use the fontawesome package. Example in the docs website.")
  sprintf("<i class=\"fa fa-%s\"></i>", iconname)
}

#' @rdname hc_add_dependency_fa
#' @export
fa_icon_mark <- function(iconname = "circle") {
  .Deprecated(msg = "Deprecated function. Use the fontawesome package. Example in the docs website.")

  faicos <- readRDS(system.file("extdata/faicos.rds", package = "highcharter"))

  idx <- purrr::map_int(iconname, function(icn) which(faicos$name %in% icn))

  cod <- faicos$code[idx]

  # this is for the plugin: need the text:code to parse
  paste0("text:", cod)
}

#' Function to export js file the configuration options
#'
#' @param hc A \code{Highcharts object}.
#' @param filename String of the exported file.
#' @param as String to define how to save the configuration options.
#'   One of 'is', 'container', 'variable'.
#' @param name A variable used to put as name of the generated object if
#'   \code{as} is \code{'variable'} and the css/js selector if is \code{as} is
#'   container.
#'
#' @examples
#'
#' fn <- "function(){
#'   console.log('Category: ' + this.category);
#'   alert('Category: ' + this.category);
#' }"
#'
#' hc <- highcharts_demo() %>%
#'   hc_plotOptions(
#'     series = list(
#'       cursor = "pointer",
#'       point = list(
#'         events = list(
#'           click = JS(fn)
#'         )
#'       )
#'     )
#'   )
#' \dontrun{
#' export_hc(hc, filename = "~/hc_is.js", as = "is")
#' export_hc(hc, filename = "~/hc_vr.js", as = "variable", name = "objectname")
#' export_hc(hc, filename = "~/hc_ct.js", as = "container", name = "#selectorid")
#' }
#'
#' @importFrom jsonlite toJSON
#' @importFrom utils head
#' @export
export_hc <- function(hc, filename = NULL, as = "is", name = NULL) {
  stopifnot(!is.null(filename))
  stopifnot(as %in% c("is", "container", "variable"))

  if (as != "is") {
    stopifnot(!is.null(name))
  }

  JS_to_json <- function(x) {
    class(x) <- "json"
    return(x)
  }

  hc$x$hc_opts <- rapply(
    object = hc$x$hc_opts,
    f = JS_to_json,
    classes = "JS_EVAL",
    how = "replace"
  )

  js <- toJSON(
    x = hc$x$hc_opts, pretty = TRUE, auto_unbox = TRUE, json_verbatim = TRUE,
    force = TRUE, null = "null", na = "null"
  )

  if (as == "container") {
    js <- sprintf("$(function(){\n\t$('%s').highcharts(\n%s\n);\n});", name, js)
  } else if (as == "variable") {
    js <- sprintf("var %s = %s;", name, js)
  }

  if (!str_detect(filename, ".js$")) {
    filename <- str_c(filename, ".js")
  }

  if (getOption("highcharter.verbose")) {
    message(sprintf("saving file in '%s'", filename))
  }

  writeLines(js, filename)
}

#' Google theme for highcharts
#'
#' Google theme for highcharts is based on \url{https://books.google.com/ngrams/}.
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_google())
#' @export
hc_theme_google <- function(...) {
  theme <-
    list(
      colors = c("#0266C8", "#F90101", "#F2B50F", "#00933B"),
      chart = list(
        style = list(
          fontFamily = "Roboto",
          color = "#444444"
        )
      ),
      xAxis = list(
        gridLineWidth = 1,
        gridLineColor = "#F3F3F3",
        lineColor = "#F3F3F3",
        minorGridLineColor = "#F3F3F3",
        tickColor = "#F3F3F3",
        tickWidth = 1
      ),
      yAxis = list(
        gridLineColor = "#F3F3F3",
        lineColor = "#F3F3F3",
        minorGridLineColor = "#F3F3F3",
        tickColor = "#F3F3F3",
        tickWidth = 1
      ),
      legendBackgroundColor = "rgba(0, 0, 0, 0.5)",
      background2 = "#505053",
      dataLabelsColor = "#B0B0B3",
      textColor = "#C0C0C0",
      contrastTextColor = "#F0F0F3",
      maskColor = "rgba(255,255,255,0.3)"
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' Flat theme for highcharts
#'
#' Flat and flatdark theme is inspired by
#' \url{https://github.com/chriskempson/base16} and
#' \url{https://github.com/Mikata-Project/ggthemr#flat}
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_flat())
#' @export
hc_theme_flat <- function(...) {
  theme <-
    list(
      colors = c(
        "#f1c40f", "#2ecc71", "#9b59b6", "#e74c3c",
        "#34495e", "#3498db", "#1abc9c", "#f39c12", "#d35400"
      ),
      chart = list(
        backgroundColor = "#ECF0F1"
      ),
      xAxis = list(
        gridLineDashStyle = "Dash",
        gridLineWidth = 1,
        gridLineColor = "#BDC3C7",
        lineColor = "#BDC3C7",
        minorGridLineColor = "#BDC3C7",
        tickColor = "#BDC3C7",
        tickWidth = 1
      ),
      yAxis = list(
        gridLineDashStyle = "Dash",
        gridLineColor = "#BDC3C7",
        lineColor = "#BDC3C7",
        minorGridLineColor = "#BDC3C7",
        tickColor = "#BDC3C7",
        tickWidth = 1
      ),
      legendBackgroundColor = "rgba(0, 0, 0, 0.5)",
      background2 = "#505053",
      dataLabelsColor = "#B0B0B3",
      textColor = "#34495e",
      contrastTextColor = "#F0F0F3",
      maskColor = "rgba(255,255,255,0.3)"
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' Highcharter theme for highcharts
#'
#' hcrt theme is used for the documentation website.
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_hcrt())
#' @export
hc_theme_hcrt <- function(...) {
  theme <-
    hc_theme(
      colors = c(
        "#47475c", # main purple

        "#61BC7B", # green
        "#508CC8", # blue
        "#F49952", # orange
        "#9C9EDB", # purple

        "#6699a1" # gray green
      ),
      chart = list(
        style = list(
          fontFamily = "IBM Plex Sans",
          color = "#666666"
        )
      ),
      title = list(
        align = "left",
        style = list(
          fontFamily = "Alegreya Sans SC",
          fontSize = "24px"
        )
      ),
      subtitle = list(
        align = "left",
        style = list(
          fontFamily = "Alegreya Sans",
          fontSize = "16px"
        )
      ),
      caption = list(
        style = list(
          fontFamily = "Alegreya Sans",
          fontSize = "14px"
        )
      ),
      credits = list(
        style = list(
          fontFamily = "Roboto",
          fontSize = "10px"
        )
      ),
      legend = list(
        align = "right",
        verticalAlign = "bottom",
        itemStyle = list(
          fontWeight = "normal",
          color = "#A2A39C"
        )
      ),
      xAxis = list(
        gridLineWidth = 1,
        gridLineColor = "#F3F3F3",
        lineColor = "#F3F3F3",
        minorGridLineColor = "#F3F3F3",
        tickColor = "#F3F3F3",
        tickWidth = 1
      ),
      yAxis = list(
        gridLineColor = "#F3F3F3",
        lineColor = "#F3F3F3",
        minorGridLineColor = "#F3F3F3",
        tickColor = "#F3F3F3",
        tickWidth = 1
      ),
      plotOptions = list(
        line = list(
          marker = list(enabled = FALSE)
        ),
        spline = list(
          marker = list(enabled = FALSE)
        ),
        area = list(
          marker = list(enabled = FALSE)
        ),
        areaspline = list(
          marker = list(enabled = FALSE)
        ),
        arearange = list(
          marker = list(enabled = FALSE)
        ),
        bubble = list(
          maxSize = "10%"
        )
      )
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' Monokai theme for highcharts
#'
#' Monokai is a well know text editor theme.
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_monokai())
#' @export
hc_theme_monokai <- function(...) {
  theme <-
    hc_theme(
      colors = c("#F92672", "#66D9EF", "#A6E22E", "#A6E22E"),
      chart = list(
        backgroundColor = "#272822",
        style = list(
          fontFamily = "Inconsolata",
          color = "#A2A39C"
        )
      ),
      title = list(
        style = list(
          color = "#A2A39C"
        ),
        align = "left"
      ),
      subtitle = list(
        style = list(
          color = "#A2A39C"
        ),
        align = "left"
      ),
      legend = list(
        align = "right",
        verticalAlign = "bottom",
        itemStyle = list(
          fontWeight = "normal",
          color = "#A2A39C"
        )
      ),
      xAxis = list(
        gridLineDashStyle = "Dot",
        gridLineWidth = 1,
        gridLineColor = "#A2A39C",
        lineColor = "#A2A39C",
        minorGridLineColor = "#A2A39C",
        tickColor = "#A2A39C",
        tickWidth = 1
      ),
      yAxis = list(
        gridLineDashStyle = "Dot",
        gridLineColor = "#A2A39C",
        lineColor = "#A2A39C",
        minorGridLineColor = "#A2A39C",
        tickColor = "#A2A39C",
        tickWidth = 1
      )
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' Lays out highchart widgets into a "grid", similar to
#' \code{grid.arrange} from \code{gridExtra}.
#'
#' @param ... either individual `highchart` objects or a mixture of
#'   individual `highchart` objects and `list`s of `highchart` objects.
#' @param ncol how many columns in the grid
#' @param rowheight Height in px.
#' @param add_htmlgrid_css A logical value to add or not `htmlgrid.css` as
#'   dependency.
#' @param browsable Logical value indicating if the returned object is converted
#'   to an HTML object browsable using \code{htmltools::browsable}.
#'
#' @examples
#'
#' charts <- lapply(1:9, function(x) {
#'   hchart(ts(cumsum(rnorm(100))))
#' })
#'
#' if (interactive()) {
#'   hw_grid(charts, rowheight = 300)
#' }
#' @importFrom grDevices n2mfrow
#' @export
hw_grid <- function(..., ncol = NULL, rowheight = NULL, add_htmlgrid_css = TRUE, browsable = TRUE) {
  input_list <- as.list(substitute(list(...)))[-1L]

  params <- list()

  for (i in seq_len(length(input_list))) {
    x <- eval.parent(input_list[[i]])
    if (any(class(x) == "list")) {
      for (j in seq_len(length(x))) {
        y <- eval(x[[j]])
        params[[length(params) + 1]] <- y
      }
    } else {
      params[[length(params) + 1]] <- x
    }
  }

  if (!all(sapply(params, function(x) inherits(x, "htmlwidget")))) {
    stop("All parameters must be htmlwidget objects")
  }


  if (is.null(ncol)) {
    ncol <- n2mfrow(length(params))[1]
  }

  if (ncol > 12) {
    ncol <- 12
  }

  ncolm <- floor(ncol / 2)

  # adding htmlwdgtgrid.css dependencies
  dep <- htmlDependency(
    name = "htmlwdgtgrid",
    version = "0.0.9",
    src = c(file = system.file("htmlwidgets/lib/highcharts/css", package = "highcharter")),
    stylesheet = "htmlwdgtgrid.css",
  )

  divs <- map(params, function(x) {
    if (add_htmlgrid_css) {
      x$dependencies <- c(x$dependencies, list(dep))
    }

    x$width <- "100%"

    if (!is.null(rowheight)) {
      x$height <- rowheight
    }

    tags$div(class = sprintf("col-1-%s mobile-col-1-%s", ncol, ncolm), x)
  })

  divgrid <- tags$div(class = "grid grid-pad", divs)

  class(divgrid) <- c(class(divgrid), "htmlwdwtgrid")

  if (browsable) {
    return(htmltools::browsable(divgrid))
  }

  divgrid
}

#' Adding data to highchart objects
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param data An R object like numeric, list, ts, xts, etc.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.
#' @examples
#'
#' highchart() %>%
#'   hc_add_series(data = abs(rnorm(5)), type = "column") %>%
#'   hc_add_series(data = purrr::map(0:4, function(x) list(x, x)), type = "scatter", color = "orange")
#' @export
hc_add_series <- function(hc, data = NULL, ...) {

  # assertthat::assert_that(is.highchart(hc))

  UseMethod("hc_add_series", data)
}


#' @export
hc_add_series.default <- function(hc, ...) {

  # assertthat::assert_that(is.highchart(hc))

  if (getOption("highcharter.verbose")) {
    message("hc_add_series.default")
  }

  validate_args("add_series", eval(substitute(alist(...))))

  hc$x$hc_opts$series <- append(hc$x$hc_opts$series, list(list(...)))

  hc
}


#' `hc_add_series` for numeric objects
#' @param hc A `highchart` `htmlwidget` object.
#' @param data A numeric object
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.
#' @export
hc_add_series.numeric <- function(hc, data, ...) {
  if (getOption("highcharter.verbose")) {
    message("hc_add_series.numeric")
  }

  data <- fix_1_length_data(data)

  hc_add_series.default(hc, data = data, ...)
}


#' hc_add_series for time series objects
#' @param hc A `highchart` `htmlwidget` object.
#' @param data A time series `ts` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.
#' @importFrom zoo as.Date
#' @importFrom stats time
#' @export
hc_add_series.ts <- function(hc, data, ...) {
  if (getOption("highcharter.verbose")) {
    message("hc_add_series.ts")
  }

  # http://stackoverflow.com/questions/29202021/
  timestamps <- data %>%
    stats::time() %>%
    zoo::as.Date() %>%
    datetime_to_timestamp()

  series <- list_parse2(data.frame(timestamps, as.vector(data)))

  hc_add_series(hc, data = series, ...)
}


#' hc_add_series for xts objects
#' @param hc A `highchart` `htmlwidget` object.
#' @param data A `xts` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.
#' @importFrom xts is.xts
#' @importFrom quantmod is.OHLC
#' @export
hc_add_series.xts <- function(hc, data, ...) {
  if (getOption("highcharter.verbose")) {
    message("hc_add_series.xts")
  }

  if (is.OHLC(data)) {
    return(hc_add_series.ohlc(hc, data, ...))
  }

  timestamps <- datetime_to_timestamp(time(data))

  series <- list_parse2(data.frame(timestamps, as.vector(data)))

  hc_add_series(hc, data = series, ...)
}


#' @rdname hc_add_series.xts
#' @param type The way to show the `xts` object. Can be 'candlestick' or 'ohlc'.
#' @export
hc_add_series.ohlc <- function(hc, data, type = "candlestick", ...) {
  if (getOption("highcharter.verbose")) {
    message("hc_add_series.xts.ohlc")
  }

  time <- datetime_to_timestamp(time(data))
  xdf <- cbind(time, as.data.frame(
    quantmod::OHLC(data)
  ))
  xds <- list_parse2(xdf)

  nm <- ifelse(!is.null(list(...)[["name"]]),
    list(...)[["name"]],
    str_extract(names(data)[1], "^[A-Za-z]+")
  )

  hc_add_series(hc, data = xds, name = nm, type = type, ...)
}


#' hc_add_series for forecast objects
#' @param hc A `highchart` `htmlwidget` object.
#' @param data A `forecast` object.
#' @param addOriginal Logical value to add the original series or not.
#' @param addLevels Logical value to show predictions bands.
#' @param fillOpacity The opacity of bands.
#' @param name The name of the series.
#' @param ... Arguments defined in
#'   \url{https://api.highcharts.com/highcharts/chart}.
#' @export
hc_add_series.forecast <- function(hc, data, addOriginal = FALSE,
                                   addLevels = TRUE, fillOpacity = 0.1, name = NULL, ...) {
  if (getOption("highcharter.verbose")) {
    message("hc_add_series.forecast")
  }

  rid <- random_id()
  method <- data$method

  if (addOriginal) {
    hc <- hc_add_series(hc, data$x, name = ifelse(is.null(name), method, name), zIndex = 3, ...)
  }


  hc <- hc_add_series(hc, data$mean, name = ifelse(is.null(name), method, name), zIndex = 2, id = rid, ...)

  if (addLevels) {
    tmf <- datetime_to_timestamp(zoo::as.Date(time(data$mean)))
    nmf <- paste(ifelse(is.null(name), method, name), "level", data$level)

    for (m in seq(ncol(data$upper))) {
      dfbands <- tibble(
        t = tmf,
        l = as.vector(data$lower[, m]),
        u = as.vector(data$upper[, m])
      )

      hc <- hc %>%
        hc_add_series(
          data = list_parse2(dfbands),
          name = nmf[m],
          type = "arearange",
          fillOpacity = fillOpacity,
          zIndex = 1,
          lineWidth = 0,
          linkedTo = rid,
          ...
        )
    }
  }


  hc
}


#' hc_add_series for density objects
#' @param hc A `highchart` `htmlwidget` object.
#' @param data A `density` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.
#' @export
hc_add_series.density <- function(hc, data, ...) {
  if (getOption("highcharter.verbose")) {
    message("hc_add_series.density")
  }

  data <- list_parse(data.frame(cbind(x = data$x, y = data$y)))

  hc_add_series(hc, data = data, ...)
}


#' hc_add_series for character and factor objects
#' @param hc A `highchart` `htmlwidget` object.
#' @param data A \code{character} or \code{factor} object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.
#' @export
hc_add_series.character <- function(hc, data, ...) {
  if (getOption("highcharter.verbose")) {
    message("hc_add_series.character")
  }

  series <- data %>%
    table() %>%
    as.data.frame(stringsAsFactors = FALSE) %>%
    setNames(c("name", "y")) %>%
    list_parse()

  hc_add_series(hc, data = series, ...)
}


#' @rdname hc_add_series.character
#' @export
hc_add_series.factor <- hc_add_series.character

#' hc_add_series for geo_json & geo_list objects
#' @param hc A `highchart` `htmlwidget` object.
#' @param data A \code{geo_json} or \code{geo_list} object.
#' @param type Type of series. Can be 'mapline', 'mapoint'.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.
#' @export
hc_add_series.geo_json <- function(hc, data, type = NULL, ...) {
  if (getOption("highcharter.verbose")) {
    message("hc_add_series.geo_json")
  }

  stopifnot(hc$x$type == "map", !is.null(type))

  hc_add_series.default(hc, data = data, geojson = TRUE, type = type, ...)
}

#' @rdname hc_add_series.geo_json
#' @export
hc_add_series.geo_list <- function(hc, data, type = NULL, ...) {
  if (getOption("highcharter.verbose")) {
    message("hc_add_series.geo_list")
  }

  stopifnot(hc$x$type == "map", !is.null(type))

  hc_add_series.default(hc, data = data, geojson = TRUE, type = type, ...)
}

#' hc_add_series for lm and loess objects
#' @param hc A `highchart` `htmlwidget` object.
#' @param data A \code{lm} or \code{loess} object.
#' @param type The type of the series: line, spline.
#' @param color A stringr color.
#' @param fillOpacity fillOpacity to the confidence interval.
#' @param ... Arguments defined in
#'   \url{https://api.highcharts.com/highcharts/chart}.
#' @importFrom broom augment
#' @importFrom rlang .data
#' @export
hc_add_series.lm <- function(hc, data, type = "line", color = "#5F83EE", fillOpacity = 0.1, ...) {
  if (getOption("highcharter.verbose")) {
    message(sprintf("hc_add_series.%s", class(data)))
  }

  data2 <- data %>%
    augment() %>%
    as.matrix.data.frame() %>%
    as.data.frame() %>%
    tibble::as_tibble()
  data2 <- arrange_(data2, .dots = names(data2)[2])
  data2 <- mutate(data2, x = names(data2)[2])
  data2 <- select(data2, !!!c(names(data2)[1]), .data$x, .data$.fitted, .data$.se.fit)

  rid <- random_id()

  hc %>%
    hc_add_series(data2,
      type = type, hcaes_("x", ".fitted"),
      id = rid, color = color, ...
    ) %>%
    hc_add_series(data2,
      type = "arearange",
      hcaes_("x",
        low = ".fitted - 2 * .se.fit",
        high = ".fitted + 2 * .se.fit"
      ),
      color = hex_to_rgba(color, fillOpacity),
      linkedTo = rid, zIndex = -2, ...
    )
}

#' @rdname hc_add_series.lm
#' @export
hc_add_series.loess <- hc_add_series.lm

#' hc_add_series for data frames objects
#' @param hc A `highchart` `htmlwidget` object.
#' @param data A `data.frame` object.
#' @param type The type of the series: line, bar, etc.
#' @param mapping The mapping, same idea as \code{ggplot2}.
#' @param fast convert to json during the composition of a highchart object
#' @param ... Arguments defined in
#'   \url{https://api.highcharts.com/highcharts/chart}.
#' @importFrom rlang .data
#' @export
hc_add_series.data.frame <- function(hc, data, type = NULL, mapping = hcaes(), fast = FALSE, ...) {
  if (getOption("highcharter.verbose")) {
    message("hc_add_series.data.frame")
  }

  if (length(mapping) == 0) {
    if (has_name(data, "series")) {
      data <- rename(data, seriess = .data$series)
    }

    return(hc_add_series(hc, data = list_parse(data), type = type, ...))
  }

  data <- mutate_mapping(data, mapping)

  series <- data_to_series(data, mapping, type = type, fast = fast, ...)

  hc_add_series_list(hc, series)
}

#' Define aesthetic mappings.
#' Similar in spirit to \code{ggplot2::aes}
#' @param x,y,... List of name value pairs giving aesthetics to map to
#'   variables. The names for x and y aesthetics are typically omitted because
#'   they are so common; all other aesthetics must be named.
#' @importFrom rlang enexprs is_missing
#' @examples
#'
#' hcaes(x = xval, color = colorvar, group = grvar)
#' @export
hcaes <- function(x, y, ...) {
  # taken from https://github.com/tidyverse/ggplot2/commit/d69762269787ed0799ab4fb1f35638cc46b5b7e6
  exprs <- rlang::enexprs(x = x, y = y, ...)

  is_missing <- vapply(exprs, rlang::is_missing, logical(1))

  mapping <- structure(exprs[!is_missing], class = "uneval")

  class(mapping) <- c("hcaes", class(mapping))

  mapping
}

#' Define aesthetic mappings using strings.
#' Similar in spirit to \code{ggplot2::aes_string}
#' @param x,y,... List of name value pairs giving aesthetics to map to
#'   variables. The names for x and y aesthetics are typically omitted because
#'   they are so common; all other aesthetics must be named.
#' @examples
#' hchart(mtcars, "point", hcaes_string("hp", "mpg", group = "cyl"))
#'
#' hcaes_string(x = "xval", color = "colorvar", group = "grvar")
#' @export
hcaes_string <- function(x, y, ...) {
  mapping <- list(...)

  if (!missing(x)) {
    mapping["x"] <- list(x)
  }

  if (!missing(y)) {
    mapping["y"] <- list(y)
  }

  mapping <- lapply(mapping, function(x) {
    if (is.character(x)) {
      parse(text = x)[[1]]
    }
    else {
      x
    }
  })

  mapping <- structure(mapping, class = "uneval")

  mapping <- mapping[names(mapping) != ""]

  class(mapping) <- c("hcaes", class(mapping))

  mapping
}

#' @rdname hcaes_string
#' @export
hcaes_ <- hcaes_string


#' Modify data frame according to mapping
#' @param data A data frame object.
#' @param mapping A mapping from \code{hcaes} function.
#' @param drop A logical argument to you drop variables or not. Default is
#' \code{FALSE}
#' @importFrom lubridate is.Date
#' @importFrom rlang "!!!" "!!" ":=" parse_quo syms .data
#' @examples
#'
#' df <- head(mtcars)
#' mutate_mapping(data = df, mapping = hcaes(x = cyl, y = wt + cyl, group = gear))
#' mutate_mapping(data = df, mapping = hcaes(x = cyl, y = wt), drop = TRUE)
#' @export
mutate_mapping <- function(data, mapping, drop = FALSE) {
  stopifnot(is.data.frame(data), inherits(mapping, "hcaes"), inherits(drop, "logical"))

  # https://stackoverflow.com/questions/45359917/dplyr-0-7-equivalent-for-deprecated-mutate
  # https://www.johnmackintosh.com/2018-02-19-theory-free-tidyeval/

  tran <- as.character(mapping)
  newv <- names(mapping)
  list_names <- setNames(tran, newv) %>% lapply(rlang::parse_quo, env = globalenv())

  data <- dplyr::mutate(data, !!!list_names)
  # Reserverd  highcharts names (#241)
  if (has_name(data, "series")) {
    data <- dplyr::rename(data, seriess = .data$series)
  }

  if (drop) {
    newv <- rlang::syms(newv)
    data <- dplyr::select(data, !!!newv)
  }

  data
}

add_arg_to_df <- function(data, ...) {
  datal <- as.list(data)

  l <- map_if(list(...), function(x) is.list(x), list)

  datal <- append(datal, l)

  as_tibble(datal)
}

#' @importFrom dplyr mutate do arrange_
#' @importFrom tibble tibble tibble_
data_to_series <- function(data, mapping, type, fast = FALSE, ...) {

  # check type and fix
  type <- ifelse(type == "point", "scatter", type)
  type <- ifelse((has_name(mapping, "size") | has_name(mapping, "z")) & type == "scatter",
    "bubble", type
  )

  # heatmap
  if (type == "heatmap") {
    if (!is.numeric(data[["x"]])) {
      data[["xf"]] <- as.factor(data[["x"]])
      data[["x"]] <- as.numeric(as.factor(data[["x"]])) - 1
    }
    if (!is.numeric(data[["y"]])) {
      data[["yf"]] <- as.factor(data[["y"]])
      data[["y"]] <- as.numeric(as.factor(data[["y"]])) - 1
    }
  }

  # x
  if (has_name(mapping, "x")) {
    if (is.Date(data[["x"]])) {
      data[["x"]] <- datetime_to_timestamp(data[["x"]])
    } else if (is.character(data[["x"]]) | is.factor(data[["x"]])) {
      data[["name"]] <- data[["x"]]
      data[["x"]] <- NULL
    }
  }

  # color
  if (has_name(mapping, "color")) {
    if (type == "treemap") {
      data <- rename(data, colorValue = .data$color)
    } else if (!all(is.hexcolor(data[["color"]]))) {
      data <- mutate(data, colorv = .data$color, color = highcharter::colorize(.data$color))
    }
  } else if (has_name(data, "color")) {
    data <- rename(data, colorv = .data$color)
  }

  # size
  if (type %in% c("bubble", "scatter")) {
    if (has_name(mapping, "size")) {
      data <- mutate(data, z = .data$size)
    }
  }

  # group
  if (!has_name(mapping, "group")) {
    data[["group"]] <- "group"
  }

  if (isTRUE(fast)) {
    # pre-convert data to json
    data <- data %>%
      group_by(.data$group) %>%
      do(data = jsonlite::toJSON(select(., -.data$group),
        dataframe = "rows",
        verbatim = TRUE
      )) %>%
      ungroup()
  } else {
    data <- data %>%
      group_by(.data$group) %>%
      do(data = list_parse(select(., -.data$group))) %>%
      ungroup()
  }

  data$type <- type

  if (length(list(...)) > 0) {
    data <- add_arg_to_df(data, ...)
  }

  if (has_name(mapping, "group") & !has_name(list(...), "name")) {
    data <- rename(data, name = .data$group)
  }

  series <- list_parse(data)

  series
}

data_to_options <- function(data, type) {
  opts <- list()

  # x
  if (has_name(data, "x")) {
    if (is.Date(data[["x"]])) {
      opts$xAxis_type <- "datetime"
    } else if (is.character(data[["x"]]) | is.factor(data[["x"]])) {
      opts$xAxis_type <- "category"
    } else {
      opts$xAxis_type <- "linear"
    }
  }

  # y
  if (has_name(data, "x")) {
    if (is.Date(data[["y"]])) {
      opts$yAxis_type <- "datetime"
    } else if (is.character(data[["y"]]) | is.factor(data[["y"]])) {
      opts$yAxis_type <- "category"
    } else {
      opts$yAxis_type <- "linear"
    }
  }

  # showInLegend
  opts$series_plotOptions_showInLegend <- "group" %in% names(data)

  # colorAxis
  opts$add_colorAxis <-
    (type == "treemap" & "color" %in% names(data)) | (type == "heatmap")

  # heatmap
  if (type == "heatmap") {
    if (!is.numeric(data[["x"]])) {
      opts$xAxis_categories <- levels(as.factor(data[["x"]]))
    }
    if (!is.numeric(data[["y"]])) {
      opts$yAxis_categories <- levels(as.factor(data[["y"]]))
    }
  }

  opts
}


#' Removing series to highchart objects
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param names The series's names to delete.
#'
#' @export
hc_rm_series <- function(hc, names = NULL) {
  stopifnot(!is.null(names))

  positions <- hc$x$hc_opts$series %>%
    map("name") %>%
    unlist()

  position <- which(positions %in% names)

  hc$x$hc_opts$series[position] <- NULL

  hc
}

#' Shortcut for data series from a list of data series
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param x A `list` or a `data.frame` of series.
#'
#' @examples
#'
#' ds <- lapply(seq(5), function(x) {
#'   list(data = cumsum(rnorm(100, 2, 5)), name = x)
#' })
#'
#' highchart() %>%
#'   hc_plotOptions(series = list(marker = list(enabled = FALSE))) %>%
#'   hc_add_series_list(ds)
#' @export
hc_add_series_list <- function(hc, x) {
  assertthat::assert_that(is.highchart(hc), (is.list(x) | is.data.frame(x)))

  if (is.data.frame(x)) {
    x <- list_parse(x)
  }

  hc$x$hc_opts$series <- append(hc$x$hc_opts$series, x)

  hc
}

#' Helpers to use highcharter as input in shiny apps
#'
#' When you use highcharter in a shiny app, for example
#' \code{renderHighcharter('my_chart')}, you can access to the actions of the
#' user using and then use the \code{hc_add_event_point} via the
#' \code{my_chart} input (\code{input$my_chart}). That's a way you can
#' use a chart as an input.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param series The name of type of series to apply the event.
#' @param event The name of event: click, mouseOut,  mouseOver. See
#'   \url{https://api.highcharts.com/highcharts/plotOptions.areasplinerange.point.events.select}
#'   for more details.
#'
#' @note Event details are accessible from hc_name_EventType, i.e. if a highchart is rendered against output$my_hc and
#'     and we wanted the coordinates of the user-clicked point we would use input$my_hc_click
#'
#' @export
hc_add_event_point <- function(hc, series = "series", event = "click") {
  fun <- paste0("function(){
  var pointinfo = {series: this.series.name, seriesid: this.series.id,
  name: this.name, x: this.x, y: this.y, category: this.category.name}
  window.x = this;
  console.log(pointinfo);

  if (typeof Shiny != 'undefined') { Shiny.onInputChange(this.series.chart.renderTo.id + '_' + '", event, "', pointinfo); }
}")

  fun <- JS(fun)

  eventobj <- structure(
    list(structure(
      list(structure(
        list(structure(
          list(fun),
          .Names = event
        )),
        .Names = "events"
      )),
      .Names = "point"
    )),
    .Names = series
  )

  hc$x$hc_opts$plotOptions <- rlist::list.merge(
    hc$x$hc_opts$plotOptions,
    eventobj
  )

  hc
}

#' @rdname hc_add_event_point
#' @export
hc_add_event_series <- function(hc, series = "series", event = "click") {
  fun <- paste0("function(){
  var seriesinfo = {name: this.name }
  console.log(seriesinfo);
  window.x = this;
  if (typeof Shiny != 'undefined') { Shiny.onInputChange(this.chart.renderTo.id + '_' + '", event, "', seriesinfo); }

}")
  fun <- JS(fun)

  eventobj <- structure(
    list(structure(
      list(structure(
        list(fun),
        .Names = event
      )),
      .Names = "events"
    )),
    .Names = series
  )

  hc$x$hc_opts$plotOptions <- rlist::list.merge(
    hc$x$hc_opts$plotOptions,
    eventobj
  )

  hc
}

#' Helper to add annotations from data frame or list
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/annotations}.
#'
#' @export
hc_add_annotation <- function(hc, ...) {
  assertthat::assert_that(is.highchart(hc))

  hc$x$hc_opts[["annotations"]] <- append(
    hc$x$hc_opts[["annotations"]],
    list(list(...))
  )

  hc
}

#' @rdname hc_add_annotation
#' @param x A \code{list} or a \code{data.frame} of annotations.
#' @details The \code{x} elements must have \code{xValue} and \code{yValue}
#'   elements
#' @export
hc_add_annotations <- function(hc, x) {
  assertthat::assert_that(is.highchart(hc), (is.list(x) | is.data.frame(x)))

  if (is.data.frame(x)) {
    x <- list_parse(x)
  }

  hc$x$hc_opts[["annotations"]] <- append(hc$x$hc_opts[["annotations"]], x)

  hc
}

#' Add modules or plugin dependencies to highcharts objects
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param name The partial path to the plugin or module,
#'   example: `"plugins/annotations.js"`
#' @examples
#'
#' data(mpg, package = "ggplot2")
#'
#' hchart(mpg, "point", hcaes(displ, hwy),
#'   regression = TRUE,
#'   regressionSettings = list(type = "polynomial", order = 5, hideInLegend = TRUE)
#' ) %>%
#'   hc_add_dependency("plugins/highcharts-regression.js")
#'
#' hchart(mpg, "point", hcaes(displ, hwy, group = drv), regression = TRUE) %>%
#'   hc_colors(c("#d35400", "#2980b9", "#2ecc71")) %>%
#'   hc_add_dependency("plugins/highcharts-regression.js")
#' @details See `vignette("modules")`
#' @importFrom purrr map_chr
#' @importFrom htmltools htmlDependency
#' @importFrom yaml yaml.load_file
#' @export
hc_add_dependency <- function(hc, name = "plugins/annotations.js") {
  stopifnot(!is.null(name))

  yml <- system.file("htmlwidgets/highchart.yaml", package = "highcharter")
  yml <- yaml.load_file(yml)[[1]]

  hc_ver <- map_chr(yml, c("version"))[map_chr(yml, c("name")) == "highcharts"]

  dep <- htmlDependency(
    name = basename(name),
    version = hc_ver,
    src = c(file = system.file(
      sprintf("htmlwidgets/lib/highcharts/%s", dirname(name)),
      package = "highcharter"
    )),
    script = basename(name)
  )

  hc$dependencies <- c(hc$dependencies, list(dep))

  hc
}

#' Sand Signika theme for highcharts
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_sandsignika())
#' @export
hc_theme_sandsignika <- function(...) {
  theme <-
    list(
      colors = c(
        "#F45B5B", "#8085E9", "#8D4654", "#7798BF", "#AAEEEE",
        "#FF0066", "#EEAAEE", "#55BF3B", "#DF5353"
      ),
      chart = list(
        backgroundColor = NULL,
        divBackgroundImage =
          "https://www.highcharts.com/samples/graphics/sand.png",
        style = list(
          fontFamily = "Signika, serif"
        )
      ),
      title = list(
        style = list(
          color = "black",
          fontSize = "16px",
          fontWeight = "bold"
        )
      ),
      subtitle = list(
        style = list(
          color = "black"
        )
      ),
      tooltip = list(
        borderWidth = 0
      ),
      legend = list(
        itemStyle = list(
          fontWeight = "bold",
          fontSize = "13px"
        )
      ),
      xAxis = list(
        labels = list(
          style = list(
            color = "#6e6e70"
          )
        )
      ),
      yAxis = list(
        labels = list(
          style = list(
            color = "#6e6e70"
          )
        )
      ),
      plotOptions = list(
        series = list(
          shadow = FALSE
        ),
        candlestick = list(
          lineColor = "#404048"
        ),
        map = list(
          shadow = FALSE
        )
      ),
      navigator = list(
        xAxis = list(
          gridLineColor = "#D0D0D8"
        )
      ),
      rangeSelector = list(
        buttonTheme = list(
          fill = "white",
          stroke = "#C0C0C8",
          "stroke-width" = 1,
          states = list(
            select = list(
              fill = "#D0D0D8"
            )
          )
        )
      ),
      scrollbar = list(
        trackBorderColor = "#C0C0C8"
      ),
      background2 = "#E0E0E8"
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' Creating highcharter themes
#'
#' Highcharts is very flexible so you can modify every element of the chart.
#' There are some exiting themes so you can apply style to charts with few
#' lines of code.
#'
#' More examples and details in \url{https://www.highcharts.com/docs/chart-design-and-style/themes}.
#'
#' @param ... A list of named parameters.
#'
#' @examples
#'
#' hc <- highcharts_demo()
#'
#' hc
#'
#' thm <- hc_theme(
#'   colors = c("red", "green", "blue"),
#'   chart = list(
#'     backgroundColor = "#15C0DE"
#'   ),
#'   title = list(
#'     style = list(
#'       color = "#333333",
#'       fontFamily = "Erica One"
#'     )
#'   ),
#'   subtitle = list(
#'     style = list(
#'       color = "#666666",
#'       fontFamily = "Shadows Into Light"
#'     )
#'   ),
#'   legend = list(
#'     itemStyle = list(
#'       fontFamily = "Tangerine",
#'       color = "black"
#'     ),
#'     itemHoverStyle = list(
#'       color = "gray"
#'     )
#'   )
#' )
#'
#' hc_add_theme(hc, thm)
#' @export
hc_theme <- function(...) {
  structure(list(...), class = "hc_theme")
}

#' Add themes to a highchart object
#'
#' Add highcharts themes to a highchart object.
#'
#' @param hc A highchart object
#' @param hc_thm A highchart theme object (\code{"hc_theme"} class)
#'
#' @examples
#'
#' highchart() %>%
#'   hc_add_series(
#'     data = c(
#'       7.0, 6.9, 9.5, 14.5, 18.2, 21.5, 25.2,
#'       26.5, 23.3, 18.3, 13.9, 9.6
#'     ),
#'     type = "column"
#'   ) %>%
#'   hc_add_theme(hc_theme_sandsignika())
#' @export
hc_add_theme <- function(hc, hc_thm) {
  assert_that(is.highchart(hc), .is_hc_theme(hc_thm))

  hc$x$fonts <- unique(c(hc$x$fonts, .hc_get_fonts(hc_thm)))

  hc$x$theme <- hc_thm

  hc
}

#' Merge themes
#'
#' Function to combine hc_theme objects.
#'
#' @param ... \code{hc_theme} objects.
#'
#' @examples
#'
#' thm <- hc_theme_merge(
#'   hc_theme_darkunica(),
#'   hc_theme(
#'     chart = list(
#'       backgroundColor = "transparent",
#'       divBackgroundImage = "http://cdn.wall-pix.net/albums/art-3Dview/00025095.jpg"
#'     ),
#'     title = list(
#'       style = list(
#'         color = "white",
#'         fontFamily = "Erica One"
#'       )
#'     )
#'   )
#' )
#' @export
hc_theme_merge <- function(...) {
  themes <- list(...)

  assert_that(unique(unlist(purrr::map(themes, class))) == "hc_theme")

  theme <- structure(list.merge(...), class = "hc_theme")

  theme
}

.hc_get_fonts <- function(lst) {
  unls <- unlist(lst)
  unls <- unls[grepl("fontFamily", names(unls))]

  fonts <- unls %>%
    str_replace_all(",\\s+sans-serif|,\\s+serif", "") %>%
    str_replace_all("\\s+", "+") %>%
    str_trim() %>%
    unlist()

  fonts
}

#' Add a map series
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param map A `list` object loaded from a geojson file.
#' @param df A `data.frame` object with data to chart. Code region and value are
#'   required.
#' @param value A string value with the name of the variable to chart.
#' @param joinBy What property to join the  \code{map} and \code{df}
#' @param ... Additional shared arguments for the data series
#'   (\url{https://api.highcharts.com/highcharts/series}).
#'
#' @examples
#'
#' library("dplyr")
#'
#' data("USArrests", package = "datasets")
#' data("usgeojson")
#'
#' USArrests <- mutate(USArrests, state = rownames(USArrests))
#'
#' highchart() %>%
#'   hc_title(text = "Violent Crime Rates by US State") %>%
#'   hc_subtitle(text = "Source: USArrests data") %>%
#'   hc_add_series_map(usgeojson, USArrests,
#'     name = "Murder arrests (per 100,000)",
#'     value = "Murder", joinBy = c("woename", "state"),
#'     dataLabels = list(
#'       enabled = TRUE,
#'       format = "{point.properties.postalcode}"
#'     )
#'   ) %>%
#'   hc_colorAxis(stops = color_stops()) %>%
#'   hc_legend(valueDecimals = 0, valueSuffix = "%") %>%
#'   hc_mapNavigation(enabled = TRUE)
#' \dontrun{
#'
#' data(worldgeojson, package = "highcharter")
#' data("GNI2014", package = "treemap")
#'
#' highchart(type = "map") %>%
#'   hc_add_series_map(map = worldgeojson, df = GNI2014, value = "GNI", joinBy = "iso3") %>%
#'   hc_colorAxis(stops = color_stops()) %>%
#'   hc_tooltip(
#'     useHTML = TRUE, headerFormat = "",
#'     pointFormat = "this is {point.name} and have {point.population} people with gni of {point.GNI}"
#'   )
#' }
#'
#' @details This function force the highchart object to be map type.
#' @importFrom utils tail
#' @export
hc_add_series_map <- function(hc, map, df, value, joinBy, ...) {
  assertthat::assert_that(
    is.highchart(hc),
    is.list(map),
    is.data.frame(df),
    value %in% names(df),
    tail(joinBy, 1) %in% names(df)
  )

  joindf <- tail(joinBy, 1)

  ddta <- rename(df, value := value)
  ddta <- list_parse(ddta)

  hc$x$type <- "map"

  hc %>%
    hc_add_series(
      mapData = map, data = ddta,
      joinBy = joinBy,
      ...
    ) %>%
    hc_colorAxis(min = 0)
}

#' Shortcut for create map from \url{https://code.highcharts.com/mapdata/}
#' collection.
#' @param map String indicating what map to chart, a list from
#'   \url{https://code.highcharts.com/mapdata/}. See examples.
#' @param download_map_data A logical value whether to download
#'   (add as a dependency) the map. Default \code{TRUE} via
#'   \code{getOption("highcharter.download_map_data")}.
#' @param data Optional data to make a choropleth, in case of use
#'   the joinBy and value are needed.
#' @param value A string value with the name of the variable to chart.
#' @param joinBy What property to join the \code{map} and \code{df}.
#' @param ... Additional shared arguments for the data series
#'   (\url{https://api.highcharts.com/highcharts/series}).
#'
#' @examples
#'
#' options(highcharter.download_map_data = TRUE)
#'
#' # hcmap(nullColor = "#DADADA")
#' # hcmap(nullColor = "#DADADA", download_map_data = FALSE)
#'
#' require(dplyr)
#' data("USArrests", package = "datasets")
#' USArrests <- mutate(USArrests, "woe-name" = rownames(USArrests))
#'
#' # hcmap(
#' #   map = "countries/us/us-all", data = USArrests,
#' #   joinBy = "woe-name", value = "UrbanPop", name = "Urban Population"
#' # )
#'
#' # download_map_data = FALSE
#' # hcmap(
#' #    map = "countries/us/us-all", data = USArrests,
#' #   joinBy = "woe-name", value = "UrbanPop", name = "Urban Population",
#' #   download_map_data = FALSE
#' # )
#' @importFrom htmltools htmlDependency
#' @importFrom rlang .data
#' @export
hcmap <- function(map = "custom/world",
                  download_map_data = getOption("highcharter.download_map_data"),
                  data = NULL, value = NULL, joinBy = NULL,
                  ...) {
  map <- fix_map_name(map)

  hc <- highchart(type = "map")

  if (download_map_data) {
    mapdata <- download_map_data(map)
  } else {
    dep <- htmlDependency(
      name = basename(map),
      version = "0.1.0",
      src = c(href = "https://code.highcharts.com/mapdata"),
      script = map
    )

    hc$dependencies <- c(hc$dependencies, list(dep))
    mapdata <- JS(sprintf("Highcharts.maps['%s']", str_replace(map, "\\.js$", "")))
  }

  if (is.null(data)) {
    hc <- hc %>%
      hc_add_series.default(
        mapData = mapdata, ...
      )
  } else {
    data <- rename(data, value := value)

    hc <- hc %>%
      hc_add_series.default(
        mapData = mapdata,
        data = list_parse(data), joinBy = joinBy, ...
      ) %>%
      hc_colorAxis(auxpar = NULL)
  }

  hc %>%
    hc_credits(enabled = TRUE)
}

#' Helper function to download the map data form a url
#'
#' The urls are listed in \url{https://code.highcharts.com/mapdata/}.
#' @param url The map's url.
#' @param showinfo Show the properties of the downloaded map to know how
#'   are the keys to add data in \code{hcmap}.
#' @param quiet Boolean parameter to turn off download messages (on by default).
#' @param method A string value for the download method used by
#'   \code{\link[utils]{download.file}}.
#' @examples
#' \dontrun{
#' mpdta <- download_map_data("https://code.highcharts.com/mapdata/countries/us/us-ca-all.js")
#' mpdta <- download_map_data("https://code.highcharts.com/mapdata/countries/us/us-ca-all.js",
#'   quiet = TRUE
#' )
#' str(mpdta, 1)
#' }
#' @seealso \code{\link{hcmap}}
#' @importFrom dplyr glimpse
#' @importFrom utils download.file
#' @importFrom stringr str_remove
#' @export
download_map_data <- function(url = "custom/world.js", showinfo = FALSE,
                              quiet = FALSE, method = "curl") {
  url <- sprintf(
    "https://code.highcharts.com/mapdata/%s",
    fix_map_name(url)
  )

  tmpfile <- tempfile(fileext = ".js")
  download.file(url, tmpfile, quiet = quiet, method = method)
  mapdata <- readLines(tmpfile, warn = FALSE, encoding = "UTF-8")
  mapdata[1] <- gsub(".* = ", "", mapdata[1])
  mapdata <- paste(mapdata, collapse = "\n")
  mapdata <- stringr::str_remove(mapdata, ";$")
  mapdata <- jsonlite::fromJSON(mapdata, simplifyVector = FALSE)

  if (showinfo) {
    glimpse(get_data_from_map(mapdata))
  }

  mapdata
}

#' Helper function to get the data inside the map data
#' The urls are listed in \url{https://code.highcharts.com/mapdata/}.
#' @param mapdata A list obtained from \code{\link{download_map_data}}.
#' @examples
#' \dontrun{
#' dta <- download_map_data("https://code.highcharts.com/mapdata/countries/us/us-ca-all.js")
#' get_data_from_map(dta)
#' }
#' @seealso \code{\link{download_map_data}}
#' @importFrom purrr map_lgl
#' @export
get_data_from_map <- function(mapdata) {
  mapdata$features %>%
    purrr::map("properties") %>%
    purrr::map_df(function(x) {
      x[!purrr::map_lgl(x, is.null)]
    })
}

fix_map_name <- function(x = "custom/world") {
  x <- stringr::str_replace(x, "\\.js$", "")
  x <- stringr::str_replace(x, "https://code\\.highcharts\\.com/mapdata/", "")
  x <- sprintf("%s.js", x)
  x
}

#' Visual comparison of Mountains Panorama
#'
#' This data comes from the \url{https://www.highcharts.com/} examples:  
#' https://www.highcharts.com/demo/3d-area-multiple
#'
#' @section Variables:
#'
#' \itemize{
#'
#'  \item \code{place}: The place.
#'  \item \code{name}: Name.
#'  \item \code{heigth}: Heigth.
#'
#' }
#'
#' @docType data
#' @name mountains_panorama
#' @usage mountains_panorama
#' @format A \code{data frame} with 91 observations and 3 variables.
"mountains_panorama"

#' City temperatures from a year in wide format
#'
#' This data comes from the \url{https://www.highcharts.com/} examples.
#'
#' @section Variables:
#'
#' \itemize{
#'
#'  \item \code{month}: The months.
#'  \item \code{tokyo}: Tokyo's temperatures.
#'  \item \code{new_york}: New York's temperatures.
#'  \item \code{berlin}: Berlin's temperatures.
#'  \item \code{london}: London's temperatures.
#'
#' }
#'
#' @docType data
#' @name citytemp
#' @usage citytemp
#' @format A \code{data frame} with 12 observations and 5 variables.
"citytemp"

#' City temperatures from a year in long format
#'
#' This data comes from the \url{https://www.highcharts.com/} examples.
#'
#' @section Variables:
#'
#' \itemize{
#'
#'  \item \code{month}: The months.
#'  \item \code{citiy}: City.
#'  \item \code{temp}: Temperatures.
#' }
#'
#' @docType data
#' @name citytemp_long
#' @usage citytemp_long
#' @format A \code{data frame} with 36 observations and 3 variables.
"citytemp_long"


#' Marshall's Favorite Bars
#'
#' Data from How I met Your Mother: Marshall's Favorite Bars.
#'
#' @section Variables:
#'
#' \itemize{
#'
#'  \item \code{bar}: Bar's name.
#'  \item \code{percent}: In percentage of awesomeness
#'
#' }
#'
#' @docType data
#' @name favorite_bars
#' @usage favorite_bars
#' @format A \code{data frame} with 5 observations and 2 variables.
"favorite_bars"

#' Marshall's Favorite Pies
#'
#' Data from How I met Your Mother: Marshall's Favorite Pies
#'
#' @section Variables:
#'
#' \itemize{
#'
#'  \item \code{pie}: Bar's name.
#'  \item \code{percent}: In percentage of tastiness
#'
#' }
#'
#' @docType data
#' @name favorite_pies
#' @usage favorite_pies
#' @format A \code{data frame} with 5 observations and 2 variables.
"favorite_pies"

#' globaltemp
#'
#' Temperature information by years.
#'
#' @section Variables:
#'
#' \itemize{
#'
#'  \item \code{date}: Date.
#'  \item \code{lower}: Minimum temperature.
#'  \item \code{median}: Median temperature.
#'  \item \code{upper}: Maximum temperature.
#' }
#'
#' @docType data
#' @name globaltemp
#' @usage globaltemp
#' @source \url{https://www.climate-lab-book.ac.uk/2016/spiralling-global-temperatures/}
#' @format A \code{data frame} with 1992 observations and 4 variables.
"globaltemp"

#' pokemon
#'
#' Information about 898 pokemon.
#'
#' @docType data
#' @name pokemon
#' @usage pokemon
#' @format A \code{data frame} with 898 observations and 24 variables.
"pokemon"

#' stars
#'
#' A sample using by Nadieh Bremer blocks. \url{http://bl.ocks.org/nbremer/eb0d1fd4118b731d069e2ff98dfadc47}.
#'
#' @section Variables:
#'
#' \itemize{
#'
#'  \item \code{bv}: BV
#'  \item \code{absmag}: Magnitude
#'  \item \code{lum}: Luminosity
#'  \item \code{temp}: Temperature
#'  \item \code{radiussun}: Radius
#'  \item \code{distance}: Distance
#' }
#'
#' @docType data
#' @name stars
#' @usage stars
#' @format A \code{data frame} with 404 observations and 6 variables.
"stars"

#' US Counties unemployment rate
#'
#' This data comes from the highcharts and is used in highmaps examples.
#'
#' @section Variables:
#'
#' \itemize{
#'
#'  \item \code{code}: The county code.
#'  \item `name`: The county name.
#'  \item \code{value}: The unemployment.
#'
#' }
#'
#' @docType data
#' @name unemployment
#' @usage unemployment
#' @format A `data.frame` with 3 variables and 3.216 observations.
"unemployment"

#' US Counties map in Geojson format (list)
#'
#' This data comes from the \url{https://code.highcharts.com/mapdata/countries/us/us-all-all.js}
#' and is used in highmaps examples.
#'
#' @docType data
#' @name uscountygeojson
#' @usage uscountygeojson
#' @format A `list` in geojson format.
"uscountygeojson"

#' US States map in Geojson format (list)
#'
#' This data comes from the \url{https://code.highcharts.com/mapdata/countries/us/us-all.js}
#' and is used in highmaps examples.
#'
#' @docType data
#' @name usgeojson
#' @usage usgeojson
#' @format A `list` in geojson format.
"usgeojson"

#' Vaccines
#'
#' The number of infected people by Measles, measured over 70-some years and across
#' all 50 states. From the WSJ analysis: \url{http://graphics.wsj.com/infectious-diseases-and-vaccines/}
#'
#' @section Variables:
#'
#' \itemize{
#'
#'  \item \code{year}: Year
#'  \item \code{state}: Name of the state
#'  \item \code{count}: Number of cases per 100,000 people. If the value is NA the count was 0.
#'
#' }
#'
#' @docType data
#' @name vaccines
#' @usage vaccines
#' @format A \code{data frame} with 3,876 observations and 3 variables.
"vaccines"

#' Weather
#'
#' Temperature information of San Francisco.
#'
#' @section Variables:
#'
#' \itemize{
#'
#'  \item \code{date}: Day in date format.
#'  \item \code{min_temperaturec}: Minimum temperature.
#'  \item \code{max_temperaturec}: Maximun temperature.
#'  \item \code{mean_temperaturec}: Mean temperature.
#' }
#'
#' @docType data
#' @name weather
#' @usage weather
#' @format A \code{data frame} with 365 observations and 4 variables.
"weather"

#' World map in Geojson format (list)
#'
#' This data comes from the \url{https://code.highcharts.com/mapdata/custom/world.js}
#' and is used in highmaps examples.#'
#'
#' @docType data
#' @name worldgeojson
#' @usage worldgeojson
#' @format A `list` in geojson format.
"worldgeojson"

#' Create a Highcharts chart widget
#'
#' This function creates a Highchart chart using \pkg{htmlwidgets}. The
#' widget can be rendered on HTML pages generated from R Markdown, Shiny, or
#' other applications.
#'
#' @param hc_opts A `list` object containing options defined as
#'    \url{https://api.highcharts.com/highcharts/}.
#' @param theme A \code{hc_theme} class object-
#' @param type A character value to set if use Highchart, Highstock or
#'   Highmap. Options are \code{"chart"}, \code{"stock"} and \code{"map"}.
#' @param width A numeric input in pixels.
#' @param height  A numeric input in pixels.
#' @param elementId	Use an explicit element ID for the widget.
#' @param google_fonts A boolean value. If TRUE (default), adds a reference to the
#'   Google Fonts API to the HTML head, downloading CSS for the font families
#'   defined in the Highcharts theme from https://fonts.googleapis.com. Set to
#'   FALSE if you load your own fonts using CSS. This option as default is
#'   controlled by \code{"highcharter.google_fonts"} option.
#' @importFrom htmlwidgets createWidget sizingPolicy
#' @export
highchart <- function(hc_opts = list(),
                      theme = getOption("highcharter.theme"),
                      type = "chart",
                      width = NULL,
                      height = NULL,
                      elementId = NULL,
                      google_fonts = getOption("highcharter.google_fonts")) {
  
  assertthat::assert_that(type %in% c("chart", "stock", "map", "gantt"))

  opts <- .join_hc_opts()

  if (identical(hc_opts, list())) {
    hc_opts <- opts$chart
  }

  unfonts <- NULL
  if (google_fonts) {
    unfonts <- unique(c(.hc_get_fonts(hc_opts), .hc_get_fonts(theme)))
  }

  opts$chart <- NULL

  # forward options using x
  x <- list(
    hc_opts = hc_opts,
    theme = theme,
    conf_opts = opts,
    type = type,
    fonts = unfonts,
    debug = getOption("highcharter.debug")
  )

  if (getOption("highcharter.debug")) {
    attr(x, "TOJSON_ARGS") <- list(pretty = getOption("highcharter.debug"))
  }

  if (getOption("highcharter.rjson")) {
    attr(x, "TOJSON_FUNC") <- rjson::toJSON
  }

  # create widget
  hc <- htmlwidgets::createWidget(
    name = "highchart",
    x,
    width = width,
    height = height,
    package = "highcharter",
    elementId = elementId,
    sizingPolicy = htmlwidgets::sizingPolicy(
      defaultWidth = "100%",
      # knitr.figure = FALSE,
      browser.fill = TRUE,
      padding = 0
    )
  )

  # set sizing for `shinyRenderer`
  hc <- hc_size(hc, width = width, height = height)

  hc
}

#' Reports whether x is a highchart object
#'
#' @param x An object to test
#' @export
is.highchart <- function(x) {
  inherits(x, "highchart") || inherits(x, "highchart2") || inherits(x, "highchartzero")
}

#' Widget output function for use in Shiny
#'
#' @param outputId The name of the input.
#' @param width A numeric input in pixels.
#' @param height  A numeric input in pixels.
#'
#' @importFrom htmlwidgets shinyWidgetOutput
#' @export
highchartOutput <- function(outputId, width = "100%", height = "400px") {
  shinyWidgetOutput(outputId, "highchart", width, height, package = "highcharter")
}

#' Widget render function for use in Shiny
#'
#' @param expr A highchart expression.
#' @param env A environment.
#' @param quoted  A boolean value.
#'
#' @importFrom htmlwidgets shinyRenderWidget
#' @export
renderHighchart <- function(expr, env = parent.frame(), quoted = FALSE) {
  if (!quoted) {
    expr <- substitute(expr)
  } # force quoted
  shinyRenderWidget(expr, highchartOutput, env, quoted = TRUE)
}


#' Create a Highcharts chart widget
#'
#' This widgets don't support options yet.
#'
#' This function creates a Highchart chart using \pkg{htmlwidgets}. The
#' widget can be rendered on HTML pages generated from R Markdown, Shiny, or
#' other applications.
#'
#' @param hc_opts A `list` object containing options defined as
#'    \url{https://api.highcharts.com/highcharts/}.
#' @param theme A \code{hc_theme} class object.
#' @param type A character value to set if use Highchart, Highstock or
#'   Highmap. Options are \code{"chart"}, \code{"stock"} and \code{"map"}.
#' @param width A numeric input in pixels.
#' @param height  A numeric input in pixels.
#' @param elementId	Use an explicit element ID for the widget.
#' @param google_fonts A boolean value. If TRUE (default), adds a reference to the
#'   Google Fonts API to the HTML head, downloading CSS for the font families
#'   defined in the Highcharts theme from https://fonts.googleapis.com. Set to
#'   FALSE if you load your own fonts using CSS.
#'
#' @export
highchart2 <- function(hc_opts = list(),
                       theme = getOption("highcharter.theme"),
                       type = "chart",
                       width = NULL,
                       height = NULL,
                       elementId = NULL,
                       google_fonts = getOption("highcharter.google_fonts")) {
  
  assertthat::assert_that(type %in% c("chart", "stock", "map", "gantt"))
  
  opts <- .join_hc_opts()
  
  if (identical(hc_opts, list())) {
    hc_opts <- opts$chart
  }
  
  unfonts <- NULL
  if (google_fonts) {
    unfonts <- unique(c(.hc_get_fonts(hc_opts), .hc_get_fonts(theme)))
  }
  
  opts$chart <- NULL
  
  # forward options using x
  x <- list(
    hc_opts = hc_opts,
    theme = theme,
    conf_opts = opts,
    type = type,
    fonts = unfonts,
    debug = getOption("highcharter.debug")
  )
  
  if (getOption("highcharter.debug")) {
    attr(x, "TOJSON_ARGS") <- list(pretty = getOption("highcharter.debug"))
  }
  
  if (getOption("highcharter.rjson")) {
    attr(x, "TOJSON_FUNC") <- rjson::toJSON
  }
  
  # create widget
  hc <- htmlwidgets::createWidget(
    name = "highchart",
    x,
    width = width,
    height = height,
    package = "highcharter",
    elementId = elementId,
    sizingPolicy = htmlwidgets::sizingPolicy(
      defaultWidth = "100%",
      # knitr.figure = FALSE,
      browser.fill = TRUE,
      padding = 0
    )
  )
  
  # set sizing for `shinyRenderer`
  hc <- hc_size(hc, width = width, height = height)
  
  hc
  
}

#' @rdname highchartOutput
#' @export
highchartOutput2 <- function(outputId, width = "100%", height = "400px") {
  shinyWidgetOutput(outputId, "highchart2", width, height, package = "highcharter")
}

#' @rdname renderHighchart
#' @export
renderHighchart2 <- function(expr, env = parent.frame(), quoted = FALSE) {
  if (!quoted) {
    expr <- substitute(expr)
  } # force quoted
  shinyRenderWidget(expr, highchartOutput2, env, quoted = TRUE)
}

#' @rdname highchart2
#' @export
highchartzero <- function(hc_opts = list(),
                          theme = NULL,
                          width = NULL,
                          height = NULL,
                          elementId = NULL) {

  x <- list(
    hc_opts = hc_opts
  )

  # create widget
  htmlwidgets::createWidget(
    name = "highchartzero",
    x,
    width = width,
    height = height,
    package = "highcharter",
    elementId = elementId,
    sizingPolicy = htmlwidgets::sizingPolicy(
      defaultWidth = "100%",
      knitr.figure = FALSE,
      knitr.defaultWidth = "100%",
      browser.fill = TRUE
    )
  )
}

#' @rdname highchartOutput
#' @export
highchartOutputZ <- function(outputId, width = "100%", height = "400px") {
  shinyWidgetOutput(outputId, "highchartzero", width, height, package = "highcharter")
}

#' @rdname renderHighchart
#' @export
renderHighchartZ <- function(expr, env = parent.frame(), quoted = FALSE) {
  if (!quoted) {
    expr <- substitute(expr)
  } # force quoted
  shinyRenderWidget(expr, highchartOutputZ, env, quoted = TRUE)
}

#' ggplot2 theme for highcharts
#'
#' ggplot2 theme is based on \url{https://ggplot2.tidyverse.org/}.
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_ggplot2())
#' @export
hc_theme_ggplot2 <- function(...) {
  theme <- hc_theme(
    chart = list(
      plotBackgroundColor = "#EBEBEB",
      style = list(
        color = "#000000",
        fontFamily = "Arial, sans-serif"
      )
    ),
    colors = c("#595959", "#F8766D", "#A3A500", "#00BF7D", "#00B0F6", "#E76BF3"),
    xAxis = list(
      labels = list(style = list(color = "#666666")),
      title = list(style = list(color = "#000000")),
      startOnTick = FALSE,
      endOnTick = FALSE,
      gridLineColor = "#FFFFFF",
      gridLineWidth = 1.5,
      tickWidth = 1.5,
      tickLength = 5,
      tickColor = "#666666",
      minorTickInterval = 0,
      minorGridLineColor = "#FFFFFF",
      minorGridLineWidth = 0.5
    ),
    yAxis = list(
      labels = list(style = list(color = "#666666")),
      title = list(style = list(color = "#000000")),
      startOnTick = FALSE,
      endOnTick = FALSE,
      gridLineColor = "#FFFFFF",
      gridLineWidth = 1.5,
      tickWidth = 1.5,
      tickLength = 5,
      tickColor = "#666666",
      minorTickInterval = 0,
      minorGridLineColor = "#FFFFFF",
      minorGridLineWidth = 0.5
    ),
    legendBackgroundColor = "rgba(0, 0, 0, 0.5)",
    background2 = "#505053",
    dataLabelsColor = "#B0B0B3",
    textColor = "#C0C0C0",
    contrastTextColor = "#F0F0F3",
    maskColor = "rgba(255,255,255,0.3)"
  )

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' Elementary (OS) theme for highcharts
#'
#' Elementary (OS) theme for highcharts was based on \url{https://elementary.io}
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_elementary())
#' @export
hc_theme_elementary <- function(...) {
  theme <-
    list(
      colors = list("#41B5E9", "#FA8832", "#34393C", "#E46151"),
      chart = list(
        style = list(
          color = "#333",
          fontFamily = "Open Sans"
        )
      ),
      title = list(
        style = list(
          fontFamily = "Raleway",
          fontWeight = "100"
        )
      ),
      subtitle = list(
        style = list(
          fontFamily = "Raleway",
          fontWeight = "100"
        )
      ),
      legend = list(
        align = "right",
        verticalAlign = "bottom"
      ),
      xAxis = list(
        gridLineWidth = 1,
        gridLineColor = "#F3F3F3",
        lineColor = "#F3F3F3",
        minorGridLineColor = "#F3F3F3",
        tickColor = "#F3F3F3",
        tickWidth = 1
      ),
      yAxis = list(
        gridLineColor = "#F3F3F3",
        lineColor = "#F3F3F3",
        minorGridLineColor = "#F3F3F3",
        tickColor = "#F3F3F3",
        tickWidth = 1
      )
    )
  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' Firefox theme for highcharts
#'
#' Firefox theme was inspired by \url{https://brand.mozilla.com/all-brands}.
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_ffx())
#' @export
hc_theme_ffx <- function(...) {
  theme <-
    hc_theme(
      colors = c("#00AACC", "#FF4E00", "#B90000", "#5F9B0A", "#CD6723"),
      chart = list(
        backgroundColor = list(
          linearGradient = list(0, 0, 0, 150),
          stops = list(
            list(0, "#CAE1F4"),
            list(1, "#EEEEEE")
          )
        ),
        style = list(
          fontFamily = "Open Sans"
        )
      ),
      title = list(
        align = "left"
      ),
      subtitle = list(
        align = "left"
      ),
      legend = list(
        align = "right",
        verticalAlign = "bottom"
      ),
      xAxis = list(
        gridLineWidth = 1,
        gridLineColor = "#F3F3F3",
        lineColor = "#F3F3F3",
        minorGridLineColor = "#F3F3F3",
        tickColor = "#F3F3F3",
        tickWidth = 1
      ),
      yAxis = list(
        gridLineColor = "#F3F3F3",
        lineColor = "#F3F3F3",
        minorGridLineColor = "#F3F3F3",
        tickColor = "#F3F3F3",
        tickWidth = 1
      )
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

# Generated by 'dev/generate-highcharts-api.R'
# Generated in 2021-02-02 12:43:12
#

#' Annotations options for highcharter objects
#'
#' A basic type of an annotation. It allows to add custom labels
#' or shapes. The items  can be tied to points, axis coordinates
#' or chart pixel coordinates.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/annotations}.
#'
#' @examples
#'
#' # Ex 1
#' highchart() %>%
#'   hc_add_series(
#'     data = c(29.9, 71.5, 106.4, 129.2, 144.0, 176.0, 135.6, 148.5, 216.4, 194.1, 95.6, 54.4)
#'   ) %>%
#'   hc_xAxis(
#'     tickInterval = 0.5,
#'     gridLineWidth = 1
#'   ) %>%
#'   hc_annotations(
#'     list(
#'       labels =
#'         list(
#'           list(
#'             point = list(x = 3, y = 129.2, xAxis = 0, yAxis = 0),
#'             text = "x: {x}<br/>y: {y}"
#'           ),
#'           list(
#'             point = list(x = 9, y = 194.1, xAxis = 0, yAxis = 0),
#'             text = "x: {x}<br/>y: {y}"
#'           ),
#'           list(
#'             point = list(x = 5, y = 100, xAxis = 0),
#'             text = "x: {x}<br/>y: {point.plotY} px"
#'           ),
#'           list(
#'             point = list(x = 0, y = 0),
#'             text = "x: {point.plotX} px<br/>y: {point.plotY} px"
#'           )
#'         )
#'     )
#'   )
#'
#' # Ex 2
#' df <- data.frame(
#'   x = 1:10,
#'   y = 1:10
#' )
#'
#' highchart() %>%
#'   hc_add_series(data = df, hcaes(x = x, y = y), type = "area") %>%
#'   hc_annotations(
#'     list(
#'       labels = list(
#'         list(point = list(x = 5, y = 5, xAxis = 0, yAxis = 0), text = "Middle"),
#'         list(point = list(x = 1, y = 1, xAxis = 0, yAxis = 0), text = "Start")
#'       )
#'     )
#'   )
#' @export
hc_annotations <- function(hc, ...) {
  .hc_opt(hc, "annotations", ...)
}

#' Boost options for highcharter objects
#'
#' Options for the Boost module. The Boost module allows certain series types
#' to be rendered by WebGL instead of the default SVG. This allows hundreds of
#' thousands of data points to be rendered in milliseconds. In addition to the
#' WebGL rendering it saves time by skipping processing and inspection of the
#' data wherever possible. This introduces some limitations to what features are
#' available in boost mode. See the docs for
#' details.
#' In addition to the global boost option, each series has a
#' boostThreshold that defines when the
#' boost should kick in.
#' Requires the modules/boost.js module.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/boost}.
#'
#' @examples
#'
#' #  Ex 1
#' options(highcharter.rjson = FALSE)
#'
#' n <- 50000
#'
#' x <- sin(4 * 2 * pi * seq(n) / n) + rnorm(n) / 10
#'
#' x <- round(x, 3)
#'
#' plot(x)
#'
#' hc1 <- highchart() %>%
#'   hc_chart(zoomType = "x") %>%
#'   hc_add_series(data = x) %>%
#'   hc_title(text = "No boost") %>%
#'   hc_boost(
#'     enabled = FALSE # Default
#'   )
#'
#' hc1
#'
#' # Boost is a stripped-down renderer-in-a-module for Highcharts. It bypasses
#' # some of the standard Highcharts features (such as animation), and focuses
#' # on pushing as many points as possible as quickly as possible.
#'
#' hc2 <- highchart() %>%
#'   hc_chart(zoomType = "x") %>%
#'   hc_add_series(data = x) %>%
#'   hc_title(text = "With boost") %>%
#'   hc_boost(enabled = TRUE)
#'
#' hc2
#'
#'
#' # # Ex 2
#' # library(MASS)
#' #
#' # n <- 20000
#' #
#' # sigma <- matrix(c(10,3,3,2),2,2)
#' # sigma
#' #
#' # mvr <- round(mvrnorm(n, rep(c(0, 0)), sigma), 2)
#' #
#' # vx <- ceiling(1+abs(max(mvr[, 1])))
#' # vy <- ceiling(1+abs(max(mvr[, 2])))
#' #
#' # # unnamed list
#' # ds <- list_parse2(as.data.frame(mvr))
#' #
#' # highchart() %>%
#' #   hc_chart(zoomType = "xy") %>%
#' #   hc_xAxis(min = -vx, max = vx) %>%
#' #   hc_yAxis(min = -vy, max = vy) %>%
#' #   hc_add_series(
#' #     data = ds, #list
#' #     type = "scatter",
#' #     name = "A lot of points!",
#' #     color = 'rgba(0,0,0,0.1)',
#' #     marker = list(radius = 2)
#' #     ) %>%
#' #   hc_boost(
#' #     enabled = TRUE
#' #   )
#' #
#' # dat <- as.data.frame(mvr)
#' # names(dat) <- c("x", "y")
#' #
#' # highchart() %>%
#' #   hc_chart(zoomType = "xy") %>%
#' #   hc_xAxis(min = -vx, max = vx) %>%
#' #   hc_yAxis(min = -vy, max = vy) %>%
#' #   hc_add_series(
#' #     data = dat,
#' #     type = "scatter",
#' #     hcaes(x, y),
#' #     name = "A lot of points!",
#' #     color = 'rgba(0,0,0,0.1)',
#' #     marker = list(radius = 2)
#' #   ) %>%
#' #   hc_boost(enabled = TRUE)
#' #
#' # # Ex3
#' # N <- 1000000
#' # n <- 5
#' # s <- seq(n)
#' # s <- s/(max(s) + min(s))
#' # s <- round(s, 2)
#' #
#' # series <- s %>%
#' #   purrr::map(~ stats::arima.sim(round(N/n), model = list(ar = .x)) + .x * n * 20) %>%
#' #   purrr::map(as.vector) %>%
#' #   purrr::map(round, 2) %>%
#' #   purrr::map(~ list(data = .x))
#' #
#' # highchart() %>%
#' #   hc_add_series_list(series) %>%
#' #   hc_chart(zoomType = "x") %>%
#' #   hc_boost(enabled = TRUE)
#' @export
hc_boost <- function(hc, ...) {
  .hc_opt(hc, "boost", ...)
}

#' Caption options for highcharter objects
#'
#' The chart's caption, which will render below the chart and will be part
#' of exported charts. The caption can be updated after chart initialization
#' through the Chart.update or Chart.caption.update methods.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/caption}.
#'
#' @examples
#'
#' highchart() %>%
#'   hc_title(text = "Chart with a caption") %>%
#'   hc_subtitle(text = "This is the subtitle") %>%
#'   hc_xAxis(categories = c("Apples", "Pears", "Banana", "Orange")) %>%
#'   hc_add_series(
#'     data = c(1, 4, 3, 5),
#'     type = "column",
#'     name = "Fruits"
#'   ) %>%
#'   hc_caption(
#'     text = "<b>The caption renders in the bottom, and is part of the exported
#'     chart.</b><br><em>Lorem ipsum dolor sit amet, consectetur adipiscing elit,
#'     sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim
#'     ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip
#'     ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate
#'     velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat
#'     cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est
#'     laborum.</em>'"
#'   )
#' @export
hc_caption <- function(hc, ...) {
  .hc_opt(hc, "caption", ...)
}

#' Chart options for highcharter objects
#'
#' General options for the chart.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/chart}.
#'
#' @examples
#'
#' hc <- highchart() %>%
#'   hc_xAxis(categories = month.abb) %>%
#'   hc_add_series(name = "Tokyo", data = sample(1:12)) %>%
#'   hc_add_series(name = "London", data = sample(1:12) + 10)
#'
#' hc
#'
#' hc %>%
#'   hc_chart(
#'     type = "column",
#'     options3d = list(enabled = TRUE, beta = 15, alpha = 15)
#'   )
#'
#'
#' hc %>%
#'   hc_chart(
#'     borderColor = "#EBBA95",
#'     borderRadius = 10,
#'     borderWidth = 2,
#'     backgroundColor = list(
#'       linearGradient = c(0, 0, 500, 500),
#'       stops = list(
#'         list(0, "rgb(255, 255, 255)"),
#'         list(1, "rgb(200, 200, 255)")
#'       )
#'     )
#'   )
#' @export
hc_chart <- function(hc, ...) {
  .hc_opt(hc, "chart", ...)
}

#' Coloraxis options for highcharter objects
#'
#' A color axis for series. Visually, the color
#' axis will appear as a gradient or as separate items inside the
#' legend, depending on whether the axis is scalar or based on data
#' classes.
#' For supported color formats, see the
#' docs article about colors.
#' A scalar color axis is represented by a gradient. The colors either
#' range between the minColor and the
#' maxColor, or for more fine grained control the
#' colors can be defined in stops. Often times, the
#' color axis needs to be adjusted to get the right color spread for the
#' data. In addition to stops, consider using a logarithmic
#' axis type, or setting min and
#' max to avoid the colors being determined by
#' outliers.
#' When dataClasses are used, the ranges are
#' subdivided into separate classes like categories based on their
#' values. This can be used for ranges between two values, but also for
#' a true category. However, when your data is categorized, it may be as
#' convenient to add each category to a separate series.
#' Color axis does not work with: sankey, sunburst, dependencywheel,
#' networkgraph, wordcloud, venn, gauge and solidgauge series
#' types.
#' Since v7.2.0 colorAxis can also be an array of options objects.
#' See the Axis object for
#' programmatic access to the axis.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/colorAxis}.
#'
#' @examples
#'
#' library(dplyr)
#'
#' data(mpg, package = "ggplot2")
#'
#' mpgman2 <- mpg %>%
#'   group_by(manufacturer, year) %>%
#'   dplyr::summarise(
#'     n = dplyr::n(),
#'     displ = mean(displ)
#'   )
#'
#' mpgman2
#'
#' hchart(
#'   mpgman2, "column", hcaes(x = manufacturer, y = n, group = year),
#'   colorKey = "displ",
#'   # color = c("#FCA50A", "#FCFFA4"),
#'   name = c("Year 1999", "Year 2008")
#' ) %>%
#'   hc_colorAxis(min = 0, max = 5)
#'
#'
#' # defaults to yAxis
#' hchart(iris, "point", hcaes(Sepal.Length, Sepal.Width)) %>%
#'   hc_colorAxis(
#'     minColor = "red",
#'     maxColor = "blue"
#'   )
#'
#'
#'
#' # Ex2
#' n <- 5
#'
#' stops <- data.frame(
#'   q = 0:n / n,
#'   c = c("#440154", "#414487", "#2A788E", "#22A884", "#7AD151", "#FDE725"),
#'   stringsAsFactors = FALSE
#' )
#'
#' stops <- list_parse2(stops)
#'
#' M <- round(matrix(rnorm(50 * 50), ncol = 50), 2)
#'
#' hchart(M) %>%
#'   hc_colorAxis(stops = stops)
#'
#' # Ex3
#' # hchart(volcano) %>%
#' #   hc_colorAxis(stops = stops, max = 200)
#' @export
hc_colorAxis <- function(hc, ...) {
  .hc_opt(hc, "colorAxis", ...)
}

#' Colors options for highcharter objects
#'
#' An array containing the default colors for the chart's series. When all
#' colors are used, new colors are pulled from the start again.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param colors A vector of colors.
#'
#' @examples
#'
#' library(viridisLite)
#'
#' cols <- viridis(3)
#' cols <- substr(cols, 0, 7)
#'
#' highchart() %>%
#'   hc_add_series(data = sample(1:12)) %>%
#'   hc_add_series(data = sample(1:12) + 10) %>%
#'   hc_add_series(data = sample(1:12) + 20) %>%
#'   hc_colors(cols)
#' @export
hc_colors <- function(hc, colors) {
  assertthat::assert_that(is.vector(colors))

  if (length(colors) == 1) {
    colors <- list(colors)
  }

  hc$x$hc_opts$colors <- colors

  hc
}


#' Credits options for highcharter objects
#'
#' Highchart by default puts a credits label in the lower right corner
#' of the chart. This can be changed using these options.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/credits}.
#'
#' @examples
#'
#' highchart() %>%
#'   hc_xAxis(categories = citytemp$month) %>%
#'   hc_add_series(name = "Tokyo", data = sample(1:12)) %>%
#'   hc_credits(
#'     enabled = TRUE,
#'     text = "htmlwidgets.org",
#'     href = "http://www.htmlwidgets.org/"
#'   )
#' @export
hc_credits <- function(hc, ...) {
  .hc_opt(hc, "credits", ...)
}

#' Drilldown options for highcharter objects
#'
#' Options for drill down, the concept of inspecting increasingly high
#' resolution data through clicking on chart items like columns or pie slices.
#' The drilldown feature requires the drilldown.js file to be loaded,
#' found in the modules directory of the download package, or online at
#' code.highcharts.com/modules/drilldown.js.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/drilldown}.
#'
#' @examples
#'
#' library(highcharter)
#' library(dplyr)
#' library(purrr)
#'
#' df <- tibble(
#'   name = c("Animals", "Fruits"),
#'   y = c(5, 2),
#'   drilldown = tolower(name)
#' )
#'
#' df
#'
#' hc <- highchart() %>%
#'   hc_title(text = "Basic drilldown") %>%
#'   hc_xAxis(type = "category") %>%
#'   hc_legend(enabled = FALSE) %>%
#'   hc_plotOptions(
#'     series = list(
#'       boderWidth = 0,
#'       dataLabels = list(enabled = TRUE)
#'     )
#'   ) %>%
#'   hc_add_series(
#'     data = df,
#'     type = "column",
#'     hcaes(name = name, y = y),
#'     name = "Things",
#'     colorByPoint = TRUE
#'   )
#'
#' dfan <- data.frame(
#'   name = c("Cats", "Dogs", "Cows", "Sheep", "Pigs"),
#'   value = c(4, 3, 1, 2, 1)
#' )
#'
#' dffru <- data.frame(
#'   name = c("Apple", "Organes"),
#'   value = c(4, 2)
#' )
#'
#'
#' dsan <- list_parse2(dfan)
#'
#' dsfru <- list_parse2(dffru)
#'
#' hc <- hc %>%
#'   hc_drilldown(
#'     allowPointDrilldown = TRUE,
#'     series = list(
#'       list(
#'         id = "animals",
#'         data = dsan
#'       ),
#'       list(
#'         id = "fruits",
#'         data = dsfru
#'       )
#'     )
#'   )
#'
#' hc
#' @export
hc_drilldown <- function(hc, ...) {
  .hc_opt(hc, "drilldown", ...)
}

#' Exporting options for highcharter objects
#'
#' Options for the exporting module. For an overview on the matter, see
#' the docs.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/exporting}.
#'
#' @examples
#'
#' highchart() %>%
#'   hc_xAxis(categories = month.abb) %>%
#'   hc_add_series(name = "Tokyo", data = sample(1:12)) %>%
#'   hc_exporting(
#'     enabled = TRUE, # always enabled
#'     filename = "custom-file-name"
#'   )
#' @export
hc_exporting <- function(hc, ...) {
  .hc_opt(hc, "exporting", ...)
}

#' Labels options for highcharter objects
#'
#' HTML labels that can be positioned anywhere in the chart area.
#' This option is deprecated since v7.1.2. Instead, use
#' annotations that support labels.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in the Highcharts labels API.
#'
#' @examples
#'
#' highchart() %>%
#'   hc_add_series(data = sample(1:12)) %>%
#'   hc_labels(
#'     items = list(
#'       list(
#'         html = "<p>Some <b>important</b><br>text</p>",
#'         style = list(
#'           left = "150%",
#'           top = "150%"
#'         )
#'       )
#'     )
#'   )
#' @export
hc_labels <- function(hc, ...) {
  .hc_opt(hc, "labels", ...)
}

#' Legend options for highcharter objects
#'
#' The legend is a box containing a symbol and name for each series
#' item or point item in the chart. Each series (or points in case
#' of pie charts) is represented by a symbol and its name in the legend.
#' It is possible to override the symbol creator function and create
#' custom legend symbols.
#'
#' A Highmaps legend by default contains one legend item per series, but if
#' a colorAxis is defined, the axis will be displayed in the legend.
#' Either as a gradient, or as multiple legend items for dataClasses.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/legend}.
#'
#' @examples
#'
#' highchart() %>%
#'   hc_xAxis(categories = month.abb) %>%
#'   hc_add_series(name = "Tokyo", data = sample(1:12)) %>%
#'   hc_add_series(name = "London", data = sample(1:12) + 10) %>%
#'   hc_add_series(name = "Other City", data = sample(1:12) + 20) %>%
#'   hc_legend(
#'     align = "left",
#'     verticalAlign = "top",
#'     layout = "vertical",
#'     x = 0,
#'     y = 100
#'   )
#' @export
hc_legend <- function(hc, ...) {
  .hc_opt(hc, "legend", ...)
}

#' Loading options for highcharter objects
#'
#' The loading options control the appearance of the loading screen
#' that covers the plot area on chart operations. This screen only
#' appears after an explicit call to chart.showLoading(). It is a
#' utility for developers to communicate to the end user that something
#' is going on, for example while retrieving new data via an XHR connection.
#' The "Loading..." text itself is not part of this configuration
#' object, but part of the lang object.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/loading}.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_loading(
#'     hideDuration = 1000,
#'     showDuration = 1000
#'   )
#' @export
hc_loading <- function(hc, ...) {
  .hc_opt(hc, "loading", ...)
}

#' Pane options for highcharter objects
#'
#' The pane serves as a container for axes and backgrounds for circular
#' gauges and polar charts.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/pane}.
#'
#' @examples
#'
#' highchart() %>%
#'   hc_chart(
#'     type = "gauge",
#'     plotBackgroundColor = NULL,
#'     plotBackgroundImage = NULL,
#'     plotBorderWidth = 0,
#'     plotShadow = FALSE
#'   ) %>%
#'   hc_title(
#'     text = "Speedometer"
#'   ) %>%
#'   hc_pane(
#'     startAngle = -150,
#'     endAngle = 150,
#'     background = list(list(
#'       backgroundColor = list(
#'         linearGradient = list(x1 = 0, y1 = 0, x2 = 0, y2 = 1),
#'         stops = list(
#'           list(0, "#FFF"),
#'           list(1, "#333")
#'         )
#'       ),
#'       borderWidth = 0,
#'       outerRadius = "109%"
#'     ), list(
#'       backgroundColor = list(
#'         linearGradient = list(x1 = 0, y1 = 0, x2 = 0, y2 = 1),
#'         stops = list(
#'           list(0, "#333"),
#'           list(1, "#FFF")
#'         )
#'       ),
#'       borderWidth = 1,
#'       outerRadius = "107%"
#'     ), list(
#'       # default background
#'     ), list(
#'       backgroundColor = "#DDD",
#'       borderWidth = 0,
#'       outerRadius = "105%",
#'       innerRadius = "103%"
#'     ))
#'   ) %>%
#'   hc_add_series(
#'     data = list(80), name = "speed", tooltip = list(valueSuffix = " km/h")
#'   ) %>%
#'   hc_yAxis(
#'     min = 0,
#'     max = 200,
#'     minorTickInterval = "auto",
#'     minorTickWidth = 1,
#'     minorTickLength = 10,
#'     minorTickPosition = "inside",
#'     minorTickColor = "#666",
#'     tickPixelInterval = 30,
#'     tickWidth = 2,
#'     tickPosition = "inside",
#'     tickLength = 10,
#'     tickColor = "#666",
#'     labels = list(
#'       step = 2,
#'       rotation = "auto"
#'     ),
#'     title = list(
#'       text = "km/h"
#'     ),
#'     plotBands = list(
#'       list(from = 0, to = 120, color = "#55BF3B"),
#'       list(from = 120, to = 160, color = "#DDDF0D"),
#'       list(from = 160, to = 200, color = "#DF5353")
#'     )
#'   )
#' @export
hc_pane <- function(hc, ...) {
  .hc_opt(hc, "pane", ...)
}

#' Plotoptions options for highcharter objects
#'
#' The plotOptions is a wrapper object for config objects for each series
#' type. The config objects for each series can also be overridden for
#' each series item as given in the series array.
#' Configuration options for the series are given in three levels. Options
#' for all series in a chart are given in the plotOptions.series object. Then options for all series of a specific
#' type are given in the plotOptions of that type, for example
#' plotOptions.line. Next, options for one single series are given in
#' the series array.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions}.
#'
#' @examples
#'
#' highchart() %>%
#'   hc_add_series(
#'     data = c(29.9, 71.5, 106.4, 129.2, 144.0, 176.0, 135.6, 148.5, 216.4, 194.1, 95.6, 54.4)
#'   ) %>%
#'   hc_plotOptions(
#'     line = list(
#'       color = "blue",
#'       marker = list(
#'         fillColor = "white",
#'         lineWidth = 2,
#'         lineColor = NULL
#'       )
#'     )
#'   )
#' @export
hc_plotOptions <- function(hc, ...) {
  .hc_opt(hc, "plotOptions", ...)
}

#' Responsive options for highcharter objects
#'
#' Allows setting a set of rules to apply for different screen or chart
#' sizes. Each rule specifies additional chart options.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/responsive}.
#'
#' @examples
#'
#' leg_500_opts <- list(enabled = FALSE)
#' leg_900_opts <- list(align = "right", verticalAlign = "middle", layout = "vertical")
#'
#'
#' # change the with of the container/windows to see the effect
#' highchart() %>%
#'   hc_add_series(data = cumsum(rnorm(100))) %>%
#'   hc_responsive(
#'     rules = list(
#'       # remove legend if there is no much space
#'       list(
#'         condition = list(maxWidth = 500),
#'         chartOptions = list(legend = leg_500_opts)
#'       ),
#'       # put legend on the right when there is much space
#'       list(
#'         condition = list(minWidth = 900),
#'         chartOptions = list(legend = leg_900_opts)
#'       )
#'     )
#'   )
#' @export
hc_responsive <- function(hc, ...) {
  .hc_opt(hc, "responsive", ...)
}

#' Series options for highcharter objects
#'
#' Series options for specific data and the data itself. In TypeScript you
#' have to cast the series options to specific series types, to get all
#' possible options for a series.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/series}.
#'
#' @examples
#'
#' highchart() %>%
#'   hc_series(
#'     list(
#'       name = "Tokyo",
#'       data = c(7.0, 6.9, 9.5, 14.5, 18.4, 21.5, 25.2, 26.5, 23.3, 18.3, 13.9, 9.6)
#'     ),
#'     list(
#'       name = "London",
#'       data = c(3.9, 4.2, 5.7, 8.5, 11.9, 15.2, 17.0, 16.6, 14.2, 10.3, 6.6, 4.8)
#'     )
#'   )
#' @export
hc_series <- function(hc, ...) {
  .hc_opt(hc, "series", ...)
}

#' Subtitle options for highcharter objects
#'
#' The chart's subtitle. This can be used both to display a subtitle below
#' the main title, and to display random text anywhere in the chart. The
#' subtitle can be updated after chart initialization through the
#' Chart.setTitle method.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/subtitle}.
#'
#' @examples
#'
#' highchart() %>%
#'   hc_add_series(
#'     data = c(7.0, 6.9, 9.5, 14.5, 18.2, 21.5, 25.2, 26.5, 23.3, 18.3, 13.9, 9.6),
#'     type = "column"
#'   ) %>%
#'   hc_subtitle(
#'     text = "And this is a subtitle with more information",
#'     align = "left",
#'     style = list(color = "#2b908f", fontWeight = "bold")
#'   )
#' @export
hc_subtitle <- function(hc, ...) {
  .hc_opt(hc, "subtitle", ...)
}

#' Title options for highcharter objects
#'
#' The chart's main title.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/title}.
#'
#' @examples
#'
#' highchart() %>%
#'   hc_add_series(
#'     data = c(7.0, 6.9, 9.5, 14.5, 18.2, 21.5, 25.2, 26.5, 23.3, 18.3, 13.9, 9.6),
#'     type = "column"
#'   ) %>%
#'   hc_title(
#'     text = "This is a title with <i>margin</i> and <b>Strong or bold text</b>",
#'     margin = 20,
#'     align = "left",
#'     style = list(color = "#22A884", useHTML = TRUE)
#'   )
#' @export
hc_title <- function(hc, ...) {
  .hc_opt(hc, "title", ...)
}

#' Tooltip options for highcharter objects
#'
#' Options for the tooltip that appears when the user hovers over a
#' series or point.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/tooltip}.
#' @param sort Logical value to implement sort according `this.point`
#'   \url{https://stackoverflow.com/a/16954666/829971}.
#' @param table Logical value to implement table in tooltip:
#'   \url{https://stackoverflow.com/a/22327749/829971}.
#'
#' @examples
#'
#' highchart() %>%
#'   hc_add_series(data = sample(1:12)) %>%
#'   hc_add_series(data = sample(1:12) + 10) %>%
#'   hc_tooltip(
#'     crosshairs = TRUE,
#'     borderWidth = 5,
#'     sort = TRUE,
#'     table = TRUE
#'   )
#' @export
hc_tooltip <- function(hc, ..., sort = FALSE, table = FALSE) {
  if (sort) {
    hc <- .hc_tooltip_sort(hc)
  }

  if (table) {
    hc <- .hc_tooltip_table(hc)
  }

  if (length(list(...))) {
    hc <- .hc_opt(hc, "tooltip", ...)
  }

  hc
}


#' Xaxis options for highcharter objects
#'
#' The X axis or category axis. Normally this is the horizontal axis,
#' though if the chart is inverted this is the vertical axis. In case of
#' multiple axes, the xAxis node is an array of configuration objects.
#' See the Axis class for programmatic
#' access to the axis.
#'
#' In Highmaps, the axis is hidden, but it is used behind the scenes to
#' control features like zooming and panning. Zooming is in effect the same
#' as setting the extremes of one of the exes.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/xAxis}.
#'
#' @examples
#'
#' highchart() %>%
#'   hc_add_series(
#'     data = c(7.0, 6.9, 9.5, 14.5, 18.2, 21.5, 25.2, 26.5, 23.3, 18.3, 13.9, 9.6),
#'     type = "spline"
#'   ) %>%
#'   hc_xAxis(
#'     title = list(text = "x Axis at top"),
#'     alternateGridColor = "#FDFFD5",
#'     opposite = TRUE,
#'     plotLines = list(
#'       list(
#'         label = list(text = "This is a plotLine"),
#'         color = "#FF0000",
#'         width = 2,
#'         value = 5.5
#'       )
#'     )
#'   )
#' @export
hc_xAxis <- function(hc, ...) {
  .hc_opt(hc, "xAxis", ...)
}

#' Yaxis options for highcharter objects
#'
#' The Y axis or value axis. Normally this is the vertical axis,
#' though if the chart is inverted this is the horizontal axis.
#' In case of multiple axes, the yAxis node is an array of
#' configuration objects.
#' See the Axis object for programmatic
#' access to the axis.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/yAxis}.
#'
#' @examples
#'
#' highchart() %>%
#'   hc_add_series(
#'     data = c(7.0, 6.9, 9.5, 14.5, 18.2, 21.5, 25.2, 26.5, 23.3, 18.3, 13.9, 9.6),
#'     type = "spline"
#'   ) %>%
#'   hc_yAxis(
#'     title = list(text = "y Axis at right"),
#'     opposite = TRUE,
#'     alternateGridColor = "#FAFAFA",
#'     minorTickInterval = "auto",
#'     minorGridLineDashStyle = "LongDashDotDot",
#'     showFirstLabel = FALSE,
#'     showLastLabel = FALSE,
#'     plotBands = list(
#'       list(
#'         from = 13,
#'         to = 17,
#'         color = "rgba(100, 0, 0, 0.1)",
#'         label = list(text = "This is a plotBand")
#'       )
#'     )
#'   )
#' @export
hc_yAxis <- function(hc, ...) {
  .hc_opt(hc, "yAxis", ...)
}

#' Zaxis options for highcharter objects
#'
#' The Z axis or depth axis for 3D plots.
#' See the Axis class for programmatic
#' access to the axis.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/zAxis}.
#'
#' @examples
#'
#' df <- data.frame(
#'   x = sample(1:5),
#'   y = sample(1:5),
#'   z = sample(1:5)
#' )
#'
#' # Note the 3d requiere highchart2() due have the 3d module
#' highchart2() %>%
#'   hc_add_series(data = df, "scatter3d", hcaes(x = x, y = y, z = z)) %>%
#'   hc_chart(
#'     type = "scatter3d",
#'     options3d = list(
#'       enabled = TRUE,
#'       alpha = 20,
#'       beta = 30,
#'       depth = 200,
#'       viewDistance = 5,
#'       frame = list(
#'         bottom = list(
#'           size = 1,
#'           color = "rgba(0,0,0,0.05)"
#'         )
#'       )
#'     )
#'   ) %>%
#'   hc_zAxis(
#'     title = list(text = "Z axis is here"),
#'     startOnTick = FALSE,
#'     tickInterval = 2,
#'     tickLength = 4,
#'     tickWidth = 1,
#'     gridLineColor = "red",
#'     gridLineDashStyle = "dot"
#'   )
#' @export
hc_zAxis <- function(hc, ...) {
  .hc_opt(hc, "zAxis", ...)
}

#' Navigator options for highcharter objects
#'
#' The navigator is a small series below the main series, displaying
#' a view of the entire data set. It provides tools to zoom in and
#' out on parts of the data as well as panning across the dataset.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highstock/navigator}.
#'
#' @examples
#'
#' highchart(type = "stock") %>%
#'   hc_add_series(AirPassengers) %>%
#'   hc_rangeSelector(selected = 4) %>%
#'   hc_navigator(
#'     outlineColor = "gray",
#'     outlineWidth = 2,
#'     series = list(
#'       color = "red",
#'       lineWidth = 2,
#'       type = "areaspline", # you can change the type
#'       fillColor = "rgba(255, 0, 0, 0.2)"
#'     ),
#'     handles = list(
#'       backgroundColor = "yellow",
#'       borderColor = "red"
#'     )
#'   )
#' @export
hc_navigator <- function(hc, ...) {
  .hc_opt(hc, "navigator", ...)
}

#' Rangeselector options for highcharter objects
#'
#' The range selector is a tool for selecting ranges to display within
#' the chart. It provides buttons to select preconfigured ranges in
#' the chart, like 1 day, 1 week, 1 month etc. It also provides input
#' boxes where min and max dates can be manually input.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highstock/rangeSelector}.
#'
#' @examples
#'
#' hc <- highchart(type = "stock") %>%
#'   hc_add_series(AirPassengers)
#'
#' hc
#'
#' hc %>%
#'   hc_rangeSelector(enabled = FALSE)
#'
#' hc %>%
#'   hc_rangeSelector(
#'     verticalAlign = "bottom",
#'     selected = 4
#'   )
#' @export
hc_rangeSelector <- function(hc, ...) {
  .hc_opt(hc, "rangeSelector", ...)
}

#' Scrollbar options for highcharter objects
#'
#' The scrollbar is a means of panning over the X axis of a stock chart.
#' Scrollbars can  also be applied to other types of axes.
#' Another approach to scrollable charts is the chart.scrollablePlotArea option that
#' is especially suitable for simpler cartesian charts on mobile.
#' In styled mode, all the presentational options for the
#' scrollbar are replaced by the classes .highcharts-scrollbar-thumb,
#' .highcharts-scrollbar-arrow, .highcharts-scrollbar-button,
#' .highcharts-scrollbar-rifles and .highcharts-scrollbar-track.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highstock/scrollbar}.
#'
#' @examples
#'
#' highchart(type = "stock") %>%
#'   hc_add_series(AirPassengers) %>%
#'   hc_rangeSelector(selected = 4) %>%
#'   hc_scrollbar(
#'     barBackgroundColor = "gray",
#'     barBorderRadius = 7,
#'     barBorderWidth = 0,
#'     buttonBackgroundColor = "gray",
#'     buttonBorderWidth = 0,
#'     buttonArrowColor = "yellow",
#'     buttonBorderRadius = 7,
#'     rifleColor = "yellow",
#'     trackBackgroundColor = "white",
#'     trackBorderWidth = 1,
#'     trackBorderColor = "silver",
#'     trackBorderRadius = 7
#'   )
#' @export
hc_scrollbar <- function(hc, ...) {
  .hc_opt(hc, "scrollbar", ...)
}

#' Mapnavigation options for highcharter objects
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highmaps/mapNavigation}.
#'
#' @examples
#'
#' hcmap(download_map_data = TRUE) %>%
#'   hc_mapNavigation(
#'     enabled = TRUE,
#'     enableMouseWheelZoom = TRUE,
#'     enableDoubleClickZoom = TRUE
#'   )
#' @export
hc_mapNavigation <- function(hc, ...) {
  .hc_opt(hc, "mapNavigation", ...)
}

#' Tufte theme for highcharts
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' n <- 15
#'
#' dta <- data.frame(
#'   x = 1:n + rnorm(n),
#'   y = 2 * 1:n + rnorm(n)
#' )
#'
#' highchart() %>%
#'   hc_chart(type = "scatter") %>%
#'   hc_add_series(data = list_parse(dta), showInLegend = FALSE) %>%
#'   hc_add_theme(hc_theme_tufte())
#' @export
hc_theme_tufte <- function(...) {
  theme <-
    hc_theme(
      colors = list("#737373", "#D8D7D6", "#B2B0AD", "#8C8984"),
      chart = list(
        style = list(
          fontFamily = "Cardo"
        )
      ),
      xAxis = list(
        lineWidth = 0,
        minorGridLineWidth = 0,
        lineColor = "transparent",
        tickColor = "#737373"
      ),
      yAxis = list(
        lineWidth = 0,
        minorGridLineWidth = 0,
        lineColor = "transparent",
        tickColor = "#737373",
        # extra to yAxis
        tickWidth = 1,
        gridLineColor = "transparent"
      )
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' @rdname hc_theme_538
#'
#' @examples
#'
#' highchart() %>%
#'   hc_chart(type = "column") %>%
#'   hc_add_series(data = round(1 + abs(rnorm(12)), 2), showInLegend = FALSE) %>%
#'   hc_xAxis(categories = month.abb) %>%
#'   hc_add_theme(hc_theme_tufte2())
#' @export
hc_theme_tufte2 <- function(...) {
  theme <- hc_theme_tufte(
    xAxis = list(tickWidth = 0, lineWidth = 1, lineColor = "#737373"),
    yAxis = list(tickWidth = 0, lineWidth = 1, gridLineColor = "white", gridZIndex = 4)
  )

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

validate_args <- function(name, lstargs) {
  lstargsnn <- lstargs[which(names(lstargs) == "")]
  lenlst <- length(lstargsnn)

  if (lenlst != 0) {
    chrargs <- lstargsnn %>%
      unlist() %>%
      as.character()

    chrargs <- paste0("'", chrargs, "'", collapse = ", ")

    txt <- ifelse(lenlst == 1, " is ", "s are ")

    stop(chrargs, " argument", txt, "not named in ", paste0("hc_", name),
      call. = FALSE
    )
  }
}

#' @importFrom rlist list.merge
.hc_opt <- function(hc, name, ...) {
  assertthat::assert_that(is.highchart(hc))
  
  if(!name %in% c("yAxis")) {
    validate_args(name, eval(substitute(alist(...))))  
  }

  if (is.null(hc$x$hc_opts[[name]])) {
    hc$x$hc_opts[[name]] <- list(...)
  } else {
    hc$x$hc_opts[[name]] <- list.merge(hc$x$hc_opts[[name]], list(...))
  }

  # Setting fonts
  hc$x$fonts <- unique(c(hc$x$fonts, .hc_get_fonts(hc$x$hc_opts)))

  hc
}

.hc_tooltip_table <- function(hc, ...) {
  # https://stackoverflow.com/a/22327749/829971
  hc %>%
    highcharter::hc_tooltip(
      shared = TRUE,
      useHTML = TRUE,
      headerFormat = "<small>{point.key}</small><table>",
      pointFormat = "<tr><td style=\"color: {series.color}\">{series.name}: </td><td style=\"text-align: right\"><b>{point.y}</b></td></tr>",
      footerFormat = "</table>"
    )
}

.hc_tooltip_sort <- function(hc, ...) {
  # https://stackoverflow.com/a/16954666/829971
  hc %>%
    highcharter::hc_tooltip(
      shared = TRUE,
      formatter = JS(
        "function(tooltip){
          function isArray(obj) {
          return Object.prototype.toString.call(obj) === '[object Array]';
          }

          function splat(obj) {
          return isArray(obj) ? obj : [obj];
          }

          var items = this.points || splat(this), series = items[0].series, s;

          // sort the values
          items.sort(function(a, b){
          return ((a.y < b.y) ? -1 : ((a.y > b.y) ? 1 : 0));
          });
          items.reverse();

          return tooltip.defaultFormatter.call(this, tooltip);
        }"
      )
    )
}

#' Helper to create charts in tooltips.
#'
#' @details This function needs to be used in the `pointFormatter` argument
#' inside of `hc_tooltip` function an `useHTML = TRUE` option.
#'
#' @param accesor A string indicating the name of the column where the data is.
#' @param hc_opts A list of options using the  \url{https://api.highcharts.com/highcharts/}
#'   syntax.
#' @param width	A numeric input in pixels indicating the with of the tooltip.
#' @param height	A numeric input in pixels indicating the height of the tooltip.
#'
#' @importFrom stringr str_glue
#' @importFrom htmlwidgets JS
#' @examples
#' 
#' require(dplyr)
#' require(purrr)
#' require(tidyr)
#' require(gapminder)
#' data(gapminder, package = "gapminder")
#'
#' gp <- gapminder %>%
#'   arrange(desc(year)) %>%
#'   distinct(country, .keep_all = TRUE)
#'
#' gp2 <- gapminder %>%
#'   nest(-country) %>%
#'   mutate(
#'     data = map(data, mutate_mapping, hcaes(x = lifeExp, y = gdpPercap), drop = TRUE),
#'     data = map(data, list_parse)
#'   ) %>%
#'   rename(ttdata = data)
#'
#' gptot <- left_join(gp, gp2)
#'
#' hc <- hchart(
#'   gptot,
#'   "point",
#'   hcaes(
#'     lifeExp,
#'     gdpPercap,
#'     name = country,
#'     size = pop,
#'     group = continent
#'   )
#' ) %>%
#'   hc_yAxis(type = "logarithmic")
#'
#' hc %>%
#'   hc_tooltip(useHTML = TRUE, pointFormatter = tooltip_chart(accesor = "ttdata"))
#'
#' hc %>%
#'   hc_tooltip(useHTML = TRUE, pointFormatter = tooltip_chart(
#'     accesor = "ttdata",
#'     hc_opts = list(chart = list(type = "column"))
#'   ))
#'
#' hc %>%
#'   hc_tooltip(
#'     useHTML = TRUE,
#'     positioner = JS("function () { return { x: this.chart.plotLeft + 10, y: 10}; }"),
#'     pointFormatter = tooltip_chart(
#'       accesor = "ttdata",
#'       hc_opts = list(
#'         title = list(text = "point.country"),
#'         xAxis = list(title = list(text = "lifeExp")),
#'         yAxis = list(title = list(text = "gdpPercap"))
#'       )
#'     )
#'   )
#'
#' hc %>%
#'   hc_tooltip(
#'     useHTML = TRUE,
#'     pointFormatter = tooltip_chart(
#'       accesor = "ttdata",
#'       hc_opts = list(
#'         legend = list(enabled = TRUE),
#'         series = list(list(color = "gray", name = "point.name"))
#'       )
#'     )
#'   )
#' 
#'
#' @export
tooltip_chart <- function(accesor = NULL,
                          hc_opts = NULL,
                          width = 250,
                          height = 150) {
  assertthat::assert_that(assertthat::is.string(accesor))

  if (is.null(hc_opts)) {
    hc_opts[["series"]][[1]] <- list(data = sprintf("point.%s", accesor))
  } else {
    if (!has_name(hc_opts, "series")) {
      hc_opts[["series"]][[1]] <- list()
    }
    hc_opts[["series"]][[1]][["data"]] <- sprintf("point.%s", accesor)
  }

  hc_opts <- rlist::list.merge(
    getOption("highcharter.chart")[c("title", "yAxis", "xAxis", "credits", "exporting")],
    list(chart = list(backgroundColor = "transparent")),
    list(legend = list(enabled = FALSE), plotOptions = list(series = list(animation = FALSE))),
    hc_opts
  )

  if (!has_name(hc_opts[["series"]][[1]], "color")) hc_opts[["series"]][[1]][["color"]] <- "point.color"

  hcopts <- toJSON(
    x = hc_opts, pretty = TRUE, auto_unbox = TRUE, json_verbatim = TRUE,
    force = TRUE, null = "null", na = "null"
  )
  hcopts <- as.character(hcopts)

  # fix point.color
  hcopts <- str_replace(hcopts, "\\{point.color\\}", "point.color")

  # remove "\"" to have access to the point object
  ts <- stringr::str_extract_all(hcopts, "\"point\\.\\w+\"") %>% unlist()
  for (t in ts) hcopts <- str_replace(hcopts, t, str_replace_all(t, "\"", ""))

  # remove "\"" in the options
  ts <- stringr::str_extract_all(hcopts, "\"\\w+\":") %>% unlist()
  for (t in ts) {
    t2 <- str_replace_all(t, "\"", "")
    hcopts <- str_replace(hcopts, t, t2)
  }

  jss <- "function() {{
  var point = this;
  console.log(point);
  console.log(point.{accesor});
  setTimeout(function() {{

  $(\"#tooltipchart-{id}\").highcharts(hcopts);

  }, 0);

  return '<div id=\"tooltipchart-{id}\" style=\"width: {w}px; height: {h}px;\"></div>';

  }}"

  jsss <- stringr::str_glue(jss, id = random_id(), w = width, h = height, accesor = accesor)

  jsss <- stringr::str_replace(jsss, "hcopts", hcopts)

  JS(jsss)
}

#' Helper for make table in tooltips
#'
#' Helper to make table in tooltips for the \code{pointFormat}
#' parameter in \code{hc_tooltip}
#'
#' @param x A string vector with description text
#' @param y A string with accessors example: \code{point.series.name},
#'   \code{point.x}
#' @param title A title tag with accessors or string
#' @param img Image tag
#' @param ... html attributes for the table element
#'
#' @examples
#'
#' x <- c("Income:", "Genre", "Runtime")
#' y <- c(
#'   "$ {point.y}", "{point.series.options.extra.genre}",
#'   "{point.series.options.extra.runtime}"
#' )
#'
#' tooltip_table(x, y)
#' @importFrom purrr map2
#' @importFrom htmltools tags tagList
#' @export
tooltip_table <- function(x, y,
                          title = NULL,
                          img = NULL, ...) {
  assertthat::assert_that(length(x) == length(y))

  tbl <- map2(x, y, function(x, y) {
    tags$tr(
      tags$th(x),
      tags$td(y)
    )
  })

  tbl <- tags$table(tbl, ...)

  if (!is.null(title)) {
    tbl <- tagList(title, tbl)
  }

  if (!is.null(img)) {
    tbl <- tagList(tbl, img)
  }

  as.character(tbl)
}

#' Setting \code{elementId}
#'
#' Function to modify the \code{id} for the container.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param id A string
#'
#' @examples
#'
#' hchart(rnorm(10)) %>%
#'   hc_elementId("newid")
#' @export
hc_elementId <- function(hc, id = NULL) {
  assertthat::assert_that(is.highchart(hc))

  hc$elementId <- as.character(id)

  hc
}

#' Changing the size of a `highchart` object
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param width	A numeric input in pixels.
#' @param height	A numeric input in pixels.
#'
#' @examples
#'
#' hc <- hchart(ts(rnorm(100)), showInLegend = FALSE)
#'
#' hc_size(hc, 200, 200)
#' @export
hc_size <- function(hc, width = NULL, height = NULL) {
  assertthat::assert_that(is.highchart(hc))

  if (!is.null(width)) {
    # Set for both static renderer and `shinyRenderer`,
    # where width is handled by Highcharts in the htmlwidget.
    hc$width <- width
    hc <- .hc_opt(hc, "chart", width = width)
  }

  if (!is.null(height)) {
    # Set for both static renderer and `shinyRenderer`,
    # where height is handled by Highcharts in the htmlwidget.
    hc$height <- height
    hc <- .hc_opt(hc, "chart", height = height)
  }

  hc
}

#' Setting Motion options to highcharts objects
#'
#' The Motion Highcharts Plugin adds an interactive HTML5 player
#' to any Highcharts chart (Highcharts, Highmaps and Highstock).
#'
#' @param hc A \code{highchart} \code{htmlwidget} object.
#' @param enabled Enable the motion plugin.
#' @param startIndex start index, default to 0.
#' @param ... Arguments defined in \url{https://github.com/TorsteinHonsi/Motion-Highcharts-Plugin/wiki}.
#'
#' @export
hc_motion <- function(hc, enabled = TRUE, startIndex = 0, ...) {
  hc <- .hc_opt(hc, "motion", enabled = enabled, startIndex = startIndex, ...)

  hc
}

#' @rdname hc_add_yAxis
#' @export
hc_yAxis_multiples <- function(hc, ...) {
  if (length(list(...)) == 1 & inherits(list(...)[[1]], "hc_axis_list")) {
    hc$x$hc_opts$yAxis <- list(...)[[1]]
  } else {
    hc$x$hc_opts$yAxis <- list(...)
  }
  hc
}

#' @rdname hc_add_yAxis
#' @export
hc_xAxis_multiples <- function(hc, ...) {
  if (length(list(...)) == 1 & inherits(list(...)[[1]], "hc_axis_list")) {
    hc$x$hc_opts$xAxis <- list(...)[[1]]
  } else {
    hc$x$hc_opts$xAxis <- list(...)
  }
  hc
}

#' @rdname hc_add_yAxis
#' @export
hc_zAxis_multiples <- function(hc, ...) {
  if (length(list(...)) == 1 & inherits(list(...)[[1]], "hc_axis_list")) {
    hc$x$hc_opts$zAxis <- list(...)[[1]]
  } else {
    hc$x$hc_opts$zAxis <- list(...)
  }
  hc
}



#' Creating multiples yAxis t use with highcharts
#'
#' @param naxis Number of axis an integer.
#' @param heights A numeric vector. This values will be normalized.
#' @param sep A numeric value for the separation (in percentage) for the panes.
#' @param offset A numeric value (in percentage).
#' @param turnopposite A logical value to turn the side of each axis or not.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/yAxis}.
#'
#' @examples
#'
#' highchart() %>%
#'   hc_yAxis_multiples(create_axis(naxis = 2, heights = c(2, 1))) %>%
#'   hc_add_series(data = c(1, 3, 2), yAxis = 0) %>%
#'   hc_add_series(data = c(20, 40, 10), yAxis = 1)
#'  
#' highchart() %>%
#'   hc_yAxis_multiples(create_axis(naxis = 3, lineWidth = 2, title = list(text = NULL))) %>%
#'   hc_add_series(data = c(1, 3, 2)) %>%
#'   hc_add_series(data = c(20, 40, 10), type = "area", yAxis = 1) %>%
#'   hc_add_series(data = c(200, 400, 500), yAxis = 2) %>%
#'   hc_add_series(data = c(500, 300, 400), type = "areaspline", yAxis = 2)
#'    
#' @importFrom dplyr bind_cols
#' @rdname hc_add_yAxis
#' @export
create_axis <- function(naxis = 2, heights = 1, sep = 0.01,
                         offset = 0, turnopposite = TRUE, ...) {
  pcnt <- function(x) paste0(x * 100, "%")

  heights <- rep(heights, length = naxis)

  heights <- (heights / sum(heights)) %>%
    map(function(x) c(x, sep)) %>%
    unlist() %>%
    head(-1) %>%
    {. / sum(.)} %>%
    round(5)

  tops <- cumsum(c(0, head(heights, -1)))

  tops <- pcnt(tops)
  heights <- pcnt(heights)

  dfaxis <- tibble(height = heights, top = tops, offset = offset)

  dfaxis <- dfaxis %>% dplyr::filter(seq_len(nrow(dfaxis)) %% 2 != 0)

  if (turnopposite) {
    ops <- rep_len(c(FALSE, TRUE), length.out = nrow(dfaxis))
    dfaxis <- dfaxis %>%
      mutate(opposite = ops)
  }

  dfaxis <- bind_cols(dfaxis, tibble(nid = seq(naxis), ...))

  axles <- list_parse(dfaxis)

  class(axles) <- "hc_axis_list"

  axles
}


#' @rdname hc_add_yAxis
#' @export
create_yaxis <- function(...){
  .Deprecated(msg = "Deprecated function. Use the `create_axis` function.")
  create_axis(...)
}

#' yAxis add highcharter objects
#'
#' The Y axis or value axis. Normally this is the vertical axis,
#' though if the chart is inverted this is the horizontal axis.
#' Add yAxis allows to add multiple axis with a relative height between Y axis.
#' Based upon the `relative` parameter the height of each Y axis is recalculated.
#' Otherwise the parameters are as supported by Y axis.
#'
#' @param hc A `highchart` `htmlwidget` object.
#' @param ... Arguments defined in \url{https://api.highcharts.com/highcharts/yAxis}.
#'
#' @examples
#'
#' # Retrieve stock data to plot.
#' aapl <- quantmod::getSymbols("AAPL",
#'   src = "yahoo",
#'   from = "2020-01-01",
#'   auto.assign = FALSE
#' )
#'
#' # Plot prices and volume with relative height.
#' highchart(type = "stock") %>%
#'   hc_title(text = "AAPLE") %>%
#'   hc_add_series(aapl, yAxis = 0, showInLegend = FALSE) %>%
#'   hc_add_yAxis(nid = 1L, title = list(text = "Prices"), relative = 2) %>%
#'   hc_add_series(aapl[, "AAPL.Volume"], yAxis = 1, type = "column", showInLegend = FALSE) %>%
#'   hc_add_yAxis(nid = 2L, title = list(text = "Volume"), relative = 1)
#' @export
hc_add_yAxis <- function(hc, ...) {

  # author @nordicgit70

  assertthat::assert_that(is.highchart(hc))

  # Check for single yAxis, by title attribute.
  if (assertthat::has_name(hc$x$hc_opts$yAxis, "title")) {
    # When yAxis only has empty title we can overwrite,
    # otherwise we move yAxis to become first in the list.
    if ((length(hc$x$hc_opts$yAxis) == 1) &&
      is.null(hc$x$hc_opts$yAxis$title$text)) {
      hc <- .hc_opt(hc, "yAxis", ...)
      return(hc)
    } else { # move the existing yAxis
      hc$x$hc_opts$yAxis <- list(hc$x$hc_opts$yAxis)
    }
  }

  # Add new yAxis to the list.
  validate_args("yAxis", eval(substitute(alist(...))))
  hc$x$hc_opts$yAxis <- append(hc$x$hc_opts$yAxis, list(list(...)))

  # Optional layout with relative heights.
  relative <- list(...)["relative"]
  if (!is.null(relative)) {
    # Calculate the total relative(s) and initiate offset.
    layout <- Reduce("+", lapply(hc$x$hc_opts$yAxis, function(y) {
      y$relative
    }))
    tops <- 0
    for (i in 1:length(hc$x$hc_opts$yAxis)) {
      part <- as.numeric(hc$x$hc_opts$yAxis[[i]]["relative"])
      height <- round((part * 100) / layout, 3)
      hc$x$hc_opts$yAxis[[i]]["top"] <- paste0(tops, "%")
      hc$x$hc_opts$yAxis[[i]]["height"] <- paste0(height, "%")
      hc$x$hc_opts$yAxis[[i]]["offset"] <- 0
      tops <- tops + height
    }
  }
  return(hc)
}

#' Chalk theme for highcharts
#'
#' @param ... A named parameters to modify the theme.
#'
#' Chalk theme for highcharts was inspired by
#' \url{https://www.amcharts.com/demos/}.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_chalk())
#' @importFrom grDevices colorRampPalette
#' @export
hc_theme_chalk <- function(...) {
  cols <- colorRampPalette(c("#FFFFFF", "#8C8984"))(4)

  theme <-
    list(
      colors = cols,
      chart = list(
        divBackgroundImage = "https://www.amcharts.com/inspiration/chalk/bg.jpg",
        backgroundColor = "transparent",
        style = list(
          fontFamily = "Shadows Into Light",
          color = "#FFFFFF"
        )
      ),
      plotOptions = list(
        scatter = list(
          marker = list(
            radius = 10
          )
        )
      ),
      title = list(
        style = list(
          fontSize = "30px",
          color = "#FFFFFF"
        )
      ),
      subtitle = list(
        style = list(
          fontSize = "20px",
          color = "#FFFFFF"
        )
      ),
      legend = list(
        enabled = TRUE,
        itemStyle = list(
          fontSize = "20px",
          color = "#FFFFFF"
        )
      ),
      credits = list(
        enabled = FALSE
      ),
      xAxis = list(
        lineWidth = 1,
        tickWidth = 1,
        gridLineColor = "transparent",
        labels = list(
          enabled = TRUE,
          style = list(
            color = "#FFFFFF",
            fontSize = "20px"
          )
        ),
        title = list(
          enabled = TRUE,
          style = list(
            color = "#FFFFFF",
            fontSize = "20px"
          )
        )
      ),
      yAxis = list(
        lineWidth = 1,
        tickWidth = 1,
        gridLineColor = "transparent",
        labels = list(
          enabled = TRUE,
          style = list(
            color = "#FFFFFF",
            fontSize = "20px"
          )
        ),
        title = list(
          enabled = TRUE,
          style = list(
            color = "#FFFFFF",
            fontSize = "20px"
          )
        )
      ),
      tooltip = list(
        backgroundColor = "#333333",
        style = list(
          color = "#FFFFFF",
          fontSize = "20px",
          padding = "10px"
        )
      )
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' Null theme for highcharts
#'
#' For Null theme the axis are removed (\code{visible = FALSE}).
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_null())
#' @export
hc_theme_null <- function(...) {
  theme <-
    list(
      chart = list(
        backgroundColor = "transparent"
      ),
      plotOptions = list(
        line = list(
          marker = list(
            enabled = FALSE
          )
        )
      ),
      legend = list(
        enabled = TRUE,
        align = "right",
        verticalAlign = "bottom"
      ),
      credits = list(
        enabled = FALSE
      ),
      xAxis = list(
        visible = FALSE
      ),
      yAxis = list(
        visible = FALSE
      )
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' Dotabuff theme for highcharts
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_db())
#' @export
hc_theme_db <- function(...) {
  theme <-
    list(
      colors = c("#A9CF54", "#C23C2A", "#FFFFFF", "#979797", "#FBB829"),
      chart = list(
        backgroundColor = "#242F39",
        style = list(
          color = "white"
        )
      ),
      legend = list(
        enabled = TRUE,
        align = "right",
        verticalAlign = "bottom",
        itemStyle = list(
          color = "#C0C0C0"
        ),
        itemHoverStyle = list(
          color = "#C0C0C0"
        ),
        itemHiddenStyle = list(
          color = "#444444"
        )
      ),
      title = list(
        text = NULL,
        style = list(
          color = "#FFFFFF"
        )
      ),
      tooltip = list(
        backgroundColor = "#1C242D",
        borderColor = "#1C242D",
        borderWidth = 1,
        borderRadius = 0,
        style = list(
          color = "#FFFFFF"
        )
      ),
      subtitle = list(
        style = list(
          color = "#666666"
        )
      ),
      xAxis = list(
        gridLineColor = "#2E3740",
        gridLineWidth = 1,
        labels = list(
          style = list(
            color = "#525252"
          )
        ),
        lineColor = "#2E3740",
        tickColor = "#2E3740",
        title = list(
          style = list(
            color = "#FFFFFF"
          ),
          text = NULL
        )
      ),
      yAxis = list(
        gridLineColor = "#2E3740",
        gridLineWidth = 1,
        labels = list(
          style = list(
            color = "#525252"
          ),
          lineColor = "#2E3740",
          tickColor = "#2E3740",
          title = list(
            style = list(
              color = "#FFFFFF"
            ),
            text = NULL
          )
        )
      )
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

#' Shortcut to make spkarlines
#' @param x A numeric vector.
#' @param type Type sparkline: line, bar, etc.
#' @param ... Additional arguments for the data series \url{https://api.highcharts.com/highcharts/series}.
#'
#' @examples
#'
#' set.seed(123)
#' x <- cumsum(rnorm(10))
#'
#' hcspark(x)
#' hcspark(x, "columnn")
#' hcspark(c(1, 4, 5), "pie")
#' hcspark(x, type = "area")
#' @export
hcspark <- function(x = NULL, type = NULL, ...) {
  .Deprecated(
    msg = "Use type 'hc_theme_sparkline' or hc_theme_sparkline_bv theme instead."
  )

  stopifnot(is.numeric(x))

  highchart() %>%
    hc_plotOptions(
      series = list(showInLegend = FALSE, dataLabels = list(enabled = FALSE)),
      line = list(marker = list(enabled = FALSE))
    ) %>%
    hc_add_series(data = x, type = type, ...) %>%
    hc_add_theme(hc_theme_sparkline())
}

#' Shortcut to make a boxplot
#' @param x A numeric vector.
#' @param var A string vector same length of x.
#' @param var2 A string vector same length of x.
#' @param outliers A boolean value to show or not the outliers.
#' @param ... Additional arguments for the data series \url{https://api.highcharts.com/highcharts/series}.
#' @examples
#' \dontrun{
#' hcboxplot(x = iris$Sepal.Length, var = iris$Species, color = "red")
#' }
#' @importFrom dplyr rename
#' @importFrom tidyr unnest
#' @importFrom grDevices boxplot.stats
#' @importFrom rlang .data
#' @export
hcboxplot <- function(x = NULL, var = NULL, var2 = NULL, outliers = TRUE, ...) {
  .Deprecated("data_to_boxplot")

  stopifnot(is.numeric(x))

  if (is.null(var)) {
    var <- NA
  }
  if (is.null(var2)) {
    var2 <- NA
  }

  df <- tibble(x, g1 = var, g2 = var2)

  get_box_values <- function(x) {
    boxplot.stats(x)$stats %>%
      t() %>%
      as.data.frame() %>%
      setNames(c("low", "q1", "median", "q3", "high"))
  }

  get_outliers_values <- function(x) {
    boxplot.stats(x)$out
  }

  series_box <- df %>%
    group_by(.data$g1, .data$g2) %>%
    do(data = get_box_values(.$x)) %>%
    ungroup() %>%
    unnest(cols = c(data)) %>%
    group_by(.data$g2) %>%
    do(data = list_parse(rename(select(., -.data$g2), name = .data$g1))) %>%
    mutate(type = "boxplot") %>%
    mutate(id = as.character(.data$g2))

  if (length(list(...)) > 0) {
    series_box <- add_arg_to_df(series_box, ...)
  }

  series_out <- df %>%
    group_by(.data$g1, .data$g2) %>%
    do(data = get_outliers_values(.$x)) %>%
    ungroup() %>%
    filter(map_lgl(data, ~ length(.x) != 0)) %>%
    group_by(.data$g2) %>%
    do(data = list_parse(select(., name = .data$g1, y = .data$data))) %>%
    mutate(type = "scatter") %>%
    mutate(linkedTo = as.character(.data$g2))

  if (length(list(...)) > 0) {
    series_out <- add_arg_to_df(series_out, ...)
  }

  if (!has_name(list(...), "color")) {
    colors <- colorize(seq(1, nrow(series_box)))
    colors <- hex_to_rgba(colors, alpha = 0.75)
  }

  if (!has_name(list(...), "name")) {
    series_box <- rename(series_box, name = .data$g2)
    series_out <- rename(series_out, name = .data$g2)
  }


  hc <- highchart() %>%
    hc_chart(type = "bar") %>%
    hc_xAxis(type = "category") %>%
    hc_plotOptions(series = list(
      marker = list(
        symbol = "circle"
      )
    ))

  hc <- hc_add_series_list(hc, list_parse(series_box))

  if (is.na(var2) || is.na(var)) {
    hc <- hc %>%
      hc_xAxis(categories = "") %>%
      hc_plotOptions(series = list(showInLegend = FALSE))
  }

  if (outliers) {
    hc <- hc_add_series_list(hc, list_parse(series_out))
  }

  hc
}

#' Shortcut to make icon arrays charts
#' @param labels A character vector
#' @param counts A integer vector
#' @param rows A integer to set
#' @param icons A character vector same length (o length 1) as labels
#' @param size Font size
#' @param ... Additional arguments for the data series \url{https://api.highcharts.com/highcharts/series}.
#' @importFrom dplyr ungroup group_by
#' @importFrom rlang .data
#' @export
hciconarray <- function(labels, counts, rows = NULL, icons = NULL, size = 4,
                        ...) {
  .Deprecated(
    msg = "Use type 'item' instead (`hchart(df, \"item\", hcaes(name = labels, y = counts))`).
Item chart provides better behaviour beside is a specific type of chart of HighchartsJS."
  )

  assertthat::assert_that(length(counts) == length(labels))

  if (is.null(rows)) {
    sizegrid <- n2mfrow(sum(counts))
    w <- sizegrid[1]
    h <- sizegrid[2]
  } else {
    h <- rows
    w <- ceiling(sum(counts) / rows)
  }

  ds <- tibble(x = rep(1:w, h), y = rep(1:h, each = w)) %>%
    head(sum(counts)) %>%
    mutate(y = -.data$y) %>%
    mutate(gr = rep(seq_along(labels), times = counts)) %>%
    left_join(tibble(gr = seq_along(labels), name = as.character(labels)),
      by = "gr"
    ) %>%
    group_by(.data$name) %>%
    do(data = list_parse2(tibble(.$x, .$y))) %>%
    ungroup() %>%
    left_join(tibble(labels = as.character(labels), counts),
      by = c("name" = "labels")
    ) %>%
    arrange_("-counts") %>%
    mutate(percent = .data$counts / sum(.data$counts) * 100)

  if (!is.null(icons)) {
    assertthat::assert_that(length(icons) %in% c(1, length(labels)))

    dsmrk <- ds %>%
      mutate(iconm = icons) %>%
      group_by(.data$name) %>%
      do(marker = list(symbol = fa_icon_mark(.$iconm)))

    ds <- ds %>%
      left_join(dsmrk, by = "name") %>%
      mutate(icon = fa_icon(icons))
  }

  ds <- mutate(ds, ...)

  hc <- highchart() %>%
    hc_chart(type = "scatter") %>%
    hc_add_series_list(ds) %>%
    hc_plotOptions(
      series =
        list(
          cursor = "default",
          marker = list(radius = size),
          states = list(hover = list(enabled = FALSE)),
          events = list(
            legendItemClick = JS("function () { return false; }")
          )
        )
    ) %>%
    hc_tooltip(pointFormat = "{point.series.options.counts} ({point.series.options.percent:.2f}%)") %>%
    hc_add_theme(
      hc_theme_merge(
        getOption("highcharter.theme"),
        hc_theme_null()
      )
    )

  if (!is.null(icons)) {
    hc <- hc %>% hc_add_dependency_fa()
  }

  hc
}

#' Shortcut for create treemaps
#'
#' This function helps to create highcharts treemaps from \code{treemap} objects
#' from the package \code{treemap}. NOTE: This function is deprecated. Please use \code{hctreemap2} instead.
#'
#' @param tm A \code{treemap} object from the treemap package.
#' @param ... Additional shared arguments for the data series
#'   (\url{https://api.highcharts.com/highcharts/series}).
#'
#' @examples
#' \dontrun{
#'
#' library("treemap")
#' library("viridis")
#'
#' data(GNI2014)
#' head(GNI2014)
#'
#' tm <- treemap(GNI2014,
#'   index = c("continent", "iso3"),
#'   vSize = "population", vColor = "GNI",
#'   type = "comp", palette = rev(viridis(6)),
#'   draw = FALSE
#' )
#'
#' hctreemap(tm, allowDrillToNode = TRUE, layoutAlgorithm = "squarified") %>%
#'   hc_title(text = "Gross National Income World Data") %>%
#'   hc_tooltip(pointFormat = "<b>{point.name}</b>:<br>
#'                              Pop: {point.value:,.0f}<br>
#'                              GNI: {point.valuecolor:,.0f}")
#' }
#'
#' @importFrom dplyr filter_ mutate_ rename select_ tbl_df
#' @importFrom purrr map map_df map_if
#' @importFrom rlang .data
#' @export
hctreemap <- function(tm, ...) {
  .Deprecated("data_to_hierarchical")

  assertthat::assert_that(is.list(tm))

  df <- tm$tm %>%
    tibble::as_tibble() %>%
    select(-.data$x0, -.data$y0, -.data$w, -.data$h, -.data$stdErr, -.data$vColorValue) %>%
    rename(value = .data$vSize, valuecolor = .data$vColor) %>%
    mutate_if(is.factor, as.character)

  ndepth <- which(names(df) == "value") - 1

  ds <- map_df(seq(ndepth), function(lvl) {
    df2 <- df %>%
      filter_(sprintf("level == %s", lvl)) %>%
      rename(name = names(df)[lvl]) %>%
      mutate(id = highcharter::str_to_id(.data$name))

    if (lvl > 1) {
      df2 <- df2 %>%
        mutate(
          parent = names(df)[lvl - 1],
          parent = highcharter::str_to_id(.data$parent)
        )
    } else {
      df2 <- df2 %>%
        mutate(parent = NA)
    }

    df2
  })

  ds <- list_parse(ds)

  ds <- map(ds, function(x) {
    if (is.na(x$parent)) {
      x$parent <- NULL
    }
    x
  })

  hc_add_series(highchart(), data = ds, type = "treemap", ...)
}

#' Shortcut to create treemaps.
#'
#' This function helps create highcharts treemaps from data frames.
#'
#' @param data data frame containing variables to organize each level of the treemap on
#' @param group_vars vector of strings containing column names of variables to generate treemap levels from. the first listed column will specify the top level of the treemap. the unique values in each of these columns must have no intersection (including NAs).
#' @param size_var string name of column containing numeric data to aggregate by
#' @param color_var string name of column containing numeric data to color by. defaults to same column as \code{size_var}
#' @param ... additional shared arguments for the data series
#'   (\url{https://api.highcharts.com/highcharts/series}).
#'
#' @return highchart plot object
#' @examples
#' \dontrun{
#'
#' library(tidyverse)
#' library(highcharter)
#' library(RColorBrewer)
#'
#' tibble(
#'   index1 = sample(LETTERS[1:5], 500, replace = T),
#'   index2 = sample(LETTERS[6:10], 500, replace = T),
#'   index3 = sample(LETTERS[11:15], 500, replace = T),
#'   value = rpois(500, 5),
#'   color_value = rpois(500, 5)
#' ) %>%
#'   hctreemap2(
#'     group_vars = c("index1", "index2", "index3"),
#'     size_var = "value",
#'     color_var = "color_value",
#'     layoutAlgorithm = "squarified",
#'     levelIsConstant = FALSE,
#'     levels = list(
#'       list(level = 1, dataLabels = list(enabled = TRUE)),
#'       list(level = 2, dataLabels = list(enabled = FALSE)),
#'       list(level = 3, dataLabels = list(enabled = FALSE))
#'     )
#'   ) %>%
#'   hc_colorAxis(
#'     minColor = brewer.pal(7, "Greens")[1],
#'     maxColor = brewer.pal(7, "Greens")[7]
#'   ) %>%
#'   hc_tooltip(pointFormat = "<b>{point.name}</b>:<br>
#'              Value: {point.value:,.0f}<br>
#'              Color Value: {point.colorValue:,.0f}")
#' }
#'
#' @importFrom dplyr bind_rows filter group_by mutate_at pull select summarise
#' @importFrom purrr map pmap_chr
#'
#' @export
hctreemap2 <- function(data, group_vars, size_var, color_var = NULL, ...) {
  .Deprecated("data_to_hierarchical")

  assertthat::assert_that(is.data.frame(data))
  assertthat::assert_that(is.character(group_vars))
  assertthat::assert_that(is.character(size_var))
  if (!is.null(color_var)) assertthat::assert_that(is.character(color_var))

  group_syms <- rlang::syms(group_vars)
  size_sym <- rlang::sym(size_var)
  color_sym <- rlang::sym(ifelse(is.null(color_var), size_var, color_var))

  if (data %>%
    select(!!!group_syms) %>%
    map(unique) %>%
    unlist() %>%
    anyDuplicated()) {
    stop("Treemap data uses same label at multiple levels.")
  }

  data <- data %>% mutate_at(group_vars, as.character)

  name_cell <- function(..., depth) paste0(list(...), seq_len(depth), collapse = "")

  data_at_depth <- function(depth) {
    data %>%
      group_by(!!!group_syms) %>%
      summarise(
        value = sum(!!size_sym),
        colorValue = sum(!!color_sym)
      ) %>%
      ungroup() %>%
      mutate(
        name = !!group_syms[[depth]],
        level = depth
      ) %>%
      mutate_at(group_vars, as.character()) %>%
      {
        if (depth == 1) {
          mutate(., id = paste0(name, 1))
        } else {
          mutate(
            .,
            parent = pmap_chr(
              list(!!!group_syms[seq_len(depth) - 1]),
              name_cell,
              depth = depth - 1
            ),
            id = paste0(parent, name, depth)
          )
        }
      }
  }

  treemap_df <- seq_along(group_vars) %>%
    map(data_at_depth) %>%
    bind_rows()

  data_list <- treemap_df %>%
    highcharter::list_parse() %>%
    purrr::map(~ .[!is.na(.)])

  colorVals <- treemap_df %>%
    filter(level == length(group_vars)) %>%
    pull(colorValue)

  highchart() %>%
    hc_add_series(
      data = data_list,
      type = "treemap",
      allowDrillToNode = TRUE, ...
    ) %>%
    hc_colorAxis(
      min = min(colorVals),
      max = max(colorVals),
      enabled = TRUE
    )
}

#' Shortcut to create parallel coordinates
#' @param df A data frame object.
#' @param ... Additional shared arguments for the data series
#'   (\url{https://api.highcharts.com/highcharts/series}) for the
#'   \code{hchar.data.frame} function.
#' @examples
#' require(viridisLite)
#'
#' n <- 15
#'
#' hcparcords(head(mtcars, n), color = hex_to_rgba(magma(n), 0.5))
#'
#' require(dplyr)
#' data(iris)
#' set.seed(123)
#'
#' iris <- sample_n(iris, 60)
#'
#' hcparcords(iris, color = colorize(iris$Species))
#' @importFrom dplyr mutate_if
#' @export
hcparcords <- function(df, ...) {
  stopifnot(is.data.frame(df))

  rescale01 <- function(x) {
    rng <- range(x, na.rm = TRUE)
    (x - rng[1]) / (rng[2] - rng[1])
  }

  df <- df[map_lgl(df, is.numeric)]

  # Add row identifier
  df <- rownames_to_column(df, ".row")

  df <- dplyr::mutate_if(df, is.numeric, rescale01)

  df <- tidyr::gather(df, "var", "val", setdiff(names(df), ".row"))

  hchart(df, "line", hcaes_(x = "var", y = "val", group = ".row")) %>%
    hc_plotOptions(series = list(showInLegend = FALSE)) %>%
    hc_yAxis(min = 0, max = 1) %>%
    hc_tooltip(sort = TRUE, table = TRUE, valueDecimals = 2)
}

#' Theme collection for highcharts
#'
#' Highcharts is very flexible so you can modify every element of the chart.
#' There are some exiting themes so you can apply style to charts with few
#' lines of code.
#'
#' @param ... A named parameters to modify the theme.
#'
#' @examples
#'
#' highcharts_demo() %>%
#'   hc_add_theme(hc_theme_538())
#' @export
hc_theme_538 <- function(...) {
  theme <-
    list(
      colors = c("#FF2700", "#008FD5", "#77AB43", "#636464", "#C4C4C4"),
      chart = list(
        backgroundColor = "#F0F0F0",
        plotBorderColor = "#606063",
        style = list(
          fontFamily = "Roboto",
          color = "#3C3C3C"
        )
      ),
      title = list(
        align = "left",
        style = list(
          fontWeight = "bold"
        )
      ),
      subtitle = list(
        align = "left"
      ),
      xAxis = list(
        gridLineWidth = 1,
        gridLineColor = "#D7D7D8",
        labels = list(
          style = list(
            fontFamily = "Unica One, sans-serif",
            color = "#3C3C3C"
          )
        ),
        lineColor = "#D7D7D8",
        minorGridLineColor = "#505053",
        tickColor = "#D7D7D8",
        tickWidth = 1,
        title = list(
          style = list(
            color = "#A0A0A3"
          )
        )
      ),
      yAxis = list(
        gridLineColor = "#D7D7D8",
        labels = list(
          style = list(
            fontFamily = "Unica One, sans-serif",
            color = "#3C3C3C"
          )
        ),
        lineColor = "#D7D7D8",
        minorGridLineColor = "#505053",
        tickColor = "#D7D7D8",
        tickWidth = 1,
        title = list(
          style = list(
            color = "#A0A0A3"
          )
        )
      ),
      tooltip = list(
        backgroundColor = "rgba(0, 0, 0, 0.85)",
        style = list(
          color = "#F0F0F0"
        )
      ),
      legend = list(
        itemStyle = list(
          color = "#3C3C3C"
        ),
        itemHiddenStyle = list(
          color = "#606063"
        )
      ),
      credits = list(
        style = list(
          color = "#666"
        )
      ),
      labels = list(
        style = list(
          color = "#D7D7D8"
        )
      ),
      legendBackgroundColor = "rgba(0, 0, 0, 0.5)",
      background2 = "#505053",
      dataLabelsColor = "#B0B0B3",
      textColor = "#C0C0C0",
      contrastTextColor = "#F0F0F3",
      maskColor = "rgba(255,255,255,0.3)"
    )

  theme <- structure(theme, class = "hc_theme")

  if (length(list(...)) > 0) {
    theme <- hc_theme_merge(
      theme,
      hc_theme(...)
    )
  }

  theme
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-helpers.R
\name{data_to_hierarchical}
\alias{data_to_hierarchical}
\title{Helper to transform data frame for treemap/sunburst highcharts format}
\usage{
data_to_hierarchical(
  data,
  group_vars,
  size_var,
  colors = getOption("highcharter.color_palette")
)
}
\arguments{
\item{data}{data frame containing variables to organize each level of
the treemap.}

\item{group_vars}{Variables to generate treemap levels.}

\item{size_var}{Variable to aggregate.}

\item{colors}{Color to chart every item in the first level.}
}
\description{
Helper to transform data frame for treemap/sunburst highcharts format
}
\examples{
\dontrun{

library(dplyr)
data(gapminder, package = "gapminder")

gapminder_2007 <- gapminder::gapminder \%>\%
  filter(year == max(year)) \%>\%
  mutate(pop_mm = round(pop / 1e6))

dout <- data_to_hierarchical(gapminder_2007, c(continent, country), pop_mm)

hchart(dout, type = "sunburst")

hchart(dout, type = "treemap")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_drilldown}
\alias{hc_drilldown}
\title{Drilldown options for highcharter objects}
\usage{
hc_drilldown(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/drilldown}.}
}
\description{
Options for drill down, the concept of inspecting increasingly high
resolution data through clicking on chart items like columns or pie slices.
The drilldown feature requires the drilldown.js file to be loaded,
found in the modules directory of the download package, or online at
code.highcharts.com/modules/drilldown.js.
}
\examples{

library(highcharter)
library(dplyr)
library(purrr)

df <- tibble(
  name = c("Animals", "Fruits"),
  y = c(5, 2),
  drilldown = tolower(name)
)

df

hc <- highchart() \%>\%
  hc_title(text = "Basic drilldown") \%>\%
  hc_xAxis(type = "category") \%>\%
  hc_legend(enabled = FALSE) \%>\%
  hc_plotOptions(
    series = list(
      boderWidth = 0,
      dataLabels = list(enabled = TRUE)
    )
  ) \%>\%
  hc_add_series(
    data = df,
    type = "column",
    hcaes(name = name, y = y),
    name = "Things",
    colorByPoint = TRUE
  )

dfan <- data.frame(
  name = c("Cats", "Dogs", "Cows", "Sheep", "Pigs"),
  value = c(4, 3, 1, 2, 1)
)

dffru <- data.frame(
  name = c("Apple", "Organes"),
  value = c(4, 2)
)


dsan <- list_parse2(dfan)

dsfru <- list_parse2(dffru)

hc <- hc \%>\%
  hc_drilldown(
    allowPointDrilldown = TRUE,
    series = list(
      list(
        id = "animals",
        data = dsan
      ),
      list(
        id = "fruits",
        data = dsfru
      )
    )
  )

hc
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hchart-shorcuts.R
\name{hctreemap2}
\alias{hctreemap2}
\title{Shortcut to create treemaps.}
\usage{
hctreemap2(data, group_vars, size_var, color_var = NULL, ...)
}
\arguments{
\item{data}{data frame containing variables to organize each level of the treemap on}

\item{group_vars}{vector of strings containing column names of variables to generate treemap levels from. the first listed column will specify the top level of the treemap. the unique values in each of these columns must have no intersection (including NAs).}

\item{size_var}{string name of column containing numeric data to aggregate by}

\item{color_var}{string name of column containing numeric data to color by. defaults to same column as \code{size_var}}

\item{...}{additional shared arguments for the data series
(\url{https://api.highcharts.com/highcharts/series}).}
}
\value{
highchart plot object
}
\description{
This function helps create highcharts treemaps from data frames.
}
\examples{
\dontrun{

library(tidyverse)
library(highcharter)
library(RColorBrewer)

tibble(
  index1 = sample(LETTERS[1:5], 500, replace = T),
  index2 = sample(LETTERS[6:10], 500, replace = T),
  index3 = sample(LETTERS[11:15], 500, replace = T),
  value = rpois(500, 5),
  color_value = rpois(500, 5)
) \%>\%
  hctreemap2(
    group_vars = c("index1", "index2", "index3"),
    size_var = "value",
    color_var = "color_value",
    layoutAlgorithm = "squarified",
    levelIsConstant = FALSE,
    levels = list(
      list(level = 1, dataLabels = list(enabled = TRUE)),
      list(level = 2, dataLabels = list(enabled = FALSE)),
      list(level = 3, dataLabels = list(enabled = FALSE))
    )
  ) \%>\%
  hc_colorAxis(
    minColor = brewer.pal(7, "Greens")[1],
    maxColor = brewer.pal(7, "Greens")[7]
  ) \%>\%
  hc_tooltip(pointFormat = "<b>{point.name}</b>:<br>
             Value: {point.value:,.0f}<br>
             Color Value: {point.colorValue:,.0f}")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_title}
\alias{hc_title}
\title{Title options for highcharter objects}
\usage{
hc_title(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/title}.}
}
\description{
The chart's main title.
}
\examples{

highchart() \%>\%
  hc_add_series(
    data = c(7.0, 6.9, 9.5, 14.5, 18.2, 21.5, 25.2, 26.5, 23.3, 18.3, 13.9, 9.6),
    type = "column"
  ) \%>\%
  hc_title(
    text = "This is a title with <i>margin</i> and <b>Strong or bold text</b>",
    margin = 20,
    align = "left",
    style = list(color = "#22A884", useHTML = TRUE)
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{hc_add_series.forecast}
\alias{hc_add_series.forecast}
\title{hc_add_series for forecast objects}
\usage{
\method{hc_add_series}{forecast}(
  hc,
  data,
  addOriginal = FALSE,
  addLevels = TRUE,
  fillOpacity = 0.1,
  name = NULL,
  ...
)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{data}{A \code{forecast} object.}

\item{addOriginal}{Logical value to add the original series or not.}

\item{addLevels}{Logical value to show predictions bands.}

\item{fillOpacity}{The opacity of bands.}

\item{name}{The name of the series.}

\item{...}{Arguments defined in
\url{https://api.highcharts.com/highcharts/chart}.}
}
\description{
hc_add_series for forecast objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{hc_add_series.data.frame}
\alias{hc_add_series.data.frame}
\title{hc_add_series for data frames objects}
\usage{
\method{hc_add_series}{data.frame}(hc, data, type = NULL, mapping = hcaes(), fast = FALSE, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{data}{A \code{data.frame} object.}

\item{type}{The type of the series: line, bar, etc.}

\item{mapping}{The mapping, same idea as \code{ggplot2}.}

\item{fast}{convert to json during the composition of a highchart object}

\item{...}{Arguments defined in
\url{https://api.highcharts.com/highcharts/chart}.}
}
\description{
hc_add_series for data frames objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{color_stops}
\alias{color_stops}
\title{Function to create \code{stops} argument in \code{hc_colorAxis}}
\usage{
color_stops(n = 10, colors = c("#440154", "#21908C", "#FDE725"))
}
\arguments{
\item{n}{A numeric indicating how much quantiles generate.}

\item{colors}{A character string of colors (ordered)}
}
\description{
Function to create \code{stops} argument in \code{hc_colorAxis}
}
\examples{

color_stops(5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-chalk.R
\name{hc_theme_chalk}
\alias{hc_theme_chalk}
\title{Chalk theme for highcharts}
\usage{
hc_theme_chalk(...)
}
\arguments{
\item{...}{A named parameters to modify the theme.

Chalk theme for highcharts was inspired by
\url{https://www.amcharts.com/demos/}.}
}
\description{
Chalk theme for highcharts
}
\examples{

highcharts_demo() \%>\%
  hc_add_theme(hc_theme_chalk())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{favorite_bars}
\alias{favorite_bars}
\title{Marshall's Favorite Bars}
\format{
A \code{data frame} with 5 observations and 2 variables.
}
\usage{
favorite_bars
}
\description{
Data from How I met Your Mother: Marshall's Favorite Bars.
}
\section{Variables}{


\itemize{

\item \code{bar}: Bar's name.
\item \code{percent}: In percentage of awesomeness

}
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_subtitle}
\alias{hc_subtitle}
\title{Subtitle options for highcharter objects}
\usage{
hc_subtitle(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/subtitle}.}
}
\description{
The chart's subtitle. This can be used both to display a subtitle below
the main title, and to display random text anywhere in the chart. The
subtitle can be updated after chart initialization through the
Chart.setTitle method.
}
\examples{

highchart() \%>\%
  hc_add_series(
    data = c(7.0, 6.9, 9.5, 14.5, 18.2, 21.5, 25.2, 26.5, 23.3, 18.3, 13.9, 9.6),
    type = "column"
  ) \%>\%
  hc_subtitle(
    text = "And this is a subtitle with more information",
    align = "left",
    style = list(color = "#2b908f", fontWeight = "bold")
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-helpers.R
\name{hc_elementId}
\alias{hc_elementId}
\title{Setting \code{elementId}}
\usage{
hc_elementId(hc, id = NULL)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{id}{A string}
}
\description{
Function to modify the \code{id} for the container.
}
\examples{

hchart(rnorm(10)) \%>\%
  hc_elementId("newid")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export_hc.R
\name{export_hc}
\alias{export_hc}
\title{Function to export js file the configuration options}
\usage{
export_hc(hc, filename = NULL, as = "is", name = NULL)
}
\arguments{
\item{hc}{A \code{Highcharts object}.}

\item{filename}{String of the exported file.}

\item{as}{String to define how to save the configuration options.
One of 'is', 'container', 'variable'.}

\item{name}{A variable used to put as name of the generated object if
\code{as} is \code{'variable'} and the css/js selector if is \code{as} is
container.}
}
\description{
Function to export js file the configuration options
}
\examples{

fn <- "function(){
  console.log('Category: ' + this.category);
  alert('Category: ' + this.category);
}"

hc <- highcharts_demo() \%>\%
  hc_plotOptions(
    series = list(
      cursor = "pointer",
      point = list(
        events = list(
          click = JS(fn)
        )
      )
    )
  )
\dontrun{
export_hc(hc, filename = "~/hc_is.js", as = "is")
export_hc(hc, filename = "~/hc_vr.js", as = "variable", name = "objectname")
export_hc(hc, filename = "~/hc_ct.js", as = "container", name = "#selectorid")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{mutate_mapping}
\alias{mutate_mapping}
\title{Modify data frame according to mapping}
\usage{
mutate_mapping(data, mapping, drop = FALSE)
}
\arguments{
\item{data}{A data frame object.}

\item{mapping}{A mapping from \code{hcaes} function.}

\item{drop}{A logical argument to you drop variables or not. Default is
\code{FALSE}}
}
\description{
Modify data frame according to mapping
}
\examples{

df <- head(mtcars)
mutate_mapping(data = df, mapping = hcaes(x = cyl, y = wt + cyl, group = gear))
mutate_mapping(data = df, mapping = hcaes(x = cyl, y = wt), drop = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_series}
\alias{hc_series}
\title{Series options for highcharter objects}
\usage{
hc_series(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/series}.}
}
\description{
Series options for specific data and the data itself. In TypeScript you
have to cast the series options to specific series types, to get all
possible options for a series.
}
\examples{

highchart() \%>\%
  hc_series(
    list(
      name = "Tokyo",
      data = c(7.0, 6.9, 9.5, 14.5, 18.4, 21.5, 25.2, 26.5, 23.3, 18.3, 13.9, 9.6)
    ),
    list(
      name = "London",
      data = c(3.9, 4.2, 5.7, 8.5, 11.9, 15.2, 17.0, 16.6, 14.2, 10.3, 6.6, 4.8)
    )
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{df_to_annotations_labels}
\alias{df_to_annotations_labels}
\title{Function to create annotations arguments from a data frame}
\usage{
df_to_annotations_labels(df, xAxis = 0, yAxis = 0)
}
\arguments{
\item{df}{A data frame with \code{x}, \code{y} and \code{text} columns names.}

\item{xAxis}{Index (js 0-based) of the x axis to put the annotations.}

\item{yAxis}{Index (js 0-based) of the y axis to put the annotations.}
}
\description{
Function to create annotations arguments from a data frame
}
\examples{

df <- data.frame(text = c("hi", "bye"), x = c(0, 1), y = c(1, 0))

df_to_annotations_labels(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{random_id}
\alias{random_id}
\title{Function to generate iids}
\usage{
random_id(n = 1, length = 10)
}
\arguments{
\item{n}{Number of ids}

\item{length}{Length of ids}
}
\description{
Function to generate iids
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{hc_add_series.geo_json}
\alias{hc_add_series.geo_json}
\alias{hc_add_series.geo_list}
\title{hc_add_series for geo_json & geo_list objects}
\usage{
\method{hc_add_series}{geo_json}(hc, data, type = NULL, ...)

\method{hc_add_series}{geo_list}(hc, data, type = NULL, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{data}{A \code{geo_json} or \code{geo_list} object.}

\item{type}{Type of series. Can be 'mapline', 'mapoint'.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.}
}
\description{
hc_add_series for geo_json & geo_list objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hchart-shorcuts.R
\name{hcparcords}
\alias{hcparcords}
\title{Shortcut to create parallel coordinates}
\usage{
hcparcords(df, ...)
}
\arguments{
\item{df}{A data frame object.}

\item{...}{Additional shared arguments for the data series
(\url{https://api.highcharts.com/highcharts/series}) for the
\code{hchar.data.frame} function.}
}
\description{
Shortcut to create parallel coordinates
}
\examples{
require(viridisLite)

n <- 15

hcparcords(head(mtcars, n), color = hex_to_rgba(magma(n), 0.5))

require(dplyr)
data(iris)
set.seed(123)

iris <- sample_n(iris, 60)

hcparcords(iris, color = colorize(iris$Species))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{hcaes_string}
\alias{hcaes_string}
\alias{hcaes_}
\title{Define aesthetic mappings using strings.
Similar in spirit to \code{ggplot2::aes_string}}
\usage{
hcaes_string(x, y, ...)

hcaes_(x, y, ...)
}
\arguments{
\item{x, y, ...}{List of name value pairs giving aesthetics to map to
variables. The names for x and y aesthetics are typically omitted because
they are so common; all other aesthetics must be named.}
}
\description{
Define aesthetic mappings using strings.
Similar in spirit to \code{ggplot2::aes_string}
}
\examples{
hchart(mtcars, "point", hcaes_string("hp", "mpg", group = "cyl"))

hcaes_string(x = "xval", color = "colorvar", group = "grvar")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{hc_add_series_list}
\alias{hc_add_series_list}
\title{Shortcut for data series from a list of data series}
\usage{
hc_add_series_list(hc, x)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{x}{A \code{list} or a \code{data.frame} of series.}
}
\description{
Shortcut for data series from a list of data series
}
\examples{

ds <- lapply(seq(5), function(x) {
  list(data = cumsum(rnorm(100, 2, 5)), name = x)
})

highchart() \%>\%
  hc_plotOptions(series = list(marker = list(enabled = FALSE))) \%>\%
  hc_add_series_list(ds)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{hcaes}
\alias{hcaes}
\title{Define aesthetic mappings.
Similar in spirit to \code{ggplot2::aes}}
\usage{
hcaes(x, y, ...)
}
\arguments{
\item{x, y, ...}{List of name value pairs giving aesthetics to map to
variables. The names for x and y aesthetics are typically omitted because
they are so common; all other aesthetics must be named.}
}
\description{
Define aesthetic mappings.
Similar in spirit to \code{ggplot2::aes}
}
\examples{

hcaes(x = xval, color = colorvar, group = grvar)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{color_classes}
\alias{color_classes}
\title{Function to create \code{dataClasses} argument in \code{hc_colorAxis}}
\usage{
color_classes(breaks = NULL, colors = c("#440154", "#21908C", "#FDE725"))
}
\arguments{
\item{breaks}{A numeric vector}

\item{colors}{A character string of colors (ordered)}
}
\description{
Function to create \code{dataClasses} argument in \code{hc_colorAxis}
}
\examples{

color_classes(c(0, 10, 20, 50))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{hc_add_series.numeric}
\alias{hc_add_series.numeric}
\title{\code{hc_add_series} for numeric objects}
\usage{
\method{hc_add_series}{numeric}(hc, data, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{data}{A numeric object}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.}
}
\description{
\code{hc_add_series} for numeric objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-helpers.R
\name{hc_size}
\alias{hc_size}
\title{Changing the size of a \code{highchart} object}
\usage{
hc_size(hc, width = NULL, height = NULL)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{width}{A numeric input in pixels.}

\item{height}{A numeric input in pixels.}
}
\description{
Changing the size of a \code{highchart} object
}
\examples{

hc <- hchart(ts(rnorm(100)), showInLegend = FALSE)

hc_size(hc, 200, 200)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-tufte.R
\name{hc_theme_tufte}
\alias{hc_theme_tufte}
\title{Tufte theme for highcharts}
\usage{
hc_theme_tufte(...)
}
\arguments{
\item{...}{A named parameters to modify the theme.}
}
\description{
Tufte theme for highcharts
}
\examples{

n <- 15

dta <- data.frame(
  x = 1:n + rnorm(n),
  y = 2 * 1:n + rnorm(n)
)

highchart() \%>\%
  hc_chart(type = "scatter") \%>\%
  hc_add_series(data = list_parse(dta), showInLegend = FALSE) \%>\%
  hc_add_theme(hc_theme_tufte())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-handdrawn.R
\name{hc_theme_handdrawn}
\alias{hc_theme_handdrawn}
\title{Hand Drawn theme for highcharts}
\usage{
hc_theme_handdrawn(...)
}
\arguments{
\item{...}{A named parameters to modify the theme.}
}
\description{
Hand Drawn theme for highcharts. Inspired by
\url{https://www.amcharts.com/demos/}.
}
\examples{

highcharts_demo() \%>\%
  hc_add_theme(hc_theme_handdrawn())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_navigator}
\alias{hc_navigator}
\title{Navigator options for highcharter objects}
\usage{
hc_navigator(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highstock/navigator}.}
}
\description{
The navigator is a small series below the main series, displaying
a view of the entire data set. It provides tools to zoom in and
out on parts of the data as well as panning across the dataset.
}
\examples{

highchart(type = "stock") \%>\%
  hc_add_series(AirPassengers) \%>\%
  hc_rangeSelector(selected = 4) \%>\%
  hc_navigator(
    outlineColor = "gray",
    outlineWidth = 2,
    series = list(
      color = "red",
      lineWidth = 2,
      type = "areaspline", # you can change the type
      fillColor = "rgba(255, 0, 0, 0.2)"
    ),
    handles = list(
      backgroundColor = "yellow",
      borderColor = "red"
    )
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{globaltemp}
\alias{globaltemp}
\title{globaltemp}
\format{
A \code{data frame} with 1992 observations and 4 variables.
}
\source{
\url{https://www.climate-lab-book.ac.uk/2016/spiralling-global-temperatures/}
}
\usage{
globaltemp
}
\description{
Temperature information by years.
}
\section{Variables}{


\itemize{

\item \code{date}: Date.
\item \code{lower}: Minimum temperature.
\item \code{median}: Median temperature.
\item \code{upper}: Maximum temperature.
}
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_credits}
\alias{hc_credits}
\title{Credits options for highcharter objects}
\usage{
hc_credits(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/credits}.}
}
\description{
Highchart by default puts a credits label in the lower right corner
of the chart. This can be changed using these options.
}
\examples{

highchart() \%>\%
  hc_xAxis(categories = citytemp$month) \%>\%
  hc_add_series(name = "Tokyo", data = sample(1:12)) \%>\%
  hc_credits(
    enabled = TRUE,
    text = "htmlwidgets.org",
    href = "http://www.htmlwidgets.org/"
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proxy.R
\name{hcpxy_redraw}
\alias{hcpxy_redraw}
\title{Redraw a higchartProxy object}
\usage{
hcpxy_redraw(proxy)
}
\arguments{
\item{proxy}{A \code{higchartProxy} object.}
}
\description{
Redraw a higchartProxy object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{highcharts_demo}
\alias{highcharts_demo}
\title{Chart a demo for testing themes}
\usage{
highcharts_demo()
}
\description{
Chart a demo for testing themes
}
\examples{

highcharts_demo()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-helpers.R
\name{tooltip_chart}
\alias{tooltip_chart}
\title{Helper to create charts in tooltips.}
\usage{
tooltip_chart(accesor = NULL, hc_opts = NULL, width = 250, height = 150)
}
\arguments{
\item{accesor}{A string indicating the name of the column where the data is.}

\item{hc_opts}{A list of options using the  \url{https://api.highcharts.com/highcharts/}
syntax.}

\item{width}{A numeric input in pixels indicating the with of the tooltip.}

\item{height}{A numeric input in pixels indicating the height of the tooltip.}
}
\description{
Helper to create charts in tooltips.
}
\details{
This function needs to be used in the \code{pointFormatter} argument
inside of \code{hc_tooltip} function an \code{useHTML = TRUE} option.
}
\examples{

require(dplyr)
require(purrr)
require(tidyr)
require(gapminder)
data(gapminder, package = "gapminder")

gp <- gapminder \%>\%
  arrange(desc(year)) \%>\%
  distinct(country, .keep_all = TRUE)

gp2 <- gapminder \%>\%
  nest(-country) \%>\%
  mutate(
    data = map(data, mutate_mapping, hcaes(x = lifeExp, y = gdpPercap), drop = TRUE),
    data = map(data, list_parse)
  ) \%>\%
  rename(ttdata = data)

gptot <- left_join(gp, gp2)

hc <- hchart(
  gptot,
  "point",
  hcaes(
    lifeExp,
    gdpPercap,
    name = country,
    size = pop,
    group = continent
  )
) \%>\%
  hc_yAxis(type = "logarithmic")

hc \%>\%
  hc_tooltip(useHTML = TRUE, pointFormatter = tooltip_chart(accesor = "ttdata"))

hc \%>\%
  hc_tooltip(useHTML = TRUE, pointFormatter = tooltip_chart(
    accesor = "ttdata",
    hc_opts = list(chart = list(type = "column"))
  ))

hc \%>\%
  hc_tooltip(
    useHTML = TRUE,
    positioner = JS("function () { return { x: this.chart.plotLeft + 10, y: 10}; }"),
    pointFormatter = tooltip_chart(
      accesor = "ttdata",
      hc_opts = list(
        title = list(text = "point.country"),
        xAxis = list(title = list(text = "lifeExp")),
        yAxis = list(title = list(text = "gdpPercap"))
      )
    )
  )

hc \%>\%
  hc_tooltip(
    useHTML = TRUE,
    pointFormatter = tooltip_chart(
      accesor = "ttdata",
      hc_opts = list(
        legend = list(enabled = TRUE),
        series = list(list(color = "gray", name = "point.name"))
      )
    )
  )


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharter-package.R
\docType{package}
\name{highcharter-package}
\alias{highcharter}
\alias{highcharter-package}
\title{An \code{htmlwidget} interface to the
Highcharts javascript chart library}
\description{
Highcharts \url{https://www.highcharts.com/} is a mature javascript
charting library. Highcharts provide a various type of charts, from
scatters to heatmaps or treemaps.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://jkunst.com/highcharter/}
  \item \url{https://github.com/jbkunst/highcharter}
  \item Report bugs at \url{https://github.com/jbkunst/highcharter/issues}
}

}
\author{
Joshua Kunst (@jbkunst)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{hc_add_event_point}
\alias{hc_add_event_point}
\alias{hc_add_event_series}
\title{Helpers to use highcharter as input in shiny apps}
\usage{
hc_add_event_point(hc, series = "series", event = "click")

hc_add_event_series(hc, series = "series", event = "click")
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{series}{The name of type of series to apply the event.}

\item{event}{The name of event: click, mouseOut,  mouseOver. See
\url{https://api.highcharts.com/highcharts/plotOptions.areasplinerange.point.events.select}
for more details.}
}
\description{
When you use highcharter in a shiny app, for example
\code{renderHighcharter('my_chart')}, you can access to the actions of the
user using and then use the \code{hc_add_event_point} via the
\code{my_chart} input (\code{input$my_chart}). That's a way you can
use a chart as an input.
}
\note{
Event details are accessible from hc_name_EventType, i.e. if a highchart is rendered against output$my_hc and
and we wanted the coordinates of the user-clicked point we would use input$my_hc_click
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{favorite_pies}
\alias{favorite_pies}
\title{Marshall's Favorite Pies}
\format{
A \code{data frame} with 5 observations and 2 variables.
}
\usage{
favorite_pies
}
\description{
Data from How I met Your Mother: Marshall's Favorite Pies
}
\section{Variables}{


\itemize{

\item \code{pie}: Bar's name.
\item \code{percent}: In percentage of tastiness

}
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mountains_panorama}
\alias{mountains_panorama}
\title{Visual comparison of Mountains Panorama}
\format{
A \code{data frame} with 91 observations and 3 variables.
}
\usage{
mountains_panorama
}
\description{
This data comes from the \url{https://www.highcharts.com/} examples:
https://www.highcharts.com/demo/3d-area-multiple
}
\section{Variables}{


\itemize{

\item \code{place}: The place.
\item \code{name}: Name.
\item \code{heigth}: Heigth.

}
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-smpl.R
\name{hc_theme_smpl}
\alias{hc_theme_smpl}
\title{Simple theme for highcharts}
\usage{
hc_theme_smpl(...)
}
\arguments{
\item{...}{A named parameters to modify the theme.}
}
\description{
Theme smpl design is inspired by hrbrmisc by hrbrmstr.
and color by \url{https://materialui.co/flatuicolors}.
}
\examples{

highcharts_demo() \%>\%
  hc_add_theme(hc_theme_smpl())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{pokemon}
\alias{pokemon}
\title{pokemon}
\format{
A \code{data frame} with 898 observations and 24 variables.
}
\usage{
pokemon
}
\description{
Information about 898 pokemon.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hchart-shorcuts.R
\name{hcboxplot}
\alias{hcboxplot}
\title{Shortcut to make a boxplot}
\usage{
hcboxplot(x = NULL, var = NULL, var2 = NULL, outliers = TRUE, ...)
}
\arguments{
\item{x}{A numeric vector.}

\item{var}{A string vector same length of x.}

\item{var2}{A string vector same length of x.}

\item{outliers}{A boolean value to show or not the outliers.}

\item{...}{Additional arguments for the data series \url{https://api.highcharts.com/highcharts/series}.}
}
\description{
Shortcut to make a boxplot
}
\examples{
\dontrun{
hcboxplot(x = iris$Sepal.Length, var = iris$Species, color = "red")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharter.R
\name{is.highchart}
\alias{is.highchart}
\title{Reports whether x is a highchart object}
\usage{
is.highchart(x)
}
\arguments{
\item{x}{An object to test}
}
\description{
Reports whether x is a highchart object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-helpers.R
\name{tooltip_table}
\alias{tooltip_table}
\title{Helper for make table in tooltips}
\usage{
tooltip_table(x, y, title = NULL, img = NULL, ...)
}
\arguments{
\item{x}{A string vector with description text}

\item{y}{A string with accessors example: \code{point.series.name},
\code{point.x}}

\item{title}{A title tag with accessors or string}

\item{img}{Image tag}

\item{...}{html attributes for the table element}
}
\description{
Helper to make table in tooltips for the \code{pointFormat}
parameter in \code{hc_tooltip}
}
\examples{

x <- c("Income:", "Genre", "Runtime")
y <- c(
  "$ {point.y}", "{point.series.options.extra.genre}",
  "{point.series.options.extra.runtime}"
)

tooltip_table(x, y)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{hc_theme}
\alias{hc_theme}
\title{Creating highcharter themes}
\usage{
hc_theme(...)
}
\arguments{
\item{...}{A list of named parameters.}
}
\description{
Highcharts is very flexible so you can modify every element of the chart.
There are some exiting themes so you can apply style to charts with few
lines of code.
}
\details{
More examples and details in \url{https://www.highcharts.com/docs/chart-design-and-style/themes}.
}
\examples{

hc <- highcharts_demo()

hc

thm <- hc_theme(
  colors = c("red", "green", "blue"),
  chart = list(
    backgroundColor = "#15C0DE"
  ),
  title = list(
    style = list(
      color = "#333333",
      fontFamily = "Erica One"
    )
  ),
  subtitle = list(
    style = list(
      color = "#666666",
      fontFamily = "Shadows Into Light"
    )
  ),
  legend = list(
    itemStyle = list(
      fontFamily = "Tangerine",
      color = "black"
    ),
    itemHoverStyle = list(
      color = "gray"
    )
  )
)

hc_add_theme(hc, thm)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proxy.R
\name{hcpxy_remove_point}
\alias{hcpxy_remove_point}
\title{Remove point to a series of a higchartProxy object}
\usage{
hcpxy_remove_point(proxy, id = NULL, i = NULL, redraw = TRUE)
}
\arguments{
\item{proxy}{A \code{higchartProxy} object.}

\item{id}{A character vector indicating the \code{id} of the series to update.}

\item{i}{The index of the point in the data array. Remember js is 0 based index.}

\item{redraw}{Whether to redraw the chart after the point is added. When
adding more than one point, it is highly recommended that the redraw option
be set to false, and instead Highcharts.Chart#redraw is explicitly called
after the adding of points is finished. Otherwise, the chart will redraw
after adding each point.}
}
\description{
Remove point to a series of a higchartProxy object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_yAxis}
\alias{hc_yAxis}
\title{Yaxis options for highcharter objects}
\usage{
hc_yAxis(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/yAxis}.}
}
\description{
The Y axis or value axis. Normally this is the vertical axis,
though if the chart is inverted this is the horizontal axis.
In case of multiple axes, the yAxis node is an array of
configuration objects.
See the Axis object for programmatic
access to the axis.
}
\examples{

highchart() \%>\%
  hc_add_series(
    data = c(7.0, 6.9, 9.5, 14.5, 18.2, 21.5, 25.2, 26.5, 23.3, 18.3, 13.9, 9.6),
    type = "spline"
  ) \%>\%
  hc_yAxis(
    title = list(text = "y Axis at right"),
    opposite = TRUE,
    alternateGridColor = "#FAFAFA",
    minorTickInterval = "auto",
    minorGridLineDashStyle = "LongDashDotDot",
    showFirstLabel = FALSE,
    showLastLabel = FALSE,
    plotBands = list(
      list(
        from = 13,
        to = 17,
        color = "rgba(100, 0, 0, 0.1)",
        label = list(text = "This is a plotBand")
      )
    )
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-elementary.R
\name{hc_theme_elementary}
\alias{hc_theme_elementary}
\title{Elementary (OS) theme for highcharts}
\usage{
hc_theme_elementary(...)
}
\arguments{
\item{...}{A named parameters to modify the theme.}
}
\description{
Elementary (OS) theme for highcharts was based on \url{https://elementary.io}
}
\examples{

highcharts_demo() \%>\%
  hc_add_theme(hc_theme_elementary())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_scrollbar}
\alias{hc_scrollbar}
\title{Scrollbar options for highcharter objects}
\usage{
hc_scrollbar(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highstock/scrollbar}.}
}
\description{
The scrollbar is a means of panning over the X axis of a stock chart.
Scrollbars can  also be applied to other types of axes.
Another approach to scrollable charts is the chart.scrollablePlotArea option that
is especially suitable for simpler cartesian charts on mobile.
In styled mode, all the presentational options for the
scrollbar are replaced by the classes .highcharts-scrollbar-thumb,
.highcharts-scrollbar-arrow, .highcharts-scrollbar-button,
.highcharts-scrollbar-rifles and .highcharts-scrollbar-track.
}
\examples{

highchart(type = "stock") \%>\%
  hc_add_series(AirPassengers) \%>\%
  hc_rangeSelector(selected = 4) \%>\%
  hc_scrollbar(
    barBackgroundColor = "gray",
    barBorderRadius = 7,
    barBorderWidth = 0,
    buttonBackgroundColor = "gray",
    buttonBorderWidth = 0,
    buttonArrowColor = "yellow",
    buttonBorderRadius = 7,
    rifleColor = "yellow",
    trackBackgroundColor = "white",
    trackBorderWidth = 1,
    trackBorderColor = "silver",
    trackBorderRadius = 7
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highmaps.R
\name{hc_add_series_map}
\alias{hc_add_series_map}
\title{Add a map series}
\usage{
hc_add_series_map(hc, map, df, value, joinBy, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{map}{A \code{list} object loaded from a geojson file.}

\item{df}{A \code{data.frame} object with data to chart. Code region and value are
required.}

\item{value}{A string value with the name of the variable to chart.}

\item{joinBy}{What property to join the  \code{map} and \code{df}}

\item{...}{Additional shared arguments for the data series
(\url{https://api.highcharts.com/highcharts/series}).}
}
\description{
Add a map series
}
\details{
This function force the highchart object to be map type.
}
\examples{

library("dplyr")

data("USArrests", package = "datasets")
data("usgeojson")

USArrests <- mutate(USArrests, state = rownames(USArrests))

highchart() \%>\%
  hc_title(text = "Violent Crime Rates by US State") \%>\%
  hc_subtitle(text = "Source: USArrests data") \%>\%
  hc_add_series_map(usgeojson, USArrests,
    name = "Murder arrests (per 100,000)",
    value = "Murder", joinBy = c("woename", "state"),
    dataLabels = list(
      enabled = TRUE,
      format = "{point.properties.postalcode}"
    )
  ) \%>\%
  hc_colorAxis(stops = color_stops()) \%>\%
  hc_legend(valueDecimals = 0, valueSuffix = "\%") \%>\%
  hc_mapNavigation(enabled = TRUE)
\dontrun{

data(worldgeojson, package = "highcharter")
data("GNI2014", package = "treemap")

highchart(type = "map") \%>\%
  hc_add_series_map(map = worldgeojson, df = GNI2014, value = "GNI", joinBy = "iso3") \%>\%
  hc_colorAxis(stops = color_stops()) \%>\%
  hc_tooltip(
    useHTML = TRUE, headerFormat = "",
    pointFormat = "this is {point.name} and have {point.population} people with gni of {point.GNI}"
  )
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{usgeojson}
\alias{usgeojson}
\title{US States map in Geojson format (list)}
\format{
A \code{list} in geojson format.
}
\usage{
usgeojson
}
\description{
This data comes from the \url{https://code.highcharts.com/mapdata/countries/us/us-all.js}
and is used in highmaps examples.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{str_to_id}
\alias{str_to_id}
\alias{str_to_id_vec}
\title{String to 'id' format}
\usage{
str_to_id(x)

str_to_id_vec(x)
}
\arguments{
\item{x}{A vector string.}
}
\description{
Turn a string to \code{id} format used in treemaps.
}
\examples{

str_to_id(" A string _ with sd / sdg    Underscores \   ")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{colorize}
\alias{colorize}
\title{Create vector of color from vector}
\usage{
colorize(x, colors = c("#440154", "#21908C", "#FDE725"))
}
\arguments{
\item{x}{A numeric, character or factor object.}

\item{colors}{A character string of colors (ordered) to colorize \code{x}}
}
\description{
Create vector of color from vector
}
\examples{

colorize(runif(10))

colorize(LETTERS[rbinom(20, 5, 0.5)], c("#FF0000", "#00FFFF"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-helpers.R
\name{hc_yAxis_multiples}
\alias{hc_yAxis_multiples}
\alias{hc_xAxis_multiples}
\alias{hc_zAxis_multiples}
\alias{create_axis}
\alias{create_yaxis}
\alias{hc_add_yAxis}
\title{Creating multiples yAxis t use with highcharts}
\usage{
hc_yAxis_multiples(hc, ...)

hc_xAxis_multiples(hc, ...)

hc_zAxis_multiples(hc, ...)

create_axis(
  naxis = 2,
  heights = 1,
  sep = 0.01,
  offset = 0,
  turnopposite = TRUE,
  ...
)

create_yaxis(...)

hc_add_yAxis(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/yAxis}.}

\item{naxis}{Number of axis an integer.}

\item{heights}{A numeric vector. This values will be normalized.}

\item{sep}{A numeric value for the separation (in percentage) for the panes.}

\item{offset}{A numeric value (in percentage).}

\item{turnopposite}{A logical value to turn the side of each axis or not.}
}
\description{
The Y axis or value axis. Normally this is the vertical axis,
though if the chart is inverted this is the horizontal axis.
Add yAxis allows to add multiple axis with a relative height between Y axis.
Based upon the \code{relative} parameter the height of each Y axis is recalculated.
Otherwise the parameters are as supported by Y axis.
}
\examples{

highchart() \%>\%
  hc_yAxis_multiples(create_axis(naxis = 2, heights = c(2, 1))) \%>\%
  hc_add_series(data = c(1, 3, 2), yAxis = 0) \%>\%
  hc_add_series(data = c(20, 40, 10), yAxis = 1)
 
highchart() \%>\%
  hc_yAxis_multiples(create_axis(naxis = 3, lineWidth = 2, title = list(text = NULL))) \%>\%
  hc_add_series(data = c(1, 3, 2)) \%>\%
  hc_add_series(data = c(20, 40, 10), type = "area", yAxis = 1) \%>\%
  hc_add_series(data = c(200, 400, 500), yAxis = 2) \%>\%
  hc_add_series(data = c(500, 300, 400), type = "areaspline", yAxis = 2)
   

# Retrieve stock data to plot.
aapl <- quantmod::getSymbols("AAPL",
  src = "yahoo",
  from = "2020-01-01",
  auto.assign = FALSE
)

# Plot prices and volume with relative height.
highchart(type = "stock") \%>\%
  hc_title(text = "AAPLE") \%>\%
  hc_add_series(aapl, yAxis = 0, showInLegend = FALSE) \%>\%
  hc_add_yAxis(nid = 1L, title = list(text = "Prices"), relative = 2) \%>\%
  hc_add_series(aapl[, "AAPL.Volume"], yAxis = 1, type = "column", showInLegend = FALSE) \%>\%
  hc_add_yAxis(nid = 2L, title = list(text = "Volume"), relative = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hchart.R
\name{hchart.survfit}
\alias{hchart.survfit}
\title{Plot survival curves using Highcharts}
\usage{
\method{hchart}{survfit}(
  object,
  ...,
  fun = NULL,
  markTimes = TRUE,
  symbol = "plus",
  markerColor = "black",
  ranges = FALSE,
  rangesOpacity = 0.3
)
}
\arguments{
\item{object}{A survfit object as returned from the \code{survfit} function}

\item{...}{Extra parameters to pass to \code{hc_add_series} function}

\item{fun}{Name of function or function used to transform the survival curve:
\code{log} will put y axis on log scale, \code{event} plots cumulative events
(f(y) = 1-y), \code{cumhaz} plots the cumulative hazard function (f(y) =
-log(y)), and \code{cloglog} creates a complimentary log-log survival plot
(f(y) = log(-log(y)) along with log scale for the x-axis.}

\item{markTimes}{Label curves marked at each censoring time? TRUE by default}

\item{symbol}{Symbol to use as marker (plus sign by default)}

\item{markerColor}{Color of the marker ("black" by default); use NULL to use
the respective color of each series}

\item{ranges}{Plot interval ranges? FALSE by default}

\item{rangesOpacity}{Opacity of the interval ranges (0.3 by default)}
}
\value{
Highcharts object to plot survival curves
}
\description{
Plot survival curves using Highcharts
}
\examples{

# Plot Kaplan-Meier curves
require("survival")
leukemia.surv <- survfit(Surv(time, status) ~ x, data = aml)
hchart(leukemia.surv)

# Plot the cumulative hazard function
lsurv2 <- survfit(Surv(time, status) ~ x, aml, type = "fleming")
hchart(lsurv2, fun = "cumhaz")

# Plot the fit of a Cox proportional hazards regression model
fit <- coxph(Surv(futime, fustat) ~ age, data = ovarian)
ovarian.surv <- survfit(fit, newdata = data.frame(age = 60))
hchart(ovarian.surv, ranges = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{hc_rm_series}
\alias{hc_rm_series}
\title{Removing series to highchart objects}
\usage{
hc_rm_series(hc, names = NULL)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{names}{The series's names to delete.}
}
\description{
Removing series to highchart objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_xAxis}
\alias{hc_xAxis}
\title{Xaxis options for highcharter objects}
\usage{
hc_xAxis(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/xAxis}.}
}
\description{
The X axis or category axis. Normally this is the horizontal axis,
though if the chart is inverted this is the vertical axis. In case of
multiple axes, the xAxis node is an array of configuration objects.
See the Axis class for programmatic
access to the axis.
}
\details{
In Highmaps, the axis is hidden, but it is used behind the scenes to
control features like zooming and panning. Zooming is in effect the same
as setting the extremes of one of the exes.
}
\examples{

highchart() \%>\%
  hc_add_series(
    data = c(7.0, 6.9, 9.5, 14.5, 18.2, 21.5, 25.2, 26.5, 23.3, 18.3, 13.9, 9.6),
    type = "spline"
  ) \%>\%
  hc_xAxis(
    title = list(text = "x Axis at top"),
    alternateGridColor = "#FDFFD5",
    opposite = TRUE,
    plotLines = list(
      list(
        label = list(text = "This is a plotLine"),
        color = "#FF0000",
        width = 2,
        value = 5.5
      )
    )
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proxy.R
\name{hcpxy_add_point}
\alias{hcpxy_add_point}
\title{Add point to a series of a higchartProxy object}
\usage{
hcpxy_add_point(
  proxy,
  id = NULL,
  point,
  redraw = TRUE,
  shift = FALSE,
  animation = TRUE
)
}
\arguments{
\item{proxy}{A \code{higchartProxy} object.}

\item{id}{A character vector indicating the \code{id} of the series to update.}

\item{point}{The point options. If options is a single number, a point with
that y value is appended to the series. If it is an list, it will be
interpreted as x and y values respectively. If it is an object,
advanced options as outlined under series.data are applied}

\item{redraw}{Whether to redraw the chart after the point is added. When
adding more than one point, it is highly recommended that the redraw option
be set to false, and instead Highcharts.Chart#redraw is explicitly called
after the adding of points is finished. Otherwise, the chart will redraw
after adding each point.}

\item{shift}{If \code{TRUE}, a point is shifted off the start of the series as
one is appended to the end.}

\item{animation}{Whether to apply animation, and optionally animation configuration.}
}
\description{
Add point to a series of a higchartProxy object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharter-package.R
\name{highcharter-exports}
\alias{highcharter-exports}
\alias{JS}
\alias{\%>\%}
\alias{tags}
\title{highcharter exported operators and S3 methods}
\description{
The following functions are imported and then re-exported
from the highcharter package to avoid listing the magrittr
as Depends of highcharter.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-ft.R
\name{hc_theme_ft}
\alias{hc_theme_ft}
\title{Financial Times theme for highcharts}
\usage{
hc_theme_ft(...)
}
\arguments{
\item{...}{A named parameters to modify the theme.}
}
\description{
Financial Times theme for highcharts
}
\examples{

highcharts_demo() \%>\%
  hc_add_theme(hc_theme_ft())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharter.R
\name{highchartOutput}
\alias{highchartOutput}
\alias{highchartOutput2}
\alias{highchartOutputZ}
\title{Widget output function for use in Shiny}
\usage{
highchartOutput(outputId, width = "100\%", height = "400px")

highchartOutput2(outputId, width = "100\%", height = "400px")

highchartOutputZ(outputId, width = "100\%", height = "400px")
}
\arguments{
\item{outputId}{The name of the input.}

\item{width}{A numeric input in pixels.}

\item{height}{A numeric input in pixels.}
}
\description{
Widget output function for use in Shiny
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-ffx.R
\name{hc_theme_ffx}
\alias{hc_theme_ffx}
\title{Firefox theme for highcharts}
\usage{
hc_theme_ffx(...)
}
\arguments{
\item{...}{A named parameters to modify the theme.}
}
\description{
Firefox theme was inspired by \url{https://brand.mozilla.com/all-brands}.
}
\examples{

highcharts_demo() \%>\%
  hc_add_theme(hc_theme_ffx())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highmaps.R
\name{hcmap}
\alias{hcmap}
\title{Shortcut for create map from \url{https://code.highcharts.com/mapdata/}
collection.}
\usage{
hcmap(
  map = "custom/world",
  download_map_data = getOption("highcharter.download_map_data"),
  data = NULL,
  value = NULL,
  joinBy = NULL,
  ...
)
}
\arguments{
\item{map}{String indicating what map to chart, a list from
\url{https://code.highcharts.com/mapdata/}. See examples.}

\item{download_map_data}{A logical value whether to download
(add as a dependency) the map. Default \code{TRUE} via
\code{getOption("highcharter.download_map_data")}.}

\item{data}{Optional data to make a choropleth, in case of use
the joinBy and value are needed.}

\item{value}{A string value with the name of the variable to chart.}

\item{joinBy}{What property to join the \code{map} and \code{df}.}

\item{...}{Additional shared arguments for the data series
(\url{https://api.highcharts.com/highcharts/series}).}
}
\description{
Shortcut for create map from \url{https://code.highcharts.com/mapdata/}
collection.
}
\examples{

options(highcharter.download_map_data = TRUE)

# hcmap(nullColor = "#DADADA")
# hcmap(nullColor = "#DADADA", download_map_data = FALSE)

require(dplyr)
data("USArrests", package = "datasets")
USArrests <- mutate(USArrests, "woe-name" = rownames(USArrests))

# hcmap(
#   map = "countries/us/us-all", data = USArrests,
#   joinBy = "woe-name", value = "UrbanPop", name = "Urban Population"
# )

# download_map_data = FALSE
# hcmap(
#    map = "countries/us/us-all", data = USArrests,
#   joinBy = "woe-name", value = "UrbanPop", name = "Urban Population",
#   download_map_data = FALSE
# )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-helpers.R
\name{hc_motion}
\alias{hc_motion}
\title{Setting Motion options to highcharts objects}
\usage{
hc_motion(hc, enabled = TRUE, startIndex = 0, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{enabled}{Enable the motion plugin.}

\item{startIndex}{start index, default to 0.}

\item{...}{Arguments defined in \url{https://github.com/TorsteinHonsi/Motion-Highcharts-Plugin/wiki}.}
}
\description{
The Motion Highcharts Plugin adds an interactive HTML5 player
to any Highcharts chart (Highcharts, Highmaps and Highstock).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proxy.R
\name{hcpxy_add_series}
\alias{hcpxy_add_series}
\title{Add data to higchartProxy element}
\usage{
hcpxy_add_series(proxy, data = NULL, ...)
}
\arguments{
\item{proxy}{A \code{higchartProxy} object.}

\item{data}{An R object supported by \code{hc_add_series} like data frame, ts, etc.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.}
}
\description{
Add data to higchartProxy element
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_colorAxis}
\alias{hc_colorAxis}
\title{Coloraxis options for highcharter objects}
\usage{
hc_colorAxis(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/colorAxis}.}
}
\description{
A color axis for series. Visually, the color
axis will appear as a gradient or as separate items inside the
legend, depending on whether the axis is scalar or based on data
classes.
For supported color formats, see the
docs article about colors.
A scalar color axis is represented by a gradient. The colors either
range between the minColor and the
maxColor, or for more fine grained control the
colors can be defined in stops. Often times, the
color axis needs to be adjusted to get the right color spread for the
data. In addition to stops, consider using a logarithmic
axis type, or setting min and
max to avoid the colors being determined by
outliers.
When dataClasses are used, the ranges are
subdivided into separate classes like categories based on their
values. This can be used for ranges between two values, but also for
a true category. However, when your data is categorized, it may be as
convenient to add each category to a separate series.
Color axis does not work with: sankey, sunburst, dependencywheel,
networkgraph, wordcloud, venn, gauge and solidgauge series
types.
Since v7.2.0 colorAxis can also be an array of options objects.
See the Axis object for
programmatic access to the axis.
}
\examples{

library(dplyr)

data(mpg, package = "ggplot2")

mpgman2 <- mpg \%>\%
  group_by(manufacturer, year) \%>\%
  dplyr::summarise(
    n = dplyr::n(),
    displ = mean(displ)
  )

mpgman2

hchart(
  mpgman2, "column", hcaes(x = manufacturer, y = n, group = year),
  colorKey = "displ",
  # color = c("#FCA50A", "#FCFFA4"),
  name = c("Year 1999", "Year 2008")
) \%>\%
  hc_colorAxis(min = 0, max = 5)


# defaults to yAxis
hchart(iris, "point", hcaes(Sepal.Length, Sepal.Width)) \%>\%
  hc_colorAxis(
    minColor = "red",
    maxColor = "blue"
  )



# Ex2
n <- 5

stops <- data.frame(
  q = 0:n / n,
  c = c("#440154", "#414487", "#2A788E", "#22A884", "#7AD151", "#FDE725"),
  stringsAsFactors = FALSE
)

stops <- list_parse2(stops)

M <- round(matrix(rnorm(50 * 50), ncol = 50), 2)

hchart(M) \%>\%
  hc_colorAxis(stops = stops)

# Ex3
# hchart(volcano) \%>\%
#   hc_colorAxis(stops = stops, max = 200)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proxy.R
\name{highchartProxy}
\alias{highchartProxy}
\title{Send commands to a Highcharts instance in a Shiny app}
\usage{
highchartProxy(shinyId, session = shiny::getDefaultReactiveDomain())
}
\arguments{
\item{shinyId}{Single-element character vector indicating the output ID of
the chart to modify}

\item{session}{The Shiny session object to which the map belongs; usually
the default value will suffice.}
}
\description{
Send commands to a Highcharts instance in a Shiny app
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-null.R
\name{hc_theme_null}
\alias{hc_theme_null}
\title{Null theme for highcharts}
\usage{
hc_theme_null(...)
}
\arguments{
\item{...}{A named parameters to modify the theme.}
}
\description{
For Null theme the axis are removed (\code{visible = FALSE}).
}
\examples{

highcharts_demo() \%>\%
  hc_add_theme(hc_theme_null())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_colors}
\alias{hc_colors}
\title{Colors options for highcharter objects}
\usage{
hc_colors(hc, colors)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{colors}{A vector of colors.}
}
\description{
An array containing the default colors for the chart's series. When all
colors are used, new colors are pulled from the start again.
}
\examples{

library(viridisLite)

cols <- viridis(3)
cols <- substr(cols, 0, 7)

highchart() \%>\%
  hc_add_series(data = sample(1:12)) \%>\%
  hc_add_series(data = sample(1:12) + 10) \%>\%
  hc_add_series(data = sample(1:12) + 20) \%>\%
  hc_colors(cols)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hchart-shorcuts.R
\name{hctreemap}
\alias{hctreemap}
\title{Shortcut for create treemaps}
\usage{
hctreemap(tm, ...)
}
\arguments{
\item{tm}{A \code{treemap} object from the treemap package.}

\item{...}{Additional shared arguments for the data series
(\url{https://api.highcharts.com/highcharts/series}).}
}
\description{
This function helps to create highcharts treemaps from \code{treemap} objects
from the package \code{treemap}. NOTE: This function is deprecated. Please use \code{hctreemap2} instead.
}
\examples{
\dontrun{

library("treemap")
library("viridis")

data(GNI2014)
head(GNI2014)

tm <- treemap(GNI2014,
  index = c("continent", "iso3"),
  vSize = "population", vColor = "GNI",
  type = "comp", palette = rev(viridis(6)),
  draw = FALSE
)

hctreemap(tm, allowDrillToNode = TRUE, layoutAlgorithm = "squarified") \%>\%
  hc_title(text = "Gross National Income World Data") \%>\%
  hc_tooltip(pointFormat = "<b>{point.name}</b>:<br>
                             Pop: {point.value:,.0f}<br>
                             GNI: {point.valuecolor:,.0f}")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharter.R
\name{renderHighchart}
\alias{renderHighchart}
\alias{renderHighchart2}
\alias{renderHighchartZ}
\title{Widget render function for use in Shiny}
\usage{
renderHighchart(expr, env = parent.frame(), quoted = FALSE)

renderHighchart2(expr, env = parent.frame(), quoted = FALSE)

renderHighchartZ(expr, env = parent.frame(), quoted = FALSE)
}
\arguments{
\item{expr}{A highchart expression.}

\item{env}{A environment.}

\item{quoted}{A boolean value.}
}
\description{
Widget render function for use in Shiny
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{hc_theme_merge}
\alias{hc_theme_merge}
\title{Merge themes}
\usage{
hc_theme_merge(...)
}
\arguments{
\item{...}{\code{hc_theme} objects.}
}
\description{
Function to combine hc_theme objects.
}
\examples{

thm <- hc_theme_merge(
  hc_theme_darkunica(),
  hc_theme(
    chart = list(
      backgroundColor = "transparent",
      divBackgroundImage = "http://cdn.wall-pix.net/albums/art-3Dview/00025095.jpg"
    ),
    title = list(
      style = list(
        color = "white",
        fontFamily = "Erica One"
      )
    )
  )
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_chart}
\alias{hc_chart}
\title{Chart options for highcharter objects}
\usage{
hc_chart(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/chart}.}
}
\description{
General options for the chart.
}
\examples{

hc <- highchart() \%>\%
  hc_xAxis(categories = month.abb) \%>\%
  hc_add_series(name = "Tokyo", data = sample(1:12)) \%>\%
  hc_add_series(name = "London", data = sample(1:12) + 10)

hc

hc \%>\%
  hc_chart(
    type = "column",
    options3d = list(enabled = TRUE, beta = 15, alpha = 15)
  )


hc \%>\%
  hc_chart(
    borderColor = "#EBBA95",
    borderRadius = 10,
    borderWidth = 2,
    backgroundColor = list(
      linearGradient = c(0, 0, 500, 500),
      stops = list(
        list(0, "rgb(255, 255, 255)"),
        list(1, "rgb(200, 200, 255)")
      )
    )
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{hc_add_theme}
\alias{hc_add_theme}
\title{Add themes to a highchart object}
\usage{
hc_add_theme(hc, hc_thm)
}
\arguments{
\item{hc}{A highchart object}

\item{hc_thm}{A highchart theme object (\code{"hc_theme"} class)}
}
\description{
Add highcharts themes to a highchart object.
}
\examples{

highchart() \%>\%
  hc_add_series(
    data = c(
      7.0, 6.9, 9.5, 14.5, 18.2, 21.5, 25.2,
      26.5, 23.3, 18.3, 13.9, 9.6
    ),
    type = "column"
  ) \%>\%
  hc_add_theme(hc_theme_sandsignika())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hchart.R
\name{hchart.igraph}
\alias{hchart.igraph}
\title{Plot igraph objects using Highcharts}
\usage{
\method{hchart}{igraph}(object, ..., layout = NULL)
}
\arguments{
\item{object}{An igraph object.}

\item{...}{Additional arguments for the data series
(\url{https://api.highcharts.com/highcharts/series}).}

\item{layout}{A layout from igraph package.}
}
\description{
Plot igraph objects using Highcharts
}
\examples{

if(require("igraph")) {

N <- 40
net <- sample_gnp(N, p = 2 / N)
wc <- cluster_walktrap(net)
V(net)$label <- seq(N)
V(net)$name <- paste("I'm #", seq(N))
V(net)$page_rank <- round(page_rank(net)$vector, 2)
V(net)$betweenness <- round(betweenness(net), 2)
V(net)$degree <- degree(net)
V(net)$size <- V(net)$degree
V(net)$comm <- as.vector(membership(wc))
V(net)$color <- colorize(membership(wc))
hchart(net, layout = layout_with_fr)

}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highmaps.R
\name{get_data_from_map}
\alias{get_data_from_map}
\title{Helper function to get the data inside the map data
The urls are listed in \url{https://code.highcharts.com/mapdata/}.}
\usage{
get_data_from_map(mapdata)
}
\arguments{
\item{mapdata}{A list obtained from \code{\link{download_map_data}}.}
}
\description{
Helper function to get the data inside the map data
The urls are listed in \url{https://code.highcharts.com/mapdata/}.
}
\examples{
\dontrun{
dta <- download_map_data("https://code.highcharts.com/mapdata/countries/us/us-ca-all.js")
get_data_from_map(dta)
}
}
\seealso{
\code{\link{download_map_data}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-helpers.R
\name{data_to_sankey}
\alias{data_to_sankey}
\title{Helper to transform data frame for sankey highcharts format}
\usage{
data_to_sankey(data = NULL)
}
\arguments{
\item{data}{A data frame}
}
\description{
Helper to transform data frame for sankey highcharts format
}
\examples{
\dontrun{
library(dplyr)
data(diamonds, package = "ggplot2")

diamonds2 <- select(diamonds, cut, color, clarity)

data_to_sankey(diamonds2)

hchart(data_to_sankey(diamonds2), "sankey", name = "diamonds")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proxy.R
\name{hcpxy_update_point}
\alias{hcpxy_update_point}
\title{Update options series in a higchartProxy object}
\usage{
hcpxy_update_point(proxy, id = NULL, id_point = NULL, ...)
}
\arguments{
\item{proxy}{A \code{higchartProxy} object.}

\item{id}{A character indicating the \code{id} of the series' point to update.}

\item{id_point}{A vector value indicating the point's index to update, (0 based).}

\item{...}{Arguments defined in \url{https://api.highcharts.com/class-reference/Highcharts.Point}.
The arguments will be the same for each series. So if you want update data it
is used this function sequentially for each point}
}
\description{
Update options series in a higchartProxy object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highmaps.R
\name{download_map_data}
\alias{download_map_data}
\title{Helper function to download the map data form a url}
\usage{
download_map_data(
  url = "custom/world.js",
  showinfo = FALSE,
  quiet = FALSE,
  method = "curl"
)
}
\arguments{
\item{url}{The map's url.}

\item{showinfo}{Show the properties of the downloaded map to know how
are the keys to add data in \code{hcmap}.}

\item{quiet}{Boolean parameter to turn off download messages (on by default).}

\item{method}{A string value for the download method used by
\code{\link[utils]{download.file}}.}
}
\description{
The urls are listed in \url{https://code.highcharts.com/mapdata/}.
}
\examples{
\dontrun{
mpdta <- download_map_data("https://code.highcharts.com/mapdata/countries/us/us-ca-all.js")
mpdta <- download_map_data("https://code.highcharts.com/mapdata/countries/us/us-ca-all.js",
  quiet = TRUE
)
str(mpdta, 1)
}
}
\seealso{
\code{\link{hcmap}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proxy.R
\name{hcpxy_update}
\alias{hcpxy_update}
\title{Update options for a higchartProxy object}
\usage{
hcpxy_update(proxy, ...)
}
\arguments{
\item{proxy}{A \code{higchartProxy} object.}

\item{...}{Named options.}
}
\description{
Update options for a higchartProxy object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharter.R
\name{highchart2}
\alias{highchart2}
\alias{highchartzero}
\title{Create a Highcharts chart widget}
\usage{
highchart2(
  hc_opts = list(),
  theme = getOption("highcharter.theme"),
  type = "chart",
  width = NULL,
  height = NULL,
  elementId = NULL,
  google_fonts = getOption("highcharter.google_fonts")
)

highchartzero(
  hc_opts = list(),
  theme = NULL,
  width = NULL,
  height = NULL,
  elementId = NULL
)
}
\arguments{
\item{hc_opts}{A \code{list} object containing options defined as
\url{https://api.highcharts.com/highcharts/}.}

\item{theme}{A \code{hc_theme} class object.}

\item{type}{A character value to set if use Highchart, Highstock or
Highmap. Options are \code{"chart"}, \code{"stock"} and \code{"map"}.}

\item{width}{A numeric input in pixels.}

\item{height}{A numeric input in pixels.}

\item{elementId}{Use an explicit element ID for the widget.}

\item{google_fonts}{A boolean value. If TRUE (default), adds a reference to the
Google Fonts API to the HTML head, downloading CSS for the font families
defined in the Highcharts theme from https://fonts.googleapis.com. Set to
FALSE if you load your own fonts using CSS.}
}
\description{
This widgets don't support options yet.
}
\details{
This function creates a Highchart chart using \pkg{htmlwidgets}. The
widget can be rendered on HTML pages generated from R Markdown, Shiny, or
other applications.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proxy.R
\name{hcpxy_loading}
\alias{hcpxy_loading}
\title{Show or hide loading text for a higchartProxy object}
\usage{
hcpxy_loading(proxy, action = "show")
}
\arguments{
\item{proxy}{A \code{higchartProxy} object.}

\item{action}{Single-element character vector indicating to \code{"show"} or
\code{"hide"} the loading text defined in \code{lang} options.}
}
\description{
Show or hide loading text for a higchartProxy object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-sparkline.R
\name{hc_theme_sparkline}
\alias{hc_theme_sparkline}
\title{Sparkline theme for highcharts}
\usage{
hc_theme_sparkline(...)
}
\arguments{
\item{...}{A named parameters to modify the theme.}
}
\description{
Sparkline theme is based on \url{https://www.highcharts.com/demo/sparkline}
and this post  \url{https://jkunst.com/blog/posts/2020-06-26-valuebox-and-sparklines/}.
}
\examples{

highcharts_demo() \%>\%
  hc_add_theme(hc_theme_sparkline())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_legend}
\alias{hc_legend}
\title{Legend options for highcharter objects}
\usage{
hc_legend(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/legend}.}
}
\description{
The legend is a box containing a symbol and name for each series
item or point item in the chart. Each series (or points in case
of pie charts) is represented by a symbol and its name in the legend.
It is possible to override the symbol creator function and create
custom legend symbols.
}
\details{
A Highmaps legend by default contains one legend item per series, but if
a colorAxis is defined, the axis will be displayed in the legend.
Either as a gradient, or as multiple legend items for dataClasses.
}
\examples{

highchart() \%>\%
  hc_xAxis(categories = month.abb) \%>\%
  hc_add_series(name = "Tokyo", data = sample(1:12)) \%>\%
  hc_add_series(name = "London", data = sample(1:12) + 10) \%>\%
  hc_add_series(name = "Other City", data = sample(1:12) + 20) \%>\%
  hc_legend(
    align = "left",
    verticalAlign = "top",
    layout = "vertical",
    x = 0,
    y = 100
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icons.R
\name{hc_add_dependency_fa}
\alias{hc_add_dependency_fa}
\alias{fa_icon}
\alias{fa_icon_mark}
\title{Helpers functions to get FontAwesome icons code}
\usage{
hc_add_dependency_fa(hc)

fa_icon(iconname = "circle")

fa_icon_mark(iconname = "circle")
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{iconname}{The icon's name}
}
\description{
Helpers functions to get FontAwesome icons code
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-flat.R
\name{hc_theme_flat}
\alias{hc_theme_flat}
\title{Flat theme for highcharts}
\usage{
hc_theme_flat(...)
}
\arguments{
\item{...}{A named parameters to modify the theme.}
}
\description{
Flat and flatdark theme is inspired by
\url{https://github.com/chriskempson/base16} and
\url{https://github.com/Mikata-Project/ggthemr#flat}
}
\examples{

highcharts_demo() \%>\%
  hc_add_theme(hc_theme_flat())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_mapNavigation}
\alias{hc_mapNavigation}
\title{Mapnavigation options for highcharter objects}
\usage{
hc_mapNavigation(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highmaps/mapNavigation}.}
}
\description{
Mapnavigation options for highcharter objects
}
\examples{

hcmap(download_map_data = TRUE) \%>\%
  hc_mapNavigation(
    enabled = TRUE,
    enableMouseWheelZoom = TRUE,
    enableDoubleClickZoom = TRUE
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{hex_to_rgba}
\alias{hex_to_rgba}
\title{Transform colors from hexadecimal format to rgba hc notation}
\usage{
hex_to_rgba(x, alpha = 1)
}
\arguments{
\item{x}{colors in hexadecimal format}

\item{alpha}{alpha}
}
\description{
Transform colors from hexadecimal format to rgba hc notation
}
\examples{

hex_to_rgba(x <- c("#440154", "#21908C", "#FDE725"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_plotOptions}
\alias{hc_plotOptions}
\title{Plotoptions options for highcharter objects}
\usage{
hc_plotOptions(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions}.}
}
\description{
The plotOptions is a wrapper object for config objects for each series
type. The config objects for each series can also be overridden for
each series item as given in the series array.
Configuration options for the series are given in three levels. Options
for all series in a chart are given in the plotOptions.series object. Then options for all series of a specific
type are given in the plotOptions of that type, for example
plotOptions.line. Next, options for one single series are given in
the series array.
}
\examples{

highchart() \%>\%
  hc_add_series(
    data = c(29.9, 71.5, 106.4, 129.2, 144.0, 176.0, 135.6, 148.5, 216.4, 194.1, 95.6, 54.4)
  ) \%>\%
  hc_plotOptions(
    line = list(
      color = "blue",
      marker = list(
        fillColor = "white",
        lineWidth = 2,
        lineColor = NULL
      )
    )
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{unemployment}
\alias{unemployment}
\title{US Counties unemployment rate}
\format{
A \code{data.frame} with 3 variables and 3.216 observations.
}
\usage{
unemployment
}
\description{
This data comes from the highcharts and is used in highmaps examples.
}
\section{Variables}{


\itemize{

\item \code{code}: The county code.
\item \code{name}: The county name.
\item \code{value}: The unemployment.

}
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_rangeSelector}
\alias{hc_rangeSelector}
\title{Rangeselector options for highcharter objects}
\usage{
hc_rangeSelector(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highstock/rangeSelector}.}
}
\description{
The range selector is a tool for selecting ranges to display within
the chart. It provides buttons to select preconfigured ranges in
the chart, like 1 day, 1 week, 1 month etc. It also provides input
boxes where min and max dates can be manually input.
}
\examples{

hc <- highchart(type = "stock") \%>\%
  hc_add_series(AirPassengers)

hc

hc \%>\%
  hc_rangeSelector(enabled = FALSE)

hc \%>\%
  hc_rangeSelector(
    verticalAlign = "bottom",
    selected = 4
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{citytemp}
\alias{citytemp}
\title{City temperatures from a year in wide format}
\format{
A \code{data frame} with 12 observations and 5 variables.
}
\usage{
citytemp
}
\description{
This data comes from the \url{https://www.highcharts.com/} examples.
}
\section{Variables}{


\itemize{

\item \code{month}: The months.
\item \code{tokyo}: Tokyo's temperatures.
\item \code{new_york}: New York's temperatures.
\item \code{berlin}: Berlin's temperatures.
\item \code{london}: London's temperatures.

}
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proxy.R
\name{hcpxy_set_data}
\alias{hcpxy_set_data}
\title{Update data for a higchartProxy object}
\usage{
hcpxy_set_data(
  proxy,
  type,
  data,
  mapping = hcaes(),
  redraw = FALSE,
  animation = NULL,
  updatePoints = TRUE
)
}
\arguments{
\item{proxy}{A \code{higchartProxy} object.}

\item{type}{series type (column, bar, line, etc)}

\item{data}{dataframe of new data to send to chart}

\item{mapping}{how data should be mapped using \code{hcaes()}}

\item{redraw}{boolean Whether to redraw the chart after the series is altered.
If doing more operations on the chart, it is a good idea to set redraw to false and call hcpxy_redraw after.}

\item{animation}{boolean When the updated data is the same length as the existing data, points will be updated by default,
and animation visualizes how the points are changed. Set false to disable animation, or a configuration object to set duration or easing.}

\item{updatePoints}{boolean When this is TRUE, points will be updated instead of replaced whenever possible.
This occurs a) when the updated data is the same length as the existing data, b) when points are matched by their id's, or c) when points can be matched by X values.
This allows updating with animation and performs better. In this case, the original array is not passed by reference. Set FALSE to prevent.}
}
\description{
Update data for a higchartProxy object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proxy.R
\name{hcpxy_remove_series}
\alias{hcpxy_remove_series}
\title{Remove series to higchartProxy element}
\usage{
hcpxy_remove_series(proxy, id = NULL, all = FALSE)
}
\arguments{
\item{proxy}{A \code{higchartProxy} object.}

\item{id}{A character vector indicating the \code{id} (or \code{id}s) of the series to remove.}

\item{all}{A logical value to indicate to remove or not all series. The values
is used only when the value is \code{TRUE}.}
}
\description{
Remove series to higchartProxy element
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_exporting}
\alias{hc_exporting}
\title{Exporting options for highcharter objects}
\usage{
hc_exporting(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/exporting}.}
}
\description{
Options for the exporting module. For an overview on the matter, see
the docs.
}
\examples{

highchart() \%>\%
  hc_xAxis(categories = month.abb) \%>\%
  hc_add_series(name = "Tokyo", data = sample(1:12)) \%>\%
  hc_exporting(
    enabled = TRUE, # always enabled
    filename = "custom-file-name"
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_responsive}
\alias{hc_responsive}
\title{Responsive options for highcharter objects}
\usage{
hc_responsive(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/responsive}.}
}
\description{
Allows setting a set of rules to apply for different screen or chart
sizes. Each rule specifies additional chart options.
}
\examples{

leg_500_opts <- list(enabled = FALSE)
leg_900_opts <- list(align = "right", verticalAlign = "middle", layout = "vertical")


# change the with of the container/windows to see the effect
highchart() \%>\%
  hc_add_series(data = cumsum(rnorm(100))) \%>\%
  hc_responsive(
    rules = list(
      # remove legend if there is no much space
      list(
        condition = list(maxWidth = 500),
        chartOptions = list(legend = leg_500_opts)
      ),
      # put legend on the right when there is much space
      list(
        condition = list(minWidth = 900),
        chartOptions = list(legend = leg_900_opts)
      )
    )
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{weather}
\alias{weather}
\title{Weather}
\format{
A \code{data frame} with 365 observations and 4 variables.
}
\usage{
weather
}
\description{
Temperature information of San Francisco.
}
\section{Variables}{


\itemize{

\item \code{date}: Day in date format.
\item \code{min_temperaturec}: Minimum temperature.
\item \code{max_temperaturec}: Maximun temperature.
\item \code{mean_temperaturec}: Mean temperature.
}
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hw_grid.R
\name{hw_grid}
\alias{hw_grid}
\title{Lays out highchart widgets into a "grid", similar to
\code{grid.arrange} from \code{gridExtra}.}
\usage{
hw_grid(
  ...,
  ncol = NULL,
  rowheight = NULL,
  add_htmlgrid_css = TRUE,
  browsable = TRUE
)
}
\arguments{
\item{...}{either individual \code{highchart} objects or a mixture of
individual \code{highchart} objects and \code{list}s of \code{highchart} objects.}

\item{ncol}{how many columns in the grid}

\item{rowheight}{Height in px.}

\item{add_htmlgrid_css}{A logical value to add or not \code{htmlgrid.css} as
dependency.}

\item{browsable}{Logical value indicating if the returned object is converted
to an HTML object browsable using \code{htmltools::browsable}.}
}
\description{
Lays out highchart widgets into a "grid", similar to
\code{grid.arrange} from \code{gridExtra}.
}
\examples{

charts <- lapply(1:9, function(x) {
  hchart(ts(cumsum(rnorm(100))))
})

if (interactive()) {
  hw_grid(charts, rowheight = 300)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_boost}
\alias{hc_boost}
\title{Boost options for highcharter objects}
\usage{
hc_boost(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/boost}.}
}
\description{
Options for the Boost module. The Boost module allows certain series types
to be rendered by WebGL instead of the default SVG. This allows hundreds of
thousands of data points to be rendered in milliseconds. In addition to the
WebGL rendering it saves time by skipping processing and inspection of the
data wherever possible. This introduces some limitations to what features are
available in boost mode. See the docs for
details.
In addition to the global boost option, each series has a
boostThreshold that defines when the
boost should kick in.
Requires the modules/boost.js module.
}
\examples{

#  Ex 1
options(highcharter.rjson = FALSE)

n <- 50000

x <- sin(4 * 2 * pi * seq(n) / n) + rnorm(n) / 10

x <- round(x, 3)

plot(x)

hc1 <- highchart() \%>\%
  hc_chart(zoomType = "x") \%>\%
  hc_add_series(data = x) \%>\%
  hc_title(text = "No boost") \%>\%
  hc_boost(
    enabled = FALSE # Default
  )

hc1

# Boost is a stripped-down renderer-in-a-module for Highcharts. It bypasses
# some of the standard Highcharts features (such as animation), and focuses
# on pushing as many points as possible as quickly as possible.

hc2 <- highchart() \%>\%
  hc_chart(zoomType = "x") \%>\%
  hc_add_series(data = x) \%>\%
  hc_title(text = "With boost") \%>\%
  hc_boost(enabled = TRUE)

hc2


# # Ex 2
# library(MASS)
#
# n <- 20000
#
# sigma <- matrix(c(10,3,3,2),2,2)
# sigma
#
# mvr <- round(mvrnorm(n, rep(c(0, 0)), sigma), 2)
#
# vx <- ceiling(1+abs(max(mvr[, 1])))
# vy <- ceiling(1+abs(max(mvr[, 2])))
#
# # unnamed list
# ds <- list_parse2(as.data.frame(mvr))
#
# highchart() \%>\%
#   hc_chart(zoomType = "xy") \%>\%
#   hc_xAxis(min = -vx, max = vx) \%>\%
#   hc_yAxis(min = -vy, max = vy) \%>\%
#   hc_add_series(
#     data = ds, #list
#     type = "scatter",
#     name = "A lot of points!",
#     color = 'rgba(0,0,0,0.1)',
#     marker = list(radius = 2)
#     ) \%>\%
#   hc_boost(
#     enabled = TRUE
#   )
#
# dat <- as.data.frame(mvr)
# names(dat) <- c("x", "y")
#
# highchart() \%>\%
#   hc_chart(zoomType = "xy") \%>\%
#   hc_xAxis(min = -vx, max = vx) \%>\%
#   hc_yAxis(min = -vy, max = vy) \%>\%
#   hc_add_series(
#     data = dat,
#     type = "scatter",
#     hcaes(x, y),
#     name = "A lot of points!",
#     color = 'rgba(0,0,0,0.1)',
#     marker = list(radius = 2)
#   ) \%>\%
#   hc_boost(enabled = TRUE)
#
# # Ex3
# N <- 1000000
# n <- 5
# s <- seq(n)
# s <- s/(max(s) + min(s))
# s <- round(s, 2)
#
# series <- s \%>\%
#   purrr::map(~ stats::arima.sim(round(N/n), model = list(ar = .x)) + .x * n * 20) \%>\%
#   purrr::map(as.vector) \%>\%
#   purrr::map(round, 2) \%>\%
#   purrr::map(~ list(data = .x))
#
# highchart() \%>\%
#   hc_add_series_list(series) \%>\%
#   hc_chart(zoomType = "x") \%>\%
#   hc_boost(enabled = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{hc_add_series}
\alias{hc_add_series}
\title{Adding data to highchart objects}
\usage{
hc_add_series(hc, data = NULL, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{data}{An R object like numeric, list, ts, xts, etc.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.}
}
\description{
Adding data to highchart objects
}
\examples{

highchart() \%>\%
  hc_add_series(data = abs(rnorm(5)), type = "column") \%>\%
  hc_add_series(data = purrr::map(0:4, function(x) list(x, x)), type = "scatter", color = "orange")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hchart-shorcuts.R
\name{hcspark}
\alias{hcspark}
\title{Shortcut to make spkarlines}
\usage{
hcspark(x = NULL, type = NULL, ...)
}
\arguments{
\item{x}{A numeric vector.}

\item{type}{Type sparkline: line, bar, etc.}

\item{...}{Additional arguments for the data series \url{https://api.highcharts.com/highcharts/series}.}
}
\description{
Shortcut to make spkarlines
}
\examples{

set.seed(123)
x <- cumsum(rnorm(10))

hcspark(x)
hcspark(x, "columnn")
hcspark(c(1, 4, 5), "pie")
hcspark(x, type = "area")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{stars}
\alias{stars}
\title{stars}
\format{
A \code{data frame} with 404 observations and 6 variables.
}
\usage{
stars
}
\description{
A sample using by Nadieh Bremer blocks. \url{http://bl.ocks.org/nbremer/eb0d1fd4118b731d069e2ff98dfadc47}.
}
\section{Variables}{


\itemize{

\item \code{bv}: BV
\item \code{absmag}: Magnitude
\item \code{lum}: Luminosity
\item \code{temp}: Temperature
\item \code{radiussun}: Radius
\item \code{distance}: Distance
}
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-ggplot2.R
\name{hc_theme_ggplot2}
\alias{hc_theme_ggplot2}
\title{ggplot2 theme for highcharts}
\usage{
hc_theme_ggplot2(...)
}
\arguments{
\item{...}{A named parameters to modify the theme.}
}
\description{
ggplot2 theme is based on \url{https://ggplot2.tidyverse.org/}.
}
\examples{

highcharts_demo() \%>\%
  hc_add_theme(hc_theme_ggplot2())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{datetime_to_timestamp}
\alias{datetime_to_timestamp}
\alias{dt_tstp}
\title{Date to timestamps}
\usage{
datetime_to_timestamp(dt)

dt_tstp(dt)
}
\arguments{
\item{dt}{Date or datetime vector}
}
\description{
Turn a date time vector to \code{timestamp} format
}
\examples{

datetime_to_timestamp(
  as.Date(c("2015-05-08", "2015-09-12"),
    format = "\%Y-\%m-\%d"
  )
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{hc_add_annotation}
\alias{hc_add_annotation}
\alias{hc_add_annotations}
\title{Helper to add annotations from data frame or list}
\usage{
hc_add_annotation(hc, ...)

hc_add_annotations(hc, x)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/annotations}.}

\item{x}{A \code{list} or a \code{data.frame} of annotations.}
}
\description{
Helper to add annotations from data frame or list
}
\details{
The \code{x} elements must have \code{xValue} and \code{yValue}
elements
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{hc_add_dependency}
\alias{hc_add_dependency}
\title{Add modules or plugin dependencies to highcharts objects}
\usage{
hc_add_dependency(hc, name = "plugins/annotations.js")
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{name}{The partial path to the plugin or module,
example: \code{"plugins/annotations.js"}}
}
\description{
Add modules or plugin dependencies to highcharts objects
}
\details{
See \code{vignette("modules")}
}
\examples{

data(mpg, package = "ggplot2")

hchart(mpg, "point", hcaes(displ, hwy),
  regression = TRUE,
  regressionSettings = list(type = "polynomial", order = 5, hideInLegend = TRUE)
) \%>\%
  hc_add_dependency("plugins/highcharts-regression.js")

hchart(mpg, "point", hcaes(displ, hwy, group = drv), regression = TRUE) \%>\%
  hc_colors(c("#d35400", "#2980b9", "#2ecc71")) \%>\%
  hc_add_dependency("plugins/highcharts-regression.js")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-google.R
\name{hc_theme_google}
\alias{hc_theme_google}
\title{Google theme for highcharts}
\usage{
hc_theme_google(...)
}
\arguments{
\item{...}{A named parameters to modify the theme.}
}
\description{
Google theme for highcharts is based on \url{https://books.google.com/ngrams/}.
}
\examples{

highcharts_demo() \%>\%
  hc_add_theme(hc_theme_google())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{get_hc_series_from_df}
\alias{get_hc_series_from_df}
\title{Auxiliar function to get series and options from tidy frame for hchart.data.frame}
\usage{
get_hc_series_from_df(data, type = NULL, ...)
}
\arguments{
\item{data}{A \code{data.frame} object.}

\item{type}{The type of chart. Possible values are line, scatter, point, column.}

\item{...}{Aesthetic mappings as \code{x y group color low high}.}
}
\description{
This function is used in \code{hchart.data.frame}.
}
\examples{

highcharter:::get_hc_series_from_df(iris, type = "point", x = Sepal.Width)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hchart.R
\name{hchart}
\alias{hchart}
\title{Create a highchart object from a particular data type}
\usage{
hchart(object, ...)
}
\arguments{
\item{object}{A R object.}

\item{...}{Additional arguments for the data series
(\url{https://api.highcharts.com/highcharts/series}).}
}
\description{
\code{hchart} uses \code{highchart} to draw a particular plot for an
object of a particular class in a single command. This defines the S3
generic that other classes and packages can extend.
}
\details{
Run \code{methods(hchart)} to see what objects are supported.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_annotations}
\alias{hc_annotations}
\title{Annotations options for highcharter objects}
\usage{
hc_annotations(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/annotations}.}
}
\description{
A basic type of an annotation. It allows to add custom labels
or shapes. The items  can be tied to points, axis coordinates
or chart pixel coordinates.
}
\examples{

# Ex 1
highchart() \%>\%
  hc_add_series(
    data = c(29.9, 71.5, 106.4, 129.2, 144.0, 176.0, 135.6, 148.5, 216.4, 194.1, 95.6, 54.4)
  ) \%>\%
  hc_xAxis(
    tickInterval = 0.5,
    gridLineWidth = 1
  ) \%>\%
  hc_annotations(
    list(
      labels =
        list(
          list(
            point = list(x = 3, y = 129.2, xAxis = 0, yAxis = 0),
            text = "x: {x}<br/>y: {y}"
          ),
          list(
            point = list(x = 9, y = 194.1, xAxis = 0, yAxis = 0),
            text = "x: {x}<br/>y: {y}"
          ),
          list(
            point = list(x = 5, y = 100, xAxis = 0),
            text = "x: {x}<br/>y: {point.plotY} px"
          ),
          list(
            point = list(x = 0, y = 0),
            text = "x: {point.plotX} px<br/>y: {point.plotY} px"
          )
        )
    )
  )

# Ex 2
df <- data.frame(
  x = 1:10,
  y = 1:10
)

highchart() \%>\%
  hc_add_series(data = df, hcaes(x = x, y = y), type = "area") \%>\%
  hc_annotations(
    list(
      labels = list(
        list(point = list(x = 5, y = 5, xAxis = 0, yAxis = 0), text = "Middle"),
        list(point = list(x = 1, y = 1, xAxis = 0, yAxis = 0), text = "Start")
      )
    )
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{is.hexcolor}
\alias{is.hexcolor}
\title{Check if a string vector is in hexadecimal color format}
\usage{
is.hexcolor(x)
}
\arguments{
\item{x}{A string vectors}
}
\description{
Check if a string vector is in hexadecimal color format
}
\examples{

x <- c("#f0f0f0", "#FFf", "#99990000", "#00FFFFFF")

is.hexcolor(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{list_parse}
\alias{list_parse}
\alias{list_parse2}
\title{Convert an object to list with identical structure}
\usage{
list_parse(df)

list_parse2(df)
}
\arguments{
\item{df}{A data frame to parse to list}
}
\description{
This functions are similar to \code{rlist::list.parse} but this removes
names. \code{NA}s are removed for compatibility with rjson::toJSON.
}
\examples{

x <- data.frame(a = 1:3, type = c("A", "C", "B"), stringsAsFactors = FALSE)
list_parse(x)
list_parse2(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_caption}
\alias{hc_caption}
\title{Caption options for highcharter objects}
\usage{
hc_caption(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/caption}.}
}
\description{
The chart's caption, which will render below the chart and will be part
of exported charts. The caption can be updated after chart initialization
through the Chart.update or Chart.caption.update methods.
}
\examples{

highchart() \%>\%
  hc_title(text = "Chart with a caption") \%>\%
  hc_subtitle(text = "This is the subtitle") \%>\%
  hc_xAxis(categories = c("Apples", "Pears", "Banana", "Orange")) \%>\%
  hc_add_series(
    data = c(1, 4, 3, 5),
    type = "column",
    name = "Fruits"
  ) \%>\%
  hc_caption(
    text = "<b>The caption renders in the bottom, and is part of the exported
    chart.</b><br><em>Lorem ipsum dolor sit amet, consectetur adipiscing elit,
    sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim
    ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip
    ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate
    velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat
    cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est
    laborum.</em>'"
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_zAxis}
\alias{hc_zAxis}
\title{Zaxis options for highcharter objects}
\usage{
hc_zAxis(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/zAxis}.}
}
\description{
The Z axis or depth axis for 3D plots.
See the Axis class for programmatic
access to the axis.
}
\examples{

df <- data.frame(
  x = sample(1:5),
  y = sample(1:5),
  z = sample(1:5)
)

# Note the 3d requiere highchart2() due have the 3d module
highchart2() \%>\%
  hc_add_series(data = df, "scatter3d", hcaes(x = x, y = y, z = z)) \%>\%
  hc_chart(
    type = "scatter3d",
    options3d = list(
      enabled = TRUE,
      alpha = 20,
      beta = 30,
      depth = 200,
      viewDistance = 5,
      frame = list(
        bottom = list(
          size = 1,
          color = "rgba(0,0,0,0.05)"
        )
      )
    )
  ) \%>\%
  hc_zAxis(
    title = list(text = "Z axis is here"),
    startOnTick = FALSE,
    tickInterval = 2,
    tickLength = 4,
    tickWidth = 1,
    gridLineColor = "red",
    gridLineDashStyle = "dot"
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{citytemp_long}
\alias{citytemp_long}
\title{City temperatures from a year in long format}
\format{
A \code{data frame} with 36 observations and 3 variables.
}
\usage{
citytemp_long
}
\description{
This data comes from the \url{https://www.highcharts.com/} examples.
}
\section{Variables}{


\itemize{

\item \code{month}: The months.
\item \code{citiy}: City.
\item \code{temp}: Temperatures.
}
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hchart-shorcuts.R
\name{hciconarray}
\alias{hciconarray}
\title{Shortcut to make icon arrays charts}
\usage{
hciconarray(labels, counts, rows = NULL, icons = NULL, size = 4, ...)
}
\arguments{
\item{labels}{A character vector}

\item{counts}{A integer vector}

\item{rows}{A integer to set}

\item{icons}{A character vector same length (o length 1) as labels}

\item{size}{Font size}

\item{...}{Additional arguments for the data series \url{https://api.highcharts.com/highcharts/series}.}
}
\description{
Shortcut to make icon arrays charts
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_labels}
\alias{hc_labels}
\title{Labels options for highcharter objects}
\usage{
hc_labels(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in the Highcharts labels API.}
}
\description{
HTML labels that can be positioned anywhere in the chart area.
This option is deprecated since v7.1.2. Instead, use
annotations that support labels.
}
\examples{

highchart() \%>\%
  hc_add_series(data = sample(1:12)) \%>\%
  hc_labels(
    items = list(
      list(
        html = "<p>Some <b>important</b><br>text</p>",
        style = list(
          left = "150\%",
          top = "150\%"
        )
      )
    )
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_tooltip}
\alias{hc_tooltip}
\title{Tooltip options for highcharter objects}
\usage{
hc_tooltip(hc, ..., sort = FALSE, table = FALSE)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/tooltip}.}

\item{sort}{Logical value to implement sort according \code{this.point}
\url{https://stackoverflow.com/a/16954666/829971}.}

\item{table}{Logical value to implement table in tooltip:
\url{https://stackoverflow.com/a/22327749/829971}.}
}
\description{
Options for the tooltip that appears when the user hovers over a
series or point.
}
\examples{

highchart() \%>\%
  hc_add_series(data = sample(1:12)) \%>\%
  hc_add_series(data = sample(1:12) + 10) \%>\%
  hc_tooltip(
    crosshairs = TRUE,
    borderWidth = 5,
    sort = TRUE,
    table = TRUE
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_loading}
\alias{hc_loading}
\title{Loading options for highcharter objects}
\usage{
hc_loading(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/loading}.}
}
\description{
The loading options control the appearance of the loading screen
that covers the plot area on chart operations. This screen only
appears after an explicit call to chart.showLoading(). It is a
utility for developers to communicate to the end user that something
is going on, for example while retrieving new data via an XHR connection.
The "Loading..." text itself is not part of this configuration
object, but part of the lang object.
}
\examples{

highcharts_demo() \%>\%
  hc_loading(
    hideDuration = 1000,
    showDuration = 1000
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{hc_add_series.lm}
\alias{hc_add_series.lm}
\alias{hc_add_series.loess}
\title{hc_add_series for lm and loess objects}
\usage{
\method{hc_add_series}{lm}(
  hc,
  data,
  type = "line",
  color = "#5F83EE",
  fillOpacity = 0.1,
  ...
)

\method{hc_add_series}{loess}(
  hc,
  data,
  type = "line",
  color = "#5F83EE",
  fillOpacity = 0.1,
  ...
)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{data}{A \code{lm} or \code{loess} object.}

\item{type}{The type of the series: line, spline.}

\item{color}{A stringr color.}

\item{fillOpacity}{fillOpacity to the confidence interval.}

\item{...}{Arguments defined in
\url{https://api.highcharts.com/highcharts/chart}.}
}
\description{
hc_add_series for lm and loess objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-superheroes.R
\name{hc_theme_superheroes}
\alias{hc_theme_superheroes}
\title{Superheroes theme for highcharts}
\usage{
hc_theme_superheroes(...)
}
\arguments{
\item{...}{A named parameters to modify the theme.}
}
\description{
The superheroes theme is inspired by \url{https://public.tableau.com/app/profile/ryansmith#!/vizhome/HeroesofNewYork/SuperheroesinNewYork}
}
\examples{

highcharts_demo() \%>\%
  hc_add_theme(hc_theme_superheroes())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-538.R, R/theme-sparkline.R,
%   R/theme-tufte.R
\name{hc_theme_538}
\alias{hc_theme_538}
\alias{hc_theme_sparkline_vb}
\alias{hc_theme_tufte2}
\title{Theme collection for highcharts}
\usage{
hc_theme_538(...)

hc_theme_sparkline_vb(...)

hc_theme_tufte2(...)
}
\arguments{
\item{...}{A named parameters to modify the theme.}
}
\description{
Highcharts is very flexible so you can modify every element of the chart.
There are some exiting themes so you can apply style to charts with few
lines of code.
}
\examples{

highcharts_demo() \%>\%
  hc_add_theme(hc_theme_538())

highcharts_demo() \%>\%
  hc_add_theme(hc_theme_sparkline_vb())

highchart() \%>\%
  hc_chart(type = "column") \%>\%
  hc_add_series(data = round(1 + abs(rnorm(12)), 2), showInLegend = FALSE) \%>\%
  hc_xAxis(categories = month.abb) \%>\%
  hc_add_theme(hc_theme_tufte2())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api-add.R
\name{hc_add_series.xts}
\alias{hc_add_series.xts}
\alias{hc_add_series.ohlc}
\title{hc_add_series for xts objects}
\usage{
\method{hc_add_series}{xts}(hc, data, ...)

\method{hc_add_series}{ohlc}(hc, data, type = "candlestick", ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{data}{A \code{xts} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.}

\item{type}{The way to show the \code{xts} object. Can be 'candlestick' or 'ohlc'.}
}
\description{
hc_add_series for xts objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{vaccines}
\alias{vaccines}
\title{Vaccines}
\format{
A \code{data frame} with 3,876 observations and 3 variables.
}
\usage{
vaccines
}
\description{
The number of infected people by Measles, measured over 70-some years and across
all 50 states. From the WSJ analysis: \url{http://graphics.wsj.com/infectious-diseases-and-vaccines/}
}
\section{Variables}{


\itemize{

\item \code{year}: Year
\item \code{state}: Name of the state
\item \code{count}: Number of cases per 100,000 people. If the value is NA the count was 0.

}
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-helpers.R
\name{data_to_boxplot}
\alias{data_to_boxplot}
\title{Helper to transform data frame for boxplot highcharts format}
\usage{
data_to_boxplot(
  data,
  variable,
  group_var = NULL,
  group_var2 = NULL,
  add_outliers = FALSE,
  ...
)
}
\arguments{
\item{data}{The data frame containing variables.}

\item{variable}{The variable to calculate the box plot data.}

\item{group_var}{A variable to split calculation}

\item{group_var2}{A second variable to create separate series.}

\item{add_outliers}{A logical value indicating if outliers series should
be calculated. Default to \code{FALSE}.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/plotOptions.series}.}
}
\description{
Helper to transform data frame for boxplot highcharts format
}
\examples{

data(pokemon)

dat <- data_to_boxplot(pokemon, height)

highchart() \%>\%
  hc_xAxis(type = "category") \%>\%
  hc_add_series_list(dat)

dat <- data_to_boxplot(pokemon, height, type_1, name = "height in meters")

highchart() \%>\%
  hc_xAxis(type = "category") \%>\%
  hc_add_series_list(dat)
\dontrun{

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/highcharts-api.R
\name{hc_pane}
\alias{hc_pane}
\title{Pane options for highcharter objects}
\usage{
hc_pane(hc, ...)
}
\arguments{
\item{hc}{A \code{highchart} \code{htmlwidget} object.}

\item{...}{Arguments defined in \url{https://api.highcharts.com/highcharts/pane}.}
}
\description{
The pane serves as a container for axes and backgrounds for circular
gauges and polar charts.
}
\examples{

highchart() \%>\%
  hc_chart(
    type = "gauge",
    plotBackgroundColor = NULL,
    plotBackgroundImage = NULL,
    plotBorderWidth = 0,
    plotShadow = FALSE
  ) \%>\%
  hc_title(
    text = "Speedometer"
  ) \%>\%
  hc_pane(
    startAngle = -150,
    endAngle = 150,
    background = list(list(
      backgroundColor = list(
        linearGradient = list(x1 = 0, y1 = 0, x2 = 0, y2 = 1),
        stops = list(
          list(0, "#FFF"),
          list(1, "#333")
        )
      ),
      borderWidth = 0,
      outerRadius = "109\%"
    ), list(
      backgroundColor = list(
        linearGradient = list(x1 = 0, y1 = 0, x2 = 0, y2 = 1),
        stops = list(
          list(0, "#333"),
          list(1, "#FFF")
        )
      ),
      borderWidth = 1,
      outerRadius = "107\%"
    ), list(
      # default background
    ), list(
      backgroundColor = "#DDD",
      borderWidth = 0,
      outerRadius = "105\%",
      innerRadius = "103\%"
    ))
  ) \%>\%
  hc_add_series(
    data = list(80), name = "speed", tooltip = list(valueSuffix = " km/h")
  ) \%>\%
  hc_yAxis(
    min = 0,
    max = 200,
    minorTickInterval = "auto",
    minorTickWidth = 1,
    minorTickLength = 10,
    minorTickPosition = "inside",
    minorTickColor = "#666",
    tickPixelInterval = 30,
    tickWidth = 2,
    tickPosition = "inside",
    tickLength = 10,
    tickColor = "#666",
    labels = list(
      step = 2,
      rotation = "auto"
    ),
    title = list(
      text = "km/h"
    ),
    plotBands = list(
      list(from = 0, to = 120, color = "#55BF3B"),
      list(from = 120, to = 160, color = "#DDDF0D"),
      list(from = 160, to = 200, color = "#DF5353")
    )
  )
}
